@@ -94,6 +94,8 @@ static gboolean gst_av1_dec_get_valid_format (GstAV1Dec * dec,
 
 #define gst_av1_dec_parent_class parent_class
 G_DEFINE_TYPE (GstAV1Dec, gst_av1_dec, GST_TYPE_VIDEO_DECODER);
+GST_ELEMENT_REGISTER_DEFINE (av1dec, "av1dec", GST_RANK_SECONDARY,
+    GST_TYPE_AV1_DEC);
 
 static void
 gst_av1_dec_class_init (GstAV1DecClass * klass)
