@@ -25,6 +25,8 @@
 #include <gst/base/base.h>
 #include <gst/video/video.h>
 
+#include "ccutils.h"
+
 G_BEGIN_DECLS
 #define GST_TYPE_CCCOMBINER \
   (gst_cc_combiner_get_type())
@@ -45,13 +47,27 @@ struct _GstCCCombiner
   GstAggregator parent;
 
   gint video_fps_n, video_fps_d;
+  gboolean progressive;
   GstClockTime previous_video_running_time_end;
   GstClockTime current_video_running_time;
   GstClockTime current_video_running_time_end;
   GstBuffer *current_video_buffer;
 
   GArray *current_frame_captions;
-  GstVideoCaptionType current_caption_type;
+  GstVideoCaptionType caption_type;
+
+  gboolean prop_schedule;
+  guint prop_max_scheduled;
+  gboolean prop_output_padding;
+
+  gboolean schedule;
+  guint max_scheduled;
+  gboolean output_padding;
+  guint current_scheduled;
+
+  CCBuffer *cc_buffer;
+  guint16 cdp_hdr_sequence_cntr;
+  const struct cdp_fps_entry *cdp_fps_entry;
 };
 
 struct _GstCCCombinerClass
@@ -61,5 +77,7 @@ struct _GstCCCombinerClass
 
 GType gst_cc_combiner_get_type (void);
 
+GST_ELEMENT_REGISTER_DECLARE (cccombiner);
+
 G_END_DECLS
 #endif /* __GST_CCCOMBINER_H__ */
