@@ -116,7 +116,7 @@
 #include <config.h>
 #endif
 
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 
 #include "gstcurlelements.h"
 #include "gstcurlhttpsrc.h"
