@@ -23,19 +23,21 @@
  * @title: dfbvideosink
  *
  * DfbVideoSink renders video frames using the
- * <ulink url="http://www.directfb.org/">DirectFB</ulink> library.
+ * [DirectFB](http://www.directfb.org/) library.
  * Rendering can happen in two different modes :
  *
  * * Standalone: this mode will take complete control of the monitor forcing
- *   <ulink url="http://www.directfb.org/">DirectFB</ulink> to fullscreen layout.
+ *   DirectFB to fullscreen layout.
+ *
  *   This is convenient to test using the  gst-launch-1.0 command line tool or
  *   other simple applications. It is possible to interrupt playback while
  *   being in this mode by pressing the Escape key.
  *   This mode handles navigation events for every input device supported by
- *   the <ulink url="http://www.directfb.org/">DirectFB</ulink> library, it will
- *   look for available video modes in the fb.modes file and try to switch
- *   the framebuffer video mode to the most suitable one. Depending on
- *   hardware acceleration capabilities the element will handle scaling or not.
+ *   the DirectFB library, it will look for available video modes in the fb.modes
+ *   file and try to switch the framebuffer video mode to the most suitable one.
+ *   Depending on hardware acceleration capabilities the element will handle
+ *   scaling or not.
+ *
  *   If no acceleration is available it will do clipping or centering of the
  *   video frames respecting the original aspect ratio.
  *
@@ -43,13 +45,14 @@
  *   #GstDfbVideoSink:surface provided by the
  *   application developer. This is a more advanced usage of the element and
  *   it is required to integrate video playback in existing
- *   <ulink url="http://www.directfb.org/">DirectFB</ulink> applications.
+ *   DirectFB applications.
+ *
  *   When using this mode the element just renders to the
  *   #GstDfbVideoSink:surface provided by the
  *   application, that means it won't handle navigation events and won't resize
  *   the #GstDfbVideoSink:surface to fit video
  *   frames geometry. Application has to implement the necessary code to grab
- *   informations about the negotiated geometry and resize there
+ *   information about the negotiated geometry and resize there
  *   #GstDfbVideoSink:surface accordingly.
  *
  * For both modes the element implements a buffer pool allocation system to
@@ -144,7 +147,7 @@ gst_dfbvideosink_layer_mode_get_type (void)
 GType
 gst_meta_dfbsurface_api_get_type (void)
 {
-  static volatile GType type;
+  static GType type;
   static const gchar *tags[] = { "memory", NULL };
 
   if (g_once_init_enter (&type)) {
@@ -509,8 +512,12 @@ G_DEFINE_TYPE_WITH_CODE (GstDfbVideoSink, gst_dfbvideosink, GST_TYPE_VIDEO_SINK,
     G_IMPLEMENT_INTERFACE (GST_TYPE_NAVIGATION,
         gst_dfbvideosink_navigation_init);
     G_IMPLEMENT_INTERFACE (GST_TYPE_COLOR_BALANCE,
-        gst_dfbvideosink_colorbalance_init));
-
+        gst_dfbvideosink_colorbalance_init);
+    GST_DEBUG_CATEGORY_INIT (dfbvideosink_debug, "dfbvideosink", 0,
+        "DirectFB video sink element");
+    );
+GST_ELEMENT_REGISTER_DEFINE (dfbvideosink, "dfbvideosink", GST_RANK_MARGINAL,
+    GST_TYPE_DFBVIDEOSINK);
 #ifndef GST_DISABLE_GST_DEBUG
 static const char *
 gst_dfbvideosink_get_format_name (DFBSurfacePixelFormat format)
@@ -598,8 +605,10 @@ gst_dfbvideosink_event_thread (GstDfbVideoSink * dfbvideosink)
             default:
               GST_DEBUG_OBJECT (dfbvideosink, "key press event %c !",
                   event.input.key_symbol);
-              gst_navigation_send_key_event (GST_NAVIGATION (dfbvideosink),
-                  "key-press", "prout");
+              gst_dfbvideosink_navigation_send_event
+                  (GST_NAVIGATION (dfbvideosink),
+                  gst_navigation_event_new_key_press ("prout",
+                      GST_NAVIGATION_MODIFIER_NONE));
           }
         } else if (event.input.type == DIET_BUTTONPRESS) {
           gint x, y;
@@ -609,8 +618,10 @@ gst_dfbvideosink_event_thread (GstDfbVideoSink * dfbvideosink)
           GST_DEBUG_OBJECT (dfbvideosink, "button %d pressed at %dx%d",
               event.input.button, x, y);
 
-          gst_navigation_send_mouse_event (GST_NAVIGATION (dfbvideosink),
-              "mouse-button-press", event.input.button, x, y);
+          gst_dfbvideosink_navigation_send_event
+              (GST_NAVIGATION (dfbvideosink),
+              gst_navigation_event_new_mouse_button_press (event.input.button,
+                  x, y, GST_NAVIGATION_MODIFIER_NONE));
         } else if (event.input.type == DIET_BUTTONRELEASE) {
           gint x, y;
 
@@ -619,14 +630,18 @@ gst_dfbvideosink_event_thread (GstDfbVideoSink * dfbvideosink)
           GST_DEBUG_OBJECT (dfbvideosink, "button %d released at %dx%d",
               event.input.button, x, y);
 
-          gst_navigation_send_mouse_event (GST_NAVIGATION (dfbvideosink),
-              "mouse-button-release", event.input.button, x, y);
+          gst_dfbvideosink_navigation_send_event
+              (GST_NAVIGATION (dfbvideosink),
+              gst_navigation_event_new_mouse_button_release (event.input.button,
+                  x, y, GST_NAVIGATION_MODIFIER_NONE));
         } else if (event.input.type == DIET_AXISMOTION) {
           gint x, y;
 
           dfbvideosink->layer->GetCursorPosition (dfbvideosink->layer, &x, &y);
-          gst_navigation_send_mouse_event (GST_NAVIGATION (dfbvideosink),
-              "mouse-move", 0, x, y);
+          gst_dfbvideosink_navigation_send_event
+              (GST_NAVIGATION (dfbvideosink),
+              gst_navigation_event_new_mouse_move (x, y,
+                  GST_NAVIGATION_MODIFIER_NONE));
         } else {
           GST_WARNING_OBJECT (dfbvideosink, "unhandled event type %d",
               event.input.type);
@@ -1976,10 +1991,9 @@ beach:
 
 static void
 gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
-    GstStructure * structure)
+    GstEvent * event)
 {
   GstDfbVideoSink *dfbvideosink = GST_DFBVIDEOSINK (navigation);
-  GstEvent *event;
   GstVideoRectangle dst = { 0, };
   GstVideoRectangle src = { 0, };
   GstVideoRectangle result;
@@ -1992,13 +2006,14 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
   dst.h = dfbvideosink->out_height;
   gst_video_sink_center_rect (src, dst, &result, dfbvideosink->hw_scaling);
 
-  event = gst_event_new_navigation (structure);
+  event = gst_event_make_writable (event);
 
   /* Our coordinates can be wrong here if we centered the video */
 
   /* Converting pointer coordinates to the non scaled geometry */
-  if (gst_structure_get_double (structure, "pointer_x", &old_x)) {
+  if (gst_navigation_event_get_coordinates (event, &old_x, &old_y)) {
     x = old_x;
+    y = old_y;
 
     if (x >= result.x && x <= (result.x + result.w)) {
       x -= result.x;
@@ -2007,13 +2022,6 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
     } else {
       x = 0;
     }
-    GST_DEBUG_OBJECT (dfbvideosink, "translated navigation event x "
-        "coordinate from %f to %f", old_x, x);
-    gst_structure_set (structure, "pointer_x", G_TYPE_DOUBLE, x, NULL);
-  }
-  if (gst_structure_get_double (structure, "pointer_y", &old_y)) {
-    y = old_y;
-
     if (y >= result.y && y <= (result.y + result.h)) {
       y -= result.y;
       y *= dfbvideosink->video_height;
@@ -2021,14 +2029,17 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
     } else {
       y = 0;
     }
+
+    GST_DEBUG_OBJECT (dfbvideosink, "translated navigation event x "
+        "coordinate from %f to %f", old_x, x);
     GST_DEBUG_OBJECT (dfbvideosink, "translated navigation event y "
         "coordinate from %fd to %fd", old_y, y);
-    gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
+    gst_navigation_event_set_coordinates (event, x, y);
   }
 
   pad = gst_pad_get_peer (GST_VIDEO_SINK_PAD (dfbvideosink));
 
-  if (GST_IS_PAD (pad) && GST_IS_EVENT (event)) {
+  if (GST_IS_PAD (pad)) {
     if (!gst_pad_send_event (pad, gst_event_ref (event))) {
       /* If upstream didn't handle the event we'll post a message with it
        * for the application in case it wants to do something with it */
@@ -2044,7 +2055,7 @@ gst_dfbvideosink_navigation_send_event (GstNavigation * navigation,
 static void
 gst_dfbvideosink_navigation_init (GstNavigationInterface * iface)
 {
-  iface->send_event = gst_dfbvideosink_navigation_send_event;
+  iface->send_event_simple = gst_dfbvideosink_navigation_send_event;
 }
 
 static void
@@ -2435,6 +2446,8 @@ gst_dfbvideosink_class_init (GstDfbVideoSinkClass * klass)
           gst_dfbvideosink_layer_mode_get_type (), DEFAULT_LAYER_MODE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  gst_type_mark_as_plugin_api (gst_dfbvideosink_layer_mode_get_type (), 0);
+
   gst_element_class_set_static_metadata (gstelement_class,
       "DirectFB video sink", "Sink/Video", "A DirectFB based videosink",
       "Julien Moutte <julien@moutte.net>");
@@ -2455,18 +2468,11 @@ gst_dfbvideosink_class_init (GstDfbVideoSinkClass * klass)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "dfbvideosink", GST_RANK_MARGINAL,
-          GST_TYPE_DFBVIDEOSINK))
-    return FALSE;
-
-  GST_DEBUG_CATEGORY_INIT (dfbvideosink_debug, "dfbvideosink", 0,
-      "DirectFB video sink element");
-
-  return TRUE;
+  return GST_ELEMENT_REGISTER (dfbvideosink, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    dfbvideosink,
+    directfb,
     "DirectFB video output plugin",
     plugin_init, VERSION, GST_LICENSE, GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
