@@ -58,7 +58,7 @@ struct _GstDtlsAgentPrivate
   GstDtlsCertificate *certificate;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GstDtlsAgent, gst_dtls_agent, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (GstDtlsAgent, gst_dtls_agent, GST_TYPE_OBJECT);
 
 static void gst_dtls_agent_finalize (GObject * gobject);
 static void gst_dtls_agent_set_property (GObject *, guint prop_id,
@@ -126,7 +126,7 @@ _gst_dtls_init_openssl (void)
     SSL_load_error_strings ();
     ERR_load_BIO_strings ();
 
-    {
+    if (!CRYPTO_get_locking_callback ()) {
       gint i;
       gint num_locks;
       num_locks = CRYPTO_num_locks ();
@@ -214,6 +214,8 @@ gst_dtls_agent_finalize (GObject * gobject)
   SSL_CTX_free (priv->ssl_context);
   priv->ssl_context = NULL;
 
+  g_clear_object (&priv->certificate);
+
   GST_DEBUG_OBJECT (gobject, "finalized");
 
   G_OBJECT_CLASS (gst_dtls_agent_parent_class)->finalize (gobject);
