@@ -21,6 +21,7 @@
 #include "config.h"
 #endif
 
+#include "gstfdkaac.h"
 #include "gstfdkaacenc.h"
 
 #include <gst/pbutils/pbutils.h>
@@ -29,7 +30,6 @@
 
 /* TODO:
  * - Add support for other AOT / profiles
- * - Expose more properties, e.g. afterburner and vbr
  * - Signal encoder delay
  * - LOAS / LATM support
  */
@@ -37,10 +37,17 @@
 enum
 {
   PROP_0,
-  PROP_BITRATE
+  PROP_AFTERBURNER,
+  PROP_BITRATE,
+  PROP_PEAK_BITRATE,
+  PROP_RATE_CONTROL,
+  PROP_VBR_PRESET,
 };
 
 #define DEFAULT_BITRATE (0)
+#define DEFAULT_PEAK_BITRATE (0)
+#define DEFAULT_RATE_CONTROL (GST_FDK_AAC_RATE_CONTROL_CONSTANT_BITRATE)
+#define DEFAULT_VBR_PRESET (GST_FDK_AAC_VBR_PRESET_MEDIUM)
 
 #define SAMPLE_RATES " 8000, " \
                     "11025, " \
@@ -55,121 +62,6 @@ enum
                     "88200, " \
                     "96000"
 
-static const struct
-{
-  gint channels;
-  CHANNEL_MODE mode;
-  GstAudioChannelPosition positions[8];
-} channel_layouts[] = {
-  {
-    /* MPEG 1: Mono */
-    1, MODE_1, {
-  GST_AUDIO_CHANNEL_POSITION_MONO}}, {
-    /* MPEG 2: Stereo */
-    2, MODE_2, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT}}, {
-    /* MPEG 3: Stereo + Center */
-    3, MODE_1_2, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT}}, {
-    /* MPEG 4: Stereo + Center + Rear center */
-    4, MODE_1_2_1, {
-  GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_CENTER}}, {
-    /* MPEG 5: 5.0 Surround */
-    5, MODE_1_2_2, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of REAR is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT}}, {
-    /* MPEG 6: 5.1 Surround */
-    6, MODE_1_2_2_1, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of REAR is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_LFE1}}, {
-    /* MPEG 7: SDDS for cinema */
-    8, MODE_1_2_2_2_1, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of REAR is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_LFE1}}
-#ifdef HAVE_FDK_AAC_2_0_0
-  , {
-    /* MPEG 11: 6.1 Surround */
-    7, MODE_6_1, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of REAR is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_LFE1}}, {
-    /* MPEG 12: 7.1 Surround */
-    8, MODE_7_1_BACK, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of SIDE is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_LFE1}}, {
-    /* MPEG 14: 5.1.2 Surround */
-    8, MODE_7_1_TOP_FRONT, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of REAR is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_LFE1,
-          GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT}}
-#endif
-#ifdef HAVE_FDK_AAC_0_1_4
-  , {
-    /* Non-standard PCE clone of mode 12 */
-    8, MODE_7_1_REAR_SURROUND, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of SIDE is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_LFE1}}, {
-    /* Non-standard PCE clone of mode 7 */
-    8, MODE_7_1_FRONT_CENTER, {
-      GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
-          /* FIXME: SURROUND instead of REAR is more to spec */
-  GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-          GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-          GST_AUDIO_CHANNEL_POSITION_LFE1}}
-#endif
-};
-
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -188,7 +80,8 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
         "rate = (int) { " SAMPLE_RATES " }, "
         "channels = (int) {1, 2, 3, 4, 5, 6, 8}, "
         "stream-format = (string) { adts, adif, raw }, "
-        "base-profile = (string) lc, " "framed = (boolean) true")
+        "profile = (string) { lc, he-aac-v1, he-aac-v2, ld }, "
+        "framed = (boolean) true")
     );
 
 GST_DEBUG_CATEGORY_STATIC (gst_fdkaacenc_debug);
@@ -206,8 +99,51 @@ static GstFlowReturn gst_fdkaacenc_handle_frame (GstAudioEncoder * enc,
     GstBuffer * in_buf);
 static GstCaps *gst_fdkaacenc_get_caps (GstAudioEncoder * enc,
     GstCaps * filter);
+static void gst_fdkaacenc_flush (GstAudioEncoder * enc);
 
 G_DEFINE_TYPE (GstFdkAacEnc, gst_fdkaacenc, GST_TYPE_AUDIO_ENCODER);
+GST_ELEMENT_REGISTER_DEFINE (fdkaacenc, "fdkaacenc", GST_RANK_PRIMARY,
+    GST_TYPE_FDKAACENC);
+
+#define GST_FDK_AAC_VBR_PRESET (gst_fdk_aac_vbr_preset_get_type ())
+static GType
+gst_fdk_aac_vbr_preset_get_type (void)
+{
+  static GType fdk_aac_vbr_preset_type = 0;
+  static const GEnumValue vbr_preset_types[] = {
+    {GST_FDK_AAC_VBR_PRESET_VERY_LOW, "Very Low Variable Bitrate", "very-low"},
+    {GST_FDK_AAC_VBR_PRESET_LOW, "Low Variable Bitrate", "low"},
+    {GST_FDK_AAC_VBR_PRESET_MEDIUM, "Medium Variable Bitrate", "medium"},
+    {GST_FDK_AAC_VBR_PRESET_HIGH, "High Variable Bitrate", "high"},
+    {GST_FDK_AAC_VBR_PRESET_VERY_HIGH, "Very High Variable Bitrate",
+        "very-high"},
+    {0, NULL, NULL}
+  };
+
+  if (!fdk_aac_vbr_preset_type)
+    fdk_aac_vbr_preset_type =
+        g_enum_register_static ("GstFdkAacVbrPreset", vbr_preset_types);
+
+  return fdk_aac_vbr_preset_type;
+}
+
+#define GST_FDK_AAC_RATE_CONTROL (gst_fdk_aac_rate_control_get_type ())
+static GType
+gst_fdk_aac_rate_control_get_type (void)
+{
+  static GType fdk_aac_rate_control_type = 0;
+  static const GEnumValue rate_control_types[] = {
+    {GST_FDK_AAC_RATE_CONTROL_CONSTANT_BITRATE, "Constant Bitrate", "cbr"},
+    {GST_FDK_AAC_RATE_CONTROL_VARIABLE_BITRATE, "Variable Bitrate", "vbr"},
+    {0, NULL, NULL}
+  };
+
+  if (!fdk_aac_rate_control_type)
+    fdk_aac_rate_control_type =
+        g_enum_register_static ("GstFdkAacRateControl", rate_control_types);
+
+  return fdk_aac_rate_control_type;
+}
 
 static void
 gst_fdkaacenc_set_property (GObject * object, guint prop_id,
@@ -219,6 +155,18 @@ gst_fdkaacenc_set_property (GObject * object, guint prop_id,
     case PROP_BITRATE:
       self->bitrate = g_value_get_int (value);
       break;
+    case PROP_AFTERBURNER:
+      self->afterburner = g_value_get_boolean (value);
+      break;
+    case PROP_PEAK_BITRATE:
+      self->peak_bitrate = g_value_get_int (value);
+      break;
+    case PROP_RATE_CONTROL:
+      self->rate_control = g_value_get_enum (value);
+      break;
+    case PROP_VBR_PRESET:
+      self->vbr_preset = g_value_get_enum (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -236,6 +184,18 @@ gst_fdkaacenc_get_property (GObject * object, guint prop_id,
     case PROP_BITRATE:
       g_value_set_int (value, self->bitrate);
       break;
+    case PROP_AFTERBURNER:
+      g_value_set_boolean (value, self->afterburner);
+      break;
+    case PROP_PEAK_BITRATE:
+      g_value_set_int (value, self->peak_bitrate);
+      break;
+    case PROP_RATE_CONTROL:
+      g_value_set_enum (value, self->rate_control);
+      break;
+    case PROP_VBR_PRESET:
+      g_value_set_enum (value, self->vbr_preset);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -260,35 +220,57 @@ gst_fdkaacenc_stop (GstAudioEncoder * enc)
 
   GST_DEBUG_OBJECT (self, "stop");
 
-  if (self->enc)
+  if (self->enc) {
     aacEncClose (&self->enc);
+    self->enc = NULL;
+  }
 
+  self->is_drained = TRUE;
   return TRUE;
 }
 
 static GstCaps *
 gst_fdkaacenc_get_caps (GstAudioEncoder * enc, GstCaps * filter)
 {
-  GstCaps *res, *caps;
-  gint i;
+  const GstFdkAacChannelLayout *layout;
+  GstCaps *res, *caps, *allowed_caps;
+  gboolean allow_mono = TRUE;
+
+  allowed_caps = gst_pad_get_allowed_caps (GST_AUDIO_ENCODER_SRC_PAD (enc));
+  GST_DEBUG_OBJECT (enc, "allowed caps %" GST_PTR_FORMAT, allowed_caps);
+
+  /* We need at least 2 channels if Parametric Stereo is in use. */
+  if (allowed_caps && gst_caps_get_size (allowed_caps) > 0) {
+    GstStructure *s = gst_caps_get_structure (allowed_caps, 0);
+    const gchar *profile = NULL;
+
+    if ((profile = gst_structure_get_string (s, "profile"))
+        && strcmp (profile, "he-aac-v2") == 0) {
+      allow_mono = FALSE;
+    }
+  }
+  gst_clear_caps (&allowed_caps);
 
   caps = gst_caps_new_empty ();
 
-  for (i = 0; i < G_N_ELEMENTS (channel_layouts); i++) {
-    guint64 channel_mask;
-    GstCaps *tmp =
-        gst_caps_make_writable (gst_pad_get_pad_template_caps
+  for (layout = channel_layouts; layout->channels; layout++) {
+    GstCaps *tmp;
+    gint channels = layout->channels;
+
+    if (channels == 1 && !allow_mono)
+      continue;
+
+    tmp = gst_caps_make_writable (gst_pad_get_pad_template_caps
         (GST_AUDIO_ENCODER_SINK_PAD (enc)));
 
-    if (channel_layouts[i].channels == 1) {
-      gst_caps_set_simple (tmp, "channels", G_TYPE_INT,
-          channel_layouts[i].channels, NULL);
+    if (channels == 1) {
+      gst_caps_set_simple (tmp, "channels", G_TYPE_INT, channels, NULL);
     } else {
-      gst_audio_channel_positions_to_mask (channel_layouts[i].positions,
-          channel_layouts[i].channels, FALSE, &channel_mask);
-      gst_caps_set_simple (tmp, "channels", G_TYPE_INT,
-          channel_layouts[i].channels, "channel-mask", GST_TYPE_BITMASK,
-          channel_mask, NULL);
+      guint64 channel_mask;
+      gst_audio_channel_positions_to_mask (layout->positions, channels, FALSE,
+          &channel_mask);
+      gst_caps_set_simple (tmp, "channels", G_TYPE_INT, channels,
+          "channel-mask", GST_TYPE_BITMASK, channel_mask, NULL);
     }
 
     gst_caps_append (caps, tmp);
@@ -308,16 +290,20 @@ gst_fdkaacenc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
   GstCaps *allowed_caps;
   GstCaps *src_caps;
   AACENC_ERROR err;
-  gint transmux = 0, aot = AOT_AAC_LC;
+  gint transmux = 0;
   gint mpegversion = 4;
+  gint aot = AOT_AAC_LC;
+  const gchar *profile_str = "lc";
   CHANNEL_MODE channel_mode;
   AACENC_InfoStruct enc_info = { 0 };
-  gint bitrate;
+  gint bitrate, signaling_mode;
+  guint bitrate_mode;
 
-  if (self->enc) {
+  if (self->enc && !self->is_drained) {
     /* drain */
     gst_fdkaacenc_handle_frame (enc, NULL);
     aacEncClose (&self->enc);
+    self->is_drained = TRUE;
   }
 
   allowed_caps = gst_pad_get_allowed_caps (GST_AUDIO_ENCODER_SRC_PAD (self));
@@ -341,6 +327,26 @@ gst_fdkaacenc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
       }
     }
 
+    if ((str = gst_structure_get_string (s, "profile"))) {
+      if (strcmp (str, "lc") == 0) {
+        GST_DEBUG_OBJECT (self, "using AAC-LC profile for output");
+        aot = AOT_AAC_LC;
+        profile_str = "lc";
+      } else if (strcmp (str, "he-aac-v1") == 0) {
+        GST_DEBUG_OBJECT (self, "using SBR (HE-AACv1) profile for output");
+        aot = AOT_SBR;
+        profile_str = "he-aac-v1";
+      } else if (strcmp (str, "he-aac-v2") == 0) {
+        GST_DEBUG_OBJECT (self, "using PS (HE-AACv2) profile for output");
+        aot = AOT_PS;
+        profile_str = "he-aac-v2";
+      } else if (strcmp (str, "ld") == 0) {
+        GST_DEBUG_OBJECT (self, "using AAC-LD profile for output");
+        aot = AOT_ER_AAC_LD;
+        profile_str = "ld";
+      }
+    }
+
     gst_structure_get_int (s, "mpegversion", &mpegversion);
   }
   if (allowed_caps)
@@ -348,20 +354,32 @@ gst_fdkaacenc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
 
   err = aacEncOpen (&self->enc, 0, GST_AUDIO_INFO_CHANNELS (info));
   if (err != AACENC_OK) {
-    GST_ERROR_OBJECT (self, "Unable to open encoder: %d\n", err);
+    GST_ERROR_OBJECT (self, "Unable to open encoder: %d", err);
     return FALSE;
   }
 
-  aot = AOT_AAC_LC;
-
   if ((err = aacEncoder_SetParam (self->enc, AACENC_AOT, aot)) != AACENC_OK) {
-    GST_ERROR_OBJECT (self, "Unable to set AOT %d: %d\n", aot, err);
+    GST_ERROR_OBJECT (self, "Unable to set AOT %d: %d", aot, err);
+    return FALSE;
+  }
+
+  /* Use explicit hierarchical signaling (2) with raw output stream-format
+   * and implicit signaling (0) with ADTS/ADIF */
+  if (transmux == 0)
+    signaling_mode = 2;
+  else
+    signaling_mode = 0;
+
+  if ((err = aacEncoder_SetParam (self->enc, AACENC_SIGNALING_MODE,
+              signaling_mode)) != AACENC_OK) {
+    GST_ERROR_OBJECT (self, "Unable to set signaling mode %d: %d",
+        signaling_mode, err);
     return FALSE;
   }
 
   if ((err = aacEncoder_SetParam (self->enc, AACENC_SAMPLERATE,
               GST_AUDIO_INFO_RATE (info))) != AACENC_OK) {
-    GST_ERROR_OBJECT (self, "Unable to set sample rate %d: %d\n",
+    GST_ERROR_OBJECT (self, "Unable to set sample rate %d: %d",
         GST_AUDIO_INFO_RATE (info), err);
     return FALSE;
   }
@@ -371,30 +389,36 @@ gst_fdkaacenc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
     self->need_reorder = FALSE;
     self->aac_positions = NULL;
   } else {
-    guint64 in_channel_mask, out_channel_mask;
-    gint i;
+    gint in_channels = GST_AUDIO_INFO_CHANNELS (info);
+    const GstAudioChannelPosition *in_positions =
+        &GST_AUDIO_INFO_POSITION (info, 0);
+    guint64 in_channel_mask;
+    const GstFdkAacChannelLayout *layout;
 
-    for (i = 0; i < G_N_ELEMENTS (channel_layouts); i++) {
-      if (channel_layouts[i].channels != GST_AUDIO_INFO_CHANNELS (info))
+    gst_audio_channel_positions_to_mask (in_positions, in_channels, FALSE,
+        &in_channel_mask);
+
+    for (layout = channel_layouts; layout->channels; layout++) {
+      gint channels = layout->channels;
+      const GstAudioChannelPosition *positions = layout->positions;
+      guint64 channel_mask;
+
+      if (channels != in_channels)
         continue;
 
-      gst_audio_channel_positions_to_mask (&GST_AUDIO_INFO_POSITION (info, 0),
-          GST_AUDIO_INFO_CHANNELS (info), FALSE, &in_channel_mask);
-      gst_audio_channel_positions_to_mask (channel_layouts[i].positions,
-          channel_layouts[i].channels, FALSE, &out_channel_mask);
-      if (in_channel_mask == out_channel_mask) {
-        channel_mode = channel_layouts[i].mode;
-        self->need_reorder =
-            memcmp (channel_layouts[i].positions,
-            &GST_AUDIO_INFO_POSITION (info, 0),
-            GST_AUDIO_INFO_CHANNELS (info) *
-            sizeof (GstAudioChannelPosition)) != 0;
-        self->aac_positions = channel_layouts[i].positions;
-        break;
-      }
+      gst_audio_channel_positions_to_mask (positions, channels, FALSE,
+          &channel_mask);
+      if (channel_mask != in_channel_mask)
+        continue;
+
+      channel_mode = layout->mode;
+      self->need_reorder = memcmp (positions, in_positions,
+          channels * sizeof *positions) != 0;
+      self->aac_positions = positions;
+      break;
     }
 
-    if (i == G_N_ELEMENTS (channel_layouts)) {
+    if (!layout->channels) {
       GST_ERROR_OBJECT (self, "Couldn't find a valid channel layout");
       return FALSE;
     }
@@ -474,6 +498,44 @@ gst_fdkaacenc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
     return FALSE;
   }
 
+  if (self->rate_control == GST_FDK_AAC_RATE_CONTROL_CONSTANT_BITRATE) {
+    /*
+     * Note that the `bitrate` property is honoured only when using
+     * constant bit rate.
+     */
+    bitrate_mode = 0;           // Constant Bitrate
+  } else {
+    bitrate_mode = self->vbr_preset;
+  }
+
+  if ((err = aacEncoder_SetParam (self->enc, AACENC_BITRATEMODE,
+              bitrate_mode)) != AACENC_OK) {
+    GST_ERROR_OBJECT (self, "Unable to set bitrate mode %d: %d",
+        bitrate_mode, err);
+    return FALSE;
+  }
+
+  if (self->peak_bitrate) {
+    if ((err = aacEncoder_SetParam (self->enc, AACENC_PEAK_BITRATE,
+                self->peak_bitrate)) != AACENC_OK) {
+      GST_ERROR_OBJECT (self, "Unable to set peak bitrate %d: %d",
+          self->peak_bitrate, err);
+      return FALSE;
+    }
+
+    GST_INFO_OBJECT (self, "Setting peak bitrate to %d", self->peak_bitrate);
+  }
+
+  if (self->afterburner) {
+    if ((err =
+            aacEncoder_SetParam (self->enc, AACENC_AFTERBURNER,
+                1)) != AACENC_OK) {
+      GST_ERROR_OBJECT (self, "Could not enable afterburner: %d", err);
+      return FALSE;
+    }
+
+    GST_INFO_OBJECT (self, "Afterburner enabled");
+  }
   if ((err = aacEncEncode (self->enc, NULL, NULL, NULL, NULL)) != AACENC_OK) {
     GST_ERROR_OBJECT (self, "Unable to initialize encoder: %d", err);
     return FALSE;
@@ -500,8 +562,7 @@ gst_fdkaacenc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
   /* raw */
   if (transmux == 0) {
     GstBuffer *codec_data =
-        gst_buffer_new_wrapped (g_memdup (enc_info.confBuf, enc_info.confSize),
-        enc_info.confSize);
+        gst_buffer_new_memdup (enc_info.confBuf, enc_info.confSize);
     gst_caps_set_simple (src_caps, "codec_data", GST_TYPE_BUFFER, codec_data,
         "stream-format", G_TYPE_STRING, "raw", NULL);
     gst_buffer_unref (codec_data);
@@ -518,6 +579,17 @@ gst_fdkaacenc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
   gst_codec_utils_aac_caps_set_level_and_profile (src_caps, enc_info.confBuf,
       enc_info.confSize);
 
+  /* The above only parses the "base" profile, which is always going to be LC.
+   * Set actual profile. */
+  gst_caps_set_simple (src_caps, "profile", G_TYPE_STRING, profile_str, NULL);
+
+  /* An AAC-LC-only decoder will not decode a stream that uses explicit
+   * hierarchical signaling */
+  if (signaling_mode == 2 && aot != AOT_AAC_LC) {
+    gst_structure_remove_field (gst_caps_get_structure (src_caps, 0),
+        "base-profile");
+  }
+
   ret = gst_audio_encoder_set_output_format (enc, src_caps);
   gst_caps_unref (src_caps);
 
@@ -566,6 +638,10 @@ gst_fdkaacenc_handle_frame (GstAudioEncoder * enc, GstBuffer * inbuf)
     in_el_sizes = 0;
     in_desc.numBufs = 0;
   }
+  /* We unset is_drained even if there's no inbuf. Basically this is a
+   * workaround for aacEncEncode always producing 1024 bytes even without any
+   * input, thus messing up with the base class counting */
+  self->is_drained = FALSE;
 
   in_desc.bufferIdentifiers = &in_id;
   in_desc.bufs = (void *) &imap.data;
@@ -626,11 +702,30 @@ out:
   return ret;
 }
 
+static void
+gst_fdkaacenc_flush (GstAudioEncoder * enc)
+{
+  GstFdkAacEnc *self = GST_FDKAACENC (enc);
+  GstAudioInfo *info = gst_audio_encoder_get_audio_info (enc);
+
+  aacEncClose (&self->enc);
+  self->enc = NULL;
+  self->is_drained = TRUE;
+
+  if (GST_AUDIO_INFO_IS_VALID (info))
+    gst_fdkaacenc_set_format (enc, info);
+}
+
 static void
 gst_fdkaacenc_init (GstFdkAacEnc * self)
 {
   self->bitrate = DEFAULT_BITRATE;
   self->enc = NULL;
+  self->is_drained = TRUE;
+  self->afterburner = FALSE;
+  self->peak_bitrate = DEFAULT_PEAK_BITRATE;
+  self->rate_control = DEFAULT_RATE_CONTROL;
+  self->vbr_preset = DEFAULT_VBR_PRESET;
 
   gst_audio_encoder_set_drainable (GST_AUDIO_ENCODER (self), TRUE);
 }
@@ -650,22 +745,79 @@ gst_fdkaacenc_class_init (GstFdkAacEncClass * klass)
   base_class->set_format = GST_DEBUG_FUNCPTR (gst_fdkaacenc_set_format);
   base_class->getcaps = GST_DEBUG_FUNCPTR (gst_fdkaacenc_get_caps);
   base_class->handle_frame = GST_DEBUG_FUNCPTR (gst_fdkaacenc_handle_frame);
+  base_class->flush = GST_DEBUG_FUNCPTR (gst_fdkaacenc_flush);
 
   g_object_class_install_property (object_class, PROP_BITRATE,
       g_param_spec_int ("bitrate",
           "Bitrate",
-          "Target Audio Bitrate (0 = fixed value based on "
-          " sample rate and channel count)",
+          "Target Audio Bitrate. Only applicable if rate-control=cbr. "
+          "(0 = fixed value based on sample rate and channel count)",
           0, G_MAXINT, DEFAULT_BITRATE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstFdkAacEnc:peak-bitrate:
+   *
+   * Peak Bitrate to adjust maximum bits per audio frame.
+   *
+   * Since: 1.22
+   */
+  g_object_class_install_property (object_class, PROP_PEAK_BITRATE,
+      g_param_spec_int ("peak-bitrate",
+          "Peak Bitrate",
+          "Peak Bitrate to adjust maximum bits per audio frame. "
+          "Bitrate is in bits/second. Only applicable if rate-control=vbr. (0 = Not set)",
+          0, G_MAXINT, DEFAULT_PEAK_BITRATE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstFdkAacEnc:afterburner:
+   *
+   * Afterburner - Quality Parameter.
+   *
+   * Since: 1.22
+   */
+  g_object_class_install_property (object_class, PROP_AFTERBURNER,
+      g_param_spec_boolean ("afterburner", "Afterburner - Quality Parameter",
+          "Additional quality control parameter. Can cause workload increase.",
+          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstFdkAacEnc:rate-control:
+   *
+   * Rate Control.
+   *
+   * Since: 1.22
+   */
+  g_object_class_install_property (object_class, PROP_RATE_CONTROL,
+      g_param_spec_enum ("rate-control", "Rate Control",
+          "Whether Constant or Variable Bitrate should be used.",
+          GST_FDK_AAC_RATE_CONTROL, DEFAULT_RATE_CONTROL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstFdkAacEnc:vbr-preset:
+   *
+   * AAC Variable Bitrate configurations.
+   *
+   * Since: 1.22
+   */
+  g_object_class_install_property (object_class, PROP_VBR_PRESET,
+      g_param_spec_enum ("vbr-preset", "Variable Bitrate Preset",
+          "AAC Variable Bitrate configurations. Requires rate-control as vbr.",
+          GST_FDK_AAC_VBR_PRESET, DEFAULT_VBR_PRESET,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_element_class_add_static_pad_template (element_class, &sink_template);
   gst_element_class_add_static_pad_template (element_class, &src_template);
 
   gst_element_class_set_static_metadata (element_class, "FDK AAC audio encoder",
-      "Codec/Encoder/Audio", "FDK AAC audio encoder",
+      "Codec/Encoder/Audio/Converter", "FDK AAC audio encoder",
       "Sebastian Dröge <sebastian@centricular.com>");
 
   GST_DEBUG_CATEGORY_INIT (gst_fdkaacenc_debug, "fdkaacenc", 0,
       "fdkaac encoder");
+
+  gst_type_mark_as_plugin_api (GST_FDK_AAC_VBR_PRESET, 0);
+  gst_type_mark_as_plugin_api (GST_FDK_AAC_RATE_CONTROL, 0);
 }
