@@ -28,6 +28,7 @@
 #include <gst/pbutils/missing-plugins.h>
 #include <gst/video/video.h>
 #include <gst/audio/audio.h>
+#include <glib/gi18n-lib.h>
 
 #include "resindvdbin.h"
 #include "resindvdsrc.h"
@@ -84,10 +85,12 @@ GST_STATIC_PAD_TEMPLATE ("subpicture",
 
 static void rsn_dvdbin_finalize (GObject * object);
 static void rsn_dvdbin_uri_handler_init (gpointer g_iface, gpointer iface_data);
+static gboolean rsndvdbin_element_init (GstPlugin * plugin);
 
 #define rsn_dvdbin_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (RsnDvdBin, rsn_dvdbin, GST_TYPE_BIN,
     G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER, rsn_dvdbin_uri_handler_init));
+GST_ELEMENT_REGISTER_DEFINE_CUSTOM (rsndvdbin, rsndvdbin_element_init);
 
 static void demux_pad_added (GstElement * element, GstPad * pad,
     RsnDvdBin * dvdbin);
@@ -100,6 +103,10 @@ static GstStateChangeReturn rsn_dvdbin_change_state (GstElement * element,
     GstStateChange transition);
 static void rsn_dvdbin_no_more_pads (RsnDvdBin * dvdbin);
 
+
+GST_DEBUG_CATEGORY (resindvd_debug);
+#define GST_CAT_DEFAULT resindvd_debug
+
 static void
 rsn_dvdbin_class_init (RsnDvdBinClass * klass)
 {
@@ -117,12 +124,12 @@ rsn_dvdbin_class_init (RsnDvdBinClass * klass)
       g_param_spec_string ("device", "Device", "DVD device location",
           NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&video_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&audio_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&subpicture_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &video_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &audio_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &subpicture_src_template);
 
   element_class->change_state = GST_DEBUG_FUNCPTR (rsn_dvdbin_change_state);
 
@@ -711,7 +718,7 @@ connect_thru_mq (RsnDvdBin * dvdbin, GstPad * pad)
 
   /* Request a pad from multiqueue, then connect this one, then
    * discover the corresponding output pad and return it */
-  mq_sink = gst_element_get_request_pad (dvdbin->pieces[DVD_ELEM_MQUEUE],
+  mq_sink = gst_element_request_pad_simple (dvdbin->pieces[DVD_ELEM_MQUEUE],
       "sink_%u");
   if (mq_sink == NULL)
     return FALSE;
@@ -788,13 +795,13 @@ demux_pad_added (GstElement * element, GstPad * pad, RsnDvdBin * dvdbin)
     GST_LOG_OBJECT (dvdbin, "Found subpicture pad w/ caps %" GST_PTR_FORMAT,
         caps);
     dest_pad =
-        gst_element_get_request_pad (dvdbin->pieces[DVD_ELEM_SPU_SELECT],
+        gst_element_request_pad_simple (dvdbin->pieces[DVD_ELEM_SPU_SELECT],
         "sink_%u");
     skip_mq = TRUE;
   } else if (can_sink_caps (dvdbin->pieces[DVD_ELEM_AUDDEC], caps)) {
     GST_LOG_OBJECT (dvdbin, "Found audio pad w/ caps %" GST_PTR_FORMAT, caps);
     dest_pad =
-        gst_element_get_request_pad (dvdbin->pieces[DVD_ELEM_AUD_SELECT],
+        gst_element_request_pad_simple (dvdbin->pieces[DVD_ELEM_AUD_SELECT],
         "sink_%u");
   } else {
     GstStructure *s;
@@ -1029,3 +1036,26 @@ rsn_dvdbin_change_state (GstElement * element, GstStateChange transition)
 
   return ret;
 }
+
+static gboolean
+rsndvdbin_element_init (GstPlugin * plugin)
+{
+  gboolean result = TRUE;
+
+  GST_DEBUG_CATEGORY_INIT (resindvd_debug, "resindvd",
+      0, "DVD playback elements from resindvd");
+
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif
+
+  result &= gst_element_register (plugin, "rsndvdbin",
+      GST_RANK_PRIMARY, RESIN_TYPE_DVDBIN);
+
+  result &= gst_flups_demux_plugin_init (plugin);
+
+  return result;
+}
