@@ -22,7 +22,6 @@
 
 #include <gst/sdp/sdp.h>
 #include "fwd.h"
-#include "gstwebrtcice.h"
 #include "transportstream.h"
 #include "webrtcsctptransport.h"
 
@@ -39,6 +38,8 @@ GType gst_webrtc_bin_pad_get_type(void);
 typedef struct _GstWebRTCBinPad GstWebRTCBinPad;
 typedef struct _GstWebRTCBinPadClass GstWebRTCBinPadClass;
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GstWebRTCBinPad, gst_object_unref);
+
 struct _GstWebRTCBinPad
 {
   GstGhostPad           parent;
@@ -46,9 +47,8 @@ struct _GstWebRTCBinPad
   GstWebRTCRTPTransceiver *trans;
   gulong                block_id;
 
-  guint32               last_ssrc;
-
   GstCaps              *received_caps;
+  char                 *msid;
 };
 
 struct _GstWebRTCBinPadClass
@@ -56,6 +56,14 @@ struct _GstWebRTCBinPadClass
   GstGhostPadClass      parent_class;
 };
 
+G_DECLARE_FINAL_TYPE (GstWebRTCBinSinkPad, gst_webrtc_bin_sink_pad, GST,
+    WEBRTC_BIN_SINK_PAD, GstWebRTCBinPad);
+#define GST_TYPE_WEBRTC_BIN_SINK_PAD (gst_webrtc_bin_sink_pad_get_type())
+
+G_DECLARE_FINAL_TYPE (GstWebRTCBinSrcPad, gst_webrtc_bin_src_pad, GST,
+    WEBRTC_BIN_SRC_PAD, GstWebRTCBinPad);
+#define GST_TYPE_WEBRTC_BIN_SRC_PAD (gst_webrtc_bin_src_pad_get_type())
+
 GType gst_webrtc_bin_get_type(void);
 #define GST_TYPE_WEBRTC_BIN            (gst_webrtc_bin_get_type())
 #define GST_WEBRTC_BIN(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_WEBRTC_BIN,GstWebRTCBin))
@@ -95,6 +103,7 @@ struct _GstWebRTCBinClass
 struct _GstWebRTCBinPrivate
 {
   guint max_sink_pad_serial;
+  guint src_pad_counter;
 
   gboolean bundle;
   GPtrArray *transceivers;
