@@ -132,18 +132,18 @@ _free_pad_block (struct pad_block *block)
   g_free (block);
 }
 
-gchar *
+const gchar *
 _enum_value_to_string (GType type, guint value)
 {
   GEnumClass *enum_class;
   GEnumValue *enum_value;
-  gchar *str = NULL;
+  const gchar *str = NULL;
 
   enum_class = g_type_class_ref (type);
   enum_value = g_enum_get_value (enum_class, value);
 
   if (enum_value)
-    str = g_strdup (enum_value->value_nick);
+    str = enum_value->value_nick;
 
   g_type_class_unref (enum_class);
 
@@ -223,3 +223,28 @@ webrtc_kind_from_caps (const GstCaps * caps)
 
   return GST_WEBRTC_KIND_UNKNOWN;
 }
+
+char *
+_get_msid_from_media (const GstSDPMedia * media)
+{
+  int i;
+
+  for (i = 0; i < gst_sdp_media_attributes_len (media); i++) {
+    const GstSDPAttribute *attr = gst_sdp_media_get_attribute (media, i);
+    const char *start, *end;
+
+    if (!attr->value)
+      continue;
+
+    start = strstr (attr->value, "msid:");
+    if (!start)
+      continue;
+
+    start += strlen ("msid:");
+    end = strstr (start, " ");
+    if (end)
+      return g_strndup (start, end - start);
+  }
+
+  return NULL;
+}
