@@ -346,11 +346,13 @@ WPEView* WPEContextThread::createWPEView(GstWpeVideoSrc* src, GstGLContext* cont
 
     WPEView* view = nullptr;
     dispatch([&]() mutable {
-        auto* manager = webkit_website_data_manager_new_ephemeral();
-        auto web_context = webkit_web_context_new_with_website_data_manager(manager);
-        g_object_unref(manager);
-
-        view = new WPEView(web_context, src, context, display, width, height);
+        if (!glib.web_context) {
+            auto *manager = webkit_website_data_manager_new_ephemeral();
+            glib.web_context =
+                webkit_web_context_new_with_website_data_manager(manager);
+            g_object_unref(manager);
+        }
+        view = new WPEView(glib.web_context, src, context, display, width, height);
     });
 
     if (view && view->hasUri()) {
@@ -693,6 +695,27 @@ void WPEView::loadUri(const gchar* uri)
     });
 }
 
+static void s_runJavascriptFinished(GObject* object, GAsyncResult* result, gpointer user_data)
+{
+    WebKitJavascriptResult* js_result;
+    GError* error = NULL;
+
+    js_result = webkit_web_view_run_javascript_finish(WEBKIT_WEB_VIEW(object), result, &error);
+    if (!js_result) {
+        GST_WARNING("Error running javascript: %s", error->message);
+        g_error_free(error);
+        return;
+    }
+    webkit_javascript_result_unref(js_result);
+}
+
+void WPEView::runJavascript(const char* script)
+{
+    s_view->dispatch([&]() {
+        webkit_web_view_run_javascript(webkit.view, script, nullptr, s_runJavascriptFinished, nullptr);
+    });
+}
+
 void WPEView::loadData(GBytes* bytes)
 {
     s_view->dispatch([this, bytes = g_bytes_ref(bytes)]() {
@@ -855,3 +878,10 @@ void WPEView::dispatchAxisEvent(struct wpe_input_axis_event& wpe_event)
         wpe_view_backend_dispatch_axis_event(backend(), &wpe_event);
     });
 }
+
+void WPEView::dispatchTouchEvent(struct wpe_input_touch_event& wpe_event)
+{
+    s_view->dispatch([&]() {
+        wpe_view_backend_dispatch_touch_event(backend(), &wpe_event);
+    });
+}
