@@ -48,10 +48,14 @@
  * ### Show the GStreamer website homepage as played with GstPlayer in a GTK+ window
  *
  * ```
- * gst-play-1.0 --videosink gtkglsink wpe://https://gstreamer.freedesktop.org
+ * export GST_PLUGIN_FEATURE_RANK="wpesrc:MAX"
+ * gst-play-1.0 --videosink gtkglsink web+https://gstreamer.freedesktop.org
  * ```
  *
- * The `web://` URI protocol is also supported, as an alias to `wpe://`. Since: 1.20
+ * Until 1.20 the `web://` URI protocol was supported, along with `wpe://`.
+ *
+ * The supported URI protocols are `web+http://`, `web+https://` and `web+file://`.
+ * Since: 1.22
  *
  * ### Composite WPE with a video stream in a single OpenGL scene
  *
@@ -108,7 +112,7 @@ static void
 gst_wpe_audio_pad_init (GstWpeAudioPad * pad)
 {
   gst_audio_info_init (&pad->info);
-  pad->discont_pending = FALSE;
+  pad->discont_pending = TRUE;
   pad->buffer_time = 0;
 }
 
@@ -134,7 +138,6 @@ struct _GstWpeSrc
   GstElement *video_src;
   GHashTable *audio_src_pads;
   GstFlowCombiner *flow_combiner;
-  gchar *uri;
 };
 
 enum
@@ -147,6 +150,7 @@ enum
 enum
 {
  SIGNAL_LOAD_BYTES,
+ SIGNAL_RUN_JAVASCRIPT,
  LAST_SIGNAL
 };
 
@@ -338,6 +342,15 @@ gst_wpe_src_load_bytes (GstWpeVideoSrc * src, GBytes * bytes)
     g_signal_emit_by_name (self->video_src, "load-bytes", bytes, NULL);
 }
 
+static void
+gst_wpe_src_run_javascript (GstWpeVideoSrc * src, const gchar * script)
+{
+  GstWpeSrc *self = GST_WPE_SRC (src);
+
+  if (self->video_src)
+    g_signal_emit_by_name (self->video_src, "run-javascript", script, NULL);
+}
+
 static void
 gst_wpe_src_set_location (GstWpeSrc * src, const gchar * location)
 {
@@ -377,7 +390,7 @@ gst_wpe_src_uri_get_type (GType)
 static const gchar *const *
 gst_wpe_src_get_protocols (GType)
 {
-  static const char *protocols[] = { "wpe", "web", NULL };
+  static const gchar *protocols[] = {"web+http", "web+https", "web+file", NULL};
   return protocols;
 }
 
@@ -385,22 +398,36 @@ static gchar *
 gst_wpe_src_get_uri (GstURIHandler * handler)
 {
   GstWpeSrc *src = GST_WPE_SRC (handler);
+  gchar *ret = NULL;
 
-  return g_strdup (src->uri);
+  g_object_get(src->video_src, "location", &ret, NULL);
+
+  return ret;
 }
 
 static gboolean
-gst_wpe_src_set_uri (GstURIHandler * handler, const gchar * uri,
+gst_wpe_src_set_uri (GstURIHandler * handler, const gchar * uristr,
     GError ** error)
 {
+  gboolean res = TRUE;
+  gchar *location;
   GstWpeSrc *src = GST_WPE_SRC (handler);
+  const gchar *protocol;
+  GstUri *uri;
 
-  if (src->uri) {
-    g_free (src->uri);
-  }
-  src->uri = g_strdup (uri);
-  gst_wpe_src_set_location(src, uri + 6);
-  return TRUE;
+  protocol = gst_uri_get_protocol (uristr);
+  g_return_val_if_fail (g_str_has_prefix (protocol, "web+"), FALSE);
+
+  uri = gst_uri_from_string (uristr);
+  gst_uri_set_scheme (uri, protocol + 4);
+
+  location = gst_uri_to_string (uri);
+  gst_wpe_src_set_location (src, location);
+
+  gst_uri_unref (uri);
+  g_free (location);
+
+  return res;
 }
 
 static void
@@ -484,7 +511,6 @@ gst_wpe_src_finalize (GObject *object)
     g_hash_table_unref (src->audio_src_pads);
     gst_flow_combiner_free (src->flow_combiner);
     gst_object_unref (src->fd_allocator);
-    g_free (src->uri);
 
     GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
@@ -500,7 +526,7 @@ gst_wpe_src_class_init (GstWpeSrcClass * klass)
   gobject_class->finalize = gst_wpe_src_finalize;
 
   g_object_class_install_property (gobject_class, PROP_LOCATION,
-      g_param_spec_string ("location", "location", "The URL to display", "",
+      g_param_spec_string ("location", "location", "The URL to display", DEFAULT_LOCATION,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
   g_object_class_install_property (gobject_class, PROP_DRAW_BACKGROUND,
       g_param_spec_boolean ("draw-background", "Draws the background",
@@ -526,6 +552,21 @@ gst_wpe_src_class_init (GstWpeSrcClass * klass)
       G_CALLBACK (gst_wpe_src_load_bytes), NULL, NULL, NULL, G_TYPE_NONE, 1,
       G_TYPE_BYTES);
 
+  /**
+   * GstWpeSrc::run-javascript:
+   * @src: the object which received the signal
+   * @script: the script to run
+   *
+   * Asynchronously run script in the context of the current page on the
+   * internal webView.
+   *
+   * Since: 1.22
+   */
+  gst_wpe_video_src_signals[SIGNAL_RUN_JAVASCRIPT] =
+      g_signal_new_class_handler ("run-javascript", G_TYPE_FROM_CLASS (klass),
+      static_cast < GSignalFlags > (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+      G_CALLBACK (gst_wpe_src_run_javascript), NULL, NULL, NULL, G_TYPE_NONE, 1,
+      G_TYPE_STRING);
   element_class->change_state = GST_DEBUG_FUNCPTR (gst_wpe_src_change_state);
 
   gst_element_class_add_static_pad_template (element_class, &video_src_factory);
