@@ -197,10 +197,12 @@ static void gst_x265_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_x265_enc_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
+static gboolean x265enc_element_init (GstPlugin * plugin);
 
 #define gst_x265_enc_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstX265Enc, gst_x265_enc, GST_TYPE_VIDEO_ENCODER,
     G_IMPLEMENT_INTERFACE (GST_TYPE_PRESET, NULL));
+GST_ELEMENT_REGISTER_DEFINE_CUSTOM (x265enc, x265enc_element_init);
 
 static gboolean
 gst_x265_enc_add_x265_chroma_format (GstStructure * s,
@@ -641,6 +643,10 @@ gst_x265_enc_start (GstVideoEncoder * encoder)
 
   g_ptr_array_set_size (x265enc->peer_profiles, 0);
 
+  /* make sure that we have enough time for first DTS,
+     this is probably overkill for most streams */
+  gst_video_encoder_set_min_pts (encoder, GST_SECOND * 60 * 60 * 1000);
+
   return TRUE;
 }
 
@@ -980,8 +986,7 @@ gst_x265_enc_init_encoder_locked (GstX265Enc * encoder)
 
   encoder->x265enc = encoder->api->encoder_open (&encoder->x265param);
   if (!encoder->x265enc) {
-    GST_ELEMENT_ERROR (encoder, STREAM, ENCODE,
-        ("Can not initialize x265 encoder."), (NULL));
+    GST_ERROR_OBJECT (encoder, "Can not open x265 encoder.");
     return FALSE;
   }
 
@@ -1006,6 +1011,10 @@ gst_x265_enc_init_encoder (GstX265Enc * encoder)
   result = gst_x265_enc_init_encoder_locked (encoder);
   GST_OBJECT_UNLOCK (encoder);
 
+  if (!result)
+    GST_ELEMENT_ERROR (encoder, STREAM, ENCODE,
+        ("Can not initialize x265 encoder."), (NULL));
+
   return result;
 }
 
@@ -1732,7 +1741,7 @@ gst_x265_enc_get_property (GObject * object, guint prop_id,
 }
 
 static gboolean
-plugin_init (GstPlugin * plugin)
+x265enc_element_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (x265_enc_debug, "x265enc", 0,
       "h265 encoding element");
@@ -1773,6 +1782,12 @@ plugin_init (GstPlugin * plugin)
       GST_RANK_PRIMARY, GST_TYPE_X265_ENC);
 }
 
+static gboolean
+plugin_init (GstPlugin * plugin)
+{
+  return GST_ELEMENT_REGISTER (x265enc, plugin);
+}
+
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     x265,
