@@ -35,6 +35,9 @@ _gst_h264_picture_free (GstH264Picture * picture)
   if (picture->notify)
     picture->notify (picture->user_data);
 
+  if (picture->discont_state)
+    gst_video_codec_state_unref (picture->discont_state);
+
   g_free (picture);
 }
 
@@ -66,7 +69,7 @@ gst_h264_picture_new (void)
 /**
  * gst_h264_picture_set_user_data:
  * @picture: a #GstH264Picture
- * @user_data: private data
+ * @user_data: (nullable): private data
  * @notify: (closure user_data): a #GDestroyNotify
  *
  * Sets @user_data on the picture and the #GDestroyNotify that will be called when
@@ -95,7 +98,7 @@ gst_h264_picture_set_user_data (GstH264Picture * picture, gpointer user_data,
  * Gets private data set on the picture via
  * gst_h264_picture_set_user_data() previously.
  *
- * Returns: (transfer none): The previously set user_data
+ * Returns: (transfer none) (nullable): The previously set user_data
  */
 gpointer
 gst_h264_picture_get_user_data (GstH264Picture * picture)
@@ -142,7 +145,7 @@ gst_h264_dpb_new (void)
       g_array_sized_new (FALSE, TRUE, sizeof (GstH264Picture *),
       GST_H264_DPB_MAX_SIZE);
   g_array_set_clear_func (dpb->pic_list,
-      (GDestroyNotify) gst_h264_picture_clear);
+      (GDestroyNotify) gst_clear_h264_picture);
 
   return dpb;
 }
@@ -461,7 +464,7 @@ gst_h264_dpb_get_long_ref_by_long_term_pic_num (GstH264Dpb * dpb,
  *
  * Find a short term reference picture which has the lowest frame_num_wrap
  *
- * Returns: (transfer full): a #GstH264Picture
+ * Returns: (transfer full) (nullable): a #GstH264Picture
  */
 GstH264Picture *
 gst_h264_dpb_get_lowest_frame_num_short_ref (GstH264Dpb * dpb)
@@ -593,7 +596,7 @@ gst_h264_dpb_get_size (GstH264Dpb * dpb)
  * @dpb: a #GstH264Dpb
  * @system_frame_number The system frame number
  *
- * Returns: (transfer full): the picture identified with the specified
+ * Returns: (transfer full) (nullable): the picture identified with the specified
  * @system_frame_number, or %NULL if DPB does not contain a #GstH264Picture
  * corresponding to the @system_frame_number
  *
