@@ -23,22 +23,62 @@
  * SECTION:element-testsrcbin
  * @title: testsrc
  *
- * This is a simple GstBin source that wraps audiotestsrc/videotestsrc
- * following specification passed in the URI (it implements the #GstURIHandler interface)
+ * This is a simple GstBin source that wraps audiotestsrc/videotestsrc following
+ * specification passed in the URI (it implements the #GstURIHandler interface)
  * in the form of `testbin://audio+video` or setting the "stream-types" property
  * with the same format.
  *
- * This element also provides GstStream and GstStreamCollection and
- * thus the element is useful for testing the new playbin3 infrastructure.
+ * This element also provides GstStream and GstStreamCollection and thus the
+ * element is useful for testing the new playbin3 infrastructure.
+ *
+ * ## The `uri` format
+ *
+ * `testbin://<stream1 definition>[+<stream2 definition>]`
+ *
+ * With **<stream definition>**:
+ *
+ *  `<media-type>,<element-properties>,[caps=<media caps>]`
+ *
+ * where:
+ *
+ * - `<media-type>`: Adds a new source of type `<media-type>`. Supported
+ *   values:
+ *      * `video`: A #videotestsrc element will be used
+ *      * `audio`: An #audiotestsrc will be used
+ *   you can use it as many time as wanted to expose new streams.
+ * - `<element-properties>`: `key=value` list of properties to be set on the
+ *   source element. See #videotestsrc properties for the video case and
+ *   #audiotestsrc properties for the audio case.
+ * - `<media caps>`: Caps to be set in the #capsfilter that follows source elements
+ *   for example to force the video source to output a full HD stream, you can use
+ *   `video/x-raw,width=1920,height=1080`.
+ *
+ * Note that stream definitions are interpreted as serialized #GstStructure.
+ *
+ * ## Examples pipeline:
+ *
+ * ### One audio stream with volume=0.5 and a white video stream in full HD at 30fps
  *
- * Example pipeline:
  * ```
- * gst-launch-1.0 playbin uri=testbin://audio,volume=0.5+video,pattern=white
+ * gst-launch-1.0 playbin3 uri="testbin://audio,volume=0.5+video,pattern=white,caps=[video/x-raw,width=1920,height=1080,framerate=30/1]"
+ * ```
+
+ * ### Single full HD stream
+ *
+ * ```
+ * gst-launch-1.0 playbin3 uri="testbin://video,pattern=green,caps=[video/x-raw,width=1920,height=1080,framerate=30/1]"
+ * ```
+ *
+ * ### Two audio streams
+ *
+ * ```
+ * gst-launch-1.0 playbin3 uri="testbin://audio+audio"
  * ```
  */
 #include <gst/gst.h>
 #include <gst/base/gstflowcombiner.h>
 #include <gst/app/gstappsink.h>
+#include "gstdebugutilsbadelements.h"
 
 static GstStaticPadTemplate video_src_template =
 GST_STATIC_PAD_TEMPLATE ("video_src_%u",
@@ -72,15 +112,18 @@ struct _GstTestSrcBin
   GstBin parent;
 
   gchar *uri;
-  GstStreamCollection *collection;
   gint group_id;
   GstFlowCombiner *flow_combiner;
+  GstCaps *streams_def;
+  GstCaps *next_streams_def;
+  gboolean expose_sources_async;
 };
 
 enum
 {
   PROP_0,
   PROP_STREAM_TYPES,
+  PROP_EXPOSE_SOURCES_ASYNC,
   PROP_LAST
 };
 
@@ -124,7 +167,7 @@ _probe_data_new (GstEvent * stream_start, GstStreamCollection * collection)
 {
   ProbeData *data = g_malloc0 (sizeof (ProbeData));
 
-  data->stream_start = gst_event_ref (stream_start);
+  data->stream_start = stream_start;
   data->collection = gst_object_ref (collection);
 
   return data;
@@ -195,6 +238,12 @@ static gboolean
 gst_test_src_bin_set_element_property (GQuark property_id, const GValue * value,
     GObject * element)
 {
+  if (property_id == g_quark_from_static_string ("__streamobj__"))
+    return TRUE;
+
+  if (property_id == g_quark_from_static_string ("caps"))
+    return TRUE;
+
   if (G_VALUE_HOLDS_STRING (value))
     gst_util_set_object_arg (element, g_quark_to_string (property_id),
         g_value_get_string (value));
@@ -245,19 +294,64 @@ gst_test_src_event_function (GstPad * pad, GstObject * parent, GstEvent * event)
   return gst_pad_event_default (pad, parent, event);
 }
 
-static void
+static gboolean
 gst_test_src_bin_setup_src (GstTestSrcBin * self, const gchar * srcfactory,
     GstStaticPadTemplate * template, GstStreamType stype,
-    GstStreamCollection * collection, gint * n_stream, GstStructure * props)
+    GstStreamCollection * collection, gint * n_stream, GstStructure * props,
+    GError ** error)
 {
-  GstElement *src = gst_element_factory_make (srcfactory, NULL);
-  GstPad *proxypad, *ghost, *pad = gst_element_get_static_pad (src, "src");
-  gchar *stream_id = g_strdup_printf ("%s_stream_%d", srcfactory, *n_stream);
-  gchar *pad_name = g_strdup_printf (template->name_template, *n_stream);
-  GstStream *stream = gst_stream_new (stream_id, NULL, stype,
+  GstElement *src;
+  GstElement *capsfilter;
+  GstPad *proxypad, *ghost, *pad;
+  gchar *stream_id;
+  gchar *pad_name;
+  GstCaps *caps = NULL;
+  GstStream *stream;
+  GstEvent *stream_start;
+  GstPadTemplate *templ;
+  const GValue *caps_value = gst_structure_get_value (props, "caps");
+
+  if (caps_value) {
+    if (GST_VALUE_HOLDS_CAPS (caps_value)) {
+      caps = gst_caps_copy (gst_value_get_caps (caps_value));
+    } else if (GST_VALUE_HOLDS_STRUCTURE (caps_value)) {
+      caps =
+          gst_caps_new_full (gst_structure_copy (gst_value_get_structure
+              (caps_value)), NULL);
+    } else if (G_VALUE_HOLDS_STRING (caps_value)) {
+      caps = gst_caps_from_string (g_value_get_string (caps_value));
+      if (!caps) {
+        if (error) {
+          *error =
+              g_error_new (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
+              "Invalid caps string: %s", g_value_get_string (caps_value));
+        }
+
+        return FALSE;
+      }
+    } else {
+      if (error) {
+        *error =
+            g_error_new (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
+            "Invalid type %s for `caps`", G_VALUE_TYPE_NAME (caps_value));
+      }
+
+      return FALSE;
+    }
+  }
+
+  capsfilter = gst_element_factory_make ("capsfilter", NULL);
+  if (caps) {
+    g_object_set (capsfilter, "caps", caps, NULL);
+    gst_caps_unref (caps);
+  }
+
+  src = gst_element_factory_make (srcfactory, NULL);
+  pad = gst_element_get_static_pad (src, "src");
+  stream_id = g_strdup_printf ("%s_stream_%d", srcfactory, *n_stream);
+  stream = gst_stream_new (stream_id, caps, stype,
       (*n_stream == 0) ? GST_STREAM_FLAG_SELECT : GST_STREAM_FLAG_UNSELECT);
-  GstEvent *stream_start =
-      gst_event_new_stream_start (gst_stream_get_stream_id (stream));
+  stream_start = gst_event_new_stream_start (gst_stream_get_stream_id (stream));
 
   gst_structure_foreach (props,
       (GstStructureForeachFunc) gst_test_src_bin_set_element_property, src);
@@ -265,18 +359,30 @@ gst_test_src_bin_setup_src (GstTestSrcBin * self, const gchar * srcfactory,
   gst_event_set_stream (stream_start, stream);
   gst_event_set_group_id (stream_start, self->group_id);
 
+  gst_structure_set (props, "__streamobj__", GST_TYPE_STREAM, stream, NULL);
+  gst_stream_collection_add_stream (collection, stream);
+
   gst_pad_add_probe (pad, (GstPadProbeType) GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM,
       (GstPadProbeCallback) src_pad_probe_cb, _probe_data_new (stream_start,
           collection), (GDestroyNotify) _probe_data_free);
 
-  gst_stream_collection_add_stream (collection, stream);
   g_free (stream_id);
 
-  gst_bin_add (GST_BIN (self), src);
+  gst_bin_add_many (GST_BIN (self), src, capsfilter, NULL);
+  if (!gst_element_link (src, capsfilter)) {
+    g_error ("Could not link src with capsfilter?!");
+  }
+
+  gst_object_unref (pad);
+
+  pad = gst_element_get_static_pad (capsfilter, "src");
+  pad_name = g_strdup_printf (template->name_template, *n_stream);
+  templ = gst_static_pad_template_get (template);
+  ghost = gst_ghost_pad_new_from_template (pad_name, pad, templ);
+  gst_object_unref (templ);
+  g_free (pad_name);
+  gst_object_unref (pad);
 
-  ghost =
-      gst_ghost_pad_new_from_template (pad_name, pad,
-      gst_static_pad_template_get (template));
   proxypad = GST_PAD (gst_proxy_pad_get_internal (GST_PROXY_PAD (ghost)));
   gst_flow_combiner_add_pad (self->flow_combiner, ghost);
   gst_pad_set_chain_function (proxypad,
@@ -284,18 +390,157 @@ gst_test_src_bin_setup_src (GstTestSrcBin * self, const gchar * srcfactory,
   gst_pad_set_event_function (ghost,
       (GstPadEventFunction) gst_test_src_event_function);
   gst_object_unref (proxypad);
+  gst_pad_store_sticky_event (ghost, stream_start);
   gst_element_add_pad (GST_ELEMENT (self), ghost);
-  gst_object_unref (pad);
+  gst_element_sync_state_with_parent (capsfilter);
   gst_element_sync_state_with_parent (src);
   *n_stream += 1;
+
+  gst_structure_set (props, "__src__", GST_TYPE_OBJECT, src, NULL);
+
+  gst_clear_caps (&caps);
+
+  return TRUE;
 }
 
 static void
-gst_test_src_bin_remove_child (GValue * val, GstBin * self)
+gst_test_src_bin_remove_child (GstElement * self, GstElement * child)
 {
-  GstElement *child = g_value_get_object (val);
+  GstPad *pad = gst_element_get_static_pad (child, "src");
+  GstPad *ghost =
+      GST_PAD (gst_proxy_pad_get_internal (GST_PROXY_PAD (gst_pad_get_peer
+              (pad))));
 
-  gst_bin_remove (self, child);
+
+  gst_element_set_locked_state (child, FALSE);
+  gst_element_set_state (child, GST_STATE_NULL);
+  gst_bin_remove (GST_BIN (self), child);
+  gst_element_remove_pad (self, ghost);
+}
+
+static GstStream *
+gst_test_check_prev_stream_def (GstTestSrcBin * self, GstCaps * prev_streams,
+    GstStructure * stream_def)
+{
+  gint i;
+
+  if (!prev_streams)
+    return NULL;
+
+  for (i = 0; i < gst_caps_get_size (prev_streams); i++) {
+    GstStructure *prev_stream = gst_caps_get_structure (prev_streams, i);
+    GstElement *e = NULL;
+    GstStream *stream = NULL;
+
+    gst_structure_get (prev_stream, "__src__", GST_TYPE_OBJECT, &e,
+        "__streamobj__", GST_TYPE_STREAM, &stream, NULL);
+    gst_structure_remove_fields (prev_stream, "__src__", "__streamobj__", NULL);
+    if (gst_structure_is_equal (prev_stream, stream_def)) {
+      g_assert (stream);
+
+      gst_caps_remove_structure (prev_streams, i);
+      gst_structure_set (stream_def, "__src__", GST_TYPE_OBJECT, e,
+          "__streamobj__", GST_TYPE_STREAM, stream, NULL);
+
+      g_assert (stream);
+      return stream;
+    }
+
+    gst_structure_set (stream_def, "__src__", GST_TYPE_OBJECT, e,
+        "__streamobj__", GST_TYPE_STREAM, stream, NULL);
+  }
+
+  return NULL;
+}
+
+static gboolean
+gst_test_src_bin_create_sources (GstTestSrcBin * self)
+{
+  gint i, n_audio = 0, n_video = 0;
+  GError *error = NULL;
+  GstStreamCollection *collection = gst_stream_collection_new (NULL);
+  GstCaps *streams_def, *prev_streams_def;
+
+  GST_OBJECT_LOCK (self);
+  streams_def = self->next_streams_def;
+  prev_streams_def = self->streams_def;
+  self->next_streams_def = NULL;
+  self->streams_def = NULL;
+  GST_OBJECT_UNLOCK (self);
+
+  GST_INFO_OBJECT (self, "Create sources %" GST_PTR_FORMAT,
+      self->next_streams_def);
+
+  self->group_id = gst_util_group_id_next ();
+  for (i = 0; i < gst_caps_get_size (streams_def); i++) {
+    GstStream *stream;
+    GstStructure *stream_def = gst_caps_get_structure (streams_def, i);
+
+    if ((stream =
+            gst_test_check_prev_stream_def (self, prev_streams_def,
+                stream_def))) {
+      GST_INFO_OBJECT (self,
+          "Reusing already existing stream: %" GST_PTR_FORMAT, stream_def);
+      gst_stream_collection_add_stream (collection, stream);
+      if (gst_structure_has_name (stream_def, "video"))
+        n_video++;
+      else
+        n_audio++;
+      continue;
+    }
+
+    if (gst_structure_has_name (stream_def, "video")) {
+      if (!gst_test_src_bin_setup_src (self, "videotestsrc",
+              &video_src_template, GST_STREAM_TYPE_VIDEO, collection, &n_video,
+              stream_def, &error)) {
+        goto failed;
+      }
+    } else if (gst_structure_has_name (stream_def, "audio")) {
+      if (!gst_test_src_bin_setup_src (self, "audiotestsrc",
+              &audio_src_template, GST_STREAM_TYPE_AUDIO, collection, &n_audio,
+              stream_def, &error)) {
+        goto failed;
+      }
+    } else {
+      GST_ERROR_OBJECT (self, "Unknown type %s",
+          gst_structure_get_name (stream_def));
+    }
+  }
+
+  if (prev_streams_def) {
+    for (i = 0; i < gst_caps_get_size (prev_streams_def); i++) {
+      GstStructure *prev_stream = gst_caps_get_structure (prev_streams_def, i);
+      GstElement *child;
+
+      gst_structure_get (prev_stream, "__src__", GST_TYPE_OBJECT, &child, NULL);
+      gst_test_src_bin_remove_child (GST_ELEMENT (self), child);
+    }
+    gst_clear_caps (&prev_streams_def);
+  }
+
+  if (!n_video && !n_audio) {
+    GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
+        ("No audio or video stream defined."), (NULL));
+    goto failed;
+  }
+
+  GST_OBJECT_LOCK (self);
+  self->streams_def = streams_def;
+  GST_OBJECT_UNLOCK (self);
+
+  gst_element_post_message (GST_ELEMENT (self),
+      gst_message_new_stream_collection (GST_OBJECT (self), collection));
+  gst_object_unref (collection);
+
+  gst_element_no_more_pads (GST_ELEMENT (self));
+
+  return TRUE;
+
+failed:
+  gst_element_post_message (GST_ELEMENT (self),
+      gst_message_new_error (GST_OBJECT (self), error, NULL)
+      );
+  return FALSE;
 }
 
 static gboolean
@@ -304,59 +549,44 @@ gst_test_src_bin_uri_handler_set_uri (GstURIHandler * handler,
 {
   GstTestSrcBin *self = GST_TEST_SRC_BIN (handler);
   gchar *tmp, *location = gst_uri_get_location (uri);
-  gint i, n_audio = 0, n_video = 0;
-  GstStreamCollection *collection = gst_stream_collection_new (NULL);
-  GstIterator *it;
-  GstCaps *streams_defs;
+  GstCaps *streams_def;
 
   for (tmp = location; *tmp != '\0'; tmp++)
     if (*tmp == '+')
       *tmp = ';';
 
-  streams_defs = gst_caps_from_string (location);
+  streams_def = gst_caps_from_string (location);
   g_free (location);
 
-  if (!streams_defs)
-    goto failed;
-
-  /* Clear us up */
-  it = gst_bin_iterate_elements (GST_BIN (self));
-  while (gst_iterator_foreach (it,
-          (GstIteratorForeachFunction) gst_test_src_bin_remove_child,
-          self) == GST_ITERATOR_RESYNC)
-    gst_iterator_resync (it);
-
-  gst_iterator_free (it);
-
-  self->group_id = gst_util_group_id_next ();
-  for (i = 0; i < gst_caps_get_size (streams_defs); i++) {
-    GstStructure *stream_def = gst_caps_get_structure (streams_defs, i);
-
-    if (gst_structure_has_name (stream_def, "video"))
-      gst_test_src_bin_setup_src (self, "videotestsrc", &video_src_template,
-          GST_STREAM_TYPE_VIDEO, collection, &n_video, stream_def);
-    else if (gst_structure_has_name (stream_def, "audio"))
-      gst_test_src_bin_setup_src (self, "audiotestsrc", &audio_src_template,
-          GST_STREAM_TYPE_AUDIO, collection, &n_audio, stream_def);
-    else
-      GST_ERROR_OBJECT (self, "Unknown type %s",
-          gst_structure_get_name (stream_def));
-  }
-
-  if (!n_video && !n_audio)
+  if (!streams_def)
     goto failed;
 
+  GST_OBJECT_LOCK (self);
+  gst_clear_caps (&self->next_streams_def);
+  self->next_streams_def = streams_def;
+  g_free (self->uri);
   self->uri = g_strdup (uri);
-  gst_element_post_message (GST_ELEMENT (self),
-      gst_message_new_stream_collection (GST_OBJECT (self), collection));
+
+  if (GST_STATE (self) >= GST_STATE_PAUSED) {
+
+    if (self->expose_sources_async) {
+      GST_OBJECT_UNLOCK (self);
+
+      gst_element_call_async (GST_ELEMENT (self),
+          (GstElementCallAsyncFunc) gst_test_src_bin_create_sources,
+          NULL, NULL);
+    } else {
+      GST_OBJECT_UNLOCK (self);
+
+      gst_test_src_bin_create_sources (self);
+    }
+  } else {
+    GST_OBJECT_UNLOCK (self);
+  }
 
   return TRUE;
 
 failed:
-  if (error)
-    *error =
-        g_error_new_literal (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
-        "No media type specified in the testbin:// URL.");
 
   return FALSE;
 }
@@ -377,6 +607,9 @@ G_DEFINE_TYPE_WITH_CODE (GstTestSrcBin, gst_test_src_bin, GST_TYPE_BIN,
     G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER, gst_test_src_bin_uri_handler_init))
 /* *INDENT-ON* */
 
+GST_ELEMENT_REGISTER_DEFINE (testsrcbin, "testsrcbin",
+    GST_RANK_NONE, gst_test_src_bin_get_type ());
+
 static void
 gst_test_src_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -392,6 +625,13 @@ gst_test_src_bin_set_property (GObject * object, guint prop_id,
       g_free (uri);
       break;
     }
+    case PROP_EXPOSE_SOURCES_ASYNC:
+    {
+      GST_OBJECT_LOCK (self);
+      self->expose_sources_async = g_value_get_boolean (value);
+      GST_OBJECT_UNLOCK (self);
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -416,6 +656,13 @@ gst_test_src_bin_get_property (GObject * object, guint prop_id, GValue * value,
       }
       break;
     }
+    case PROP_EXPOSE_SOURCES_ASYNC:
+    {
+      GST_OBJECT_LOCK (self);
+      g_value_set_boolean (value, self->expose_sources_async);
+      GST_OBJECT_UNLOCK (self);
+      break;
+    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -428,6 +675,21 @@ gst_test_src_bin_change_state (GstElement * element, GstStateChange transition)
   GstTestSrcBin *self = GST_TEST_SRC_BIN (element);
   GstStateChangeReturn result = GST_STATE_CHANGE_FAILURE;
 
+  switch (transition) {
+    case GST_STATE_CHANGE_READY_TO_PAUSED:{
+      if (self->expose_sources_async) {
+        gst_element_call_async (element,
+            (GstElementCallAsyncFunc) gst_test_src_bin_create_sources,
+            NULL, NULL);
+      } else {
+        gst_test_src_bin_create_sources (self);
+      }
+      break;
+    }
+    default:
+      break;
+  }
+
   result =
       GST_ELEMENT_CLASS (gst_test_src_bin_parent_class)->change_state (element,
       transition);
@@ -449,8 +711,12 @@ gst_test_src_bin_finalize (GObject * object)
 {
   GstTestSrcBin *self = GST_TEST_SRC_BIN (object);
 
+  G_OBJECT_CLASS (gst_test_src_bin_parent_class)->finalize (object);
+
   g_free (self->uri);
+  gst_clear_caps (&self->streams_def);
   gst_flow_combiner_free (self->flow_combiner);
+
 }
 
 static void
@@ -470,7 +736,7 @@ gst_test_src_bin_class_init (GstTestSrcBinClass * klass)
   gobject_class->set_property = gst_test_src_bin_set_property;
 
   /**
-   * GstTestSrcBin::stream-types:
+   * GstTestSrcBin:stream-types:
    *
    * String describing the stream types to expose, eg. "video+audio".
    */
@@ -479,6 +745,20 @@ gst_test_src_bin_class_init (GstTestSrcBinClass * klass)
           "String describing the stream types to expose, eg. \"video+audio\".",
           NULL, (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
+  /**
+   * GstTestSrcBin:expose-sources-async:
+   *
+   * Whether to expose sources at random time to simulate a source that is
+   * reading a file and exposing the srcpads later.
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_EXPOSE_SOURCES_ASYNC,
+      g_param_spec_boolean ("expose-sources-async", "Expose Sources Async",
+          " Whether to expose sources at random time to simulate a source that is"
+          " reading a file and exposing the srcpads later.",
+          FALSE, (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_test_src_bin_change_state);
   gst_element_class_add_pad_template (gstelement_klass,
