@@ -47,15 +47,15 @@
 
 /**
  * SECTION:element-gaussianblur
+ * @title: gaussianblur
  *
  * Gaussianblur blurs the video stream in realtime.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
- * gst-launch -v videotestsrc ! gaussianblur ! videoconvert ! autovideosink
+ * gst-launch-1.0 -v videotestsrc ! gaussianblur ! videoconvert ! autovideosink
  * ]| This pipeline shows the effect of gaussianblur on a test stream
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -66,7 +66,6 @@
 #include <math.h>
 #include <gst/gst.h>
 
-#include "gstplugin.h"
 #include "gstgaussblur.h"
 
 static void gst_gaussianblur_finalize (GObject * object);
@@ -111,8 +110,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
 enum
 {
   PROP_0,
-  PROP_SIGMA,
-  PROP_LAST
+  PROP_SIGMA
 };
 
 static gboolean make_gaussian_kernel (GstGaussianBlur * gb, float sigma);
@@ -121,10 +119,13 @@ static void gaussian_smooth (GstGaussianBlur * gb, guint8 * image,
 
 #define gst_gaussianblur_parent_class parent_class
 G_DEFINE_TYPE (GstGaussianBlur, gst_gaussianblur, GST_TYPE_VIDEO_FILTER);
-
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (gaussianblur, "gaussianblur",
+    GST_RANK_NONE, GST_TYPE_GAUSSIANBLUR,
+    GST_DEBUG_CATEGORY_INIT (gst_gauss_blur_debug, "gaussianblur", 0,
+        "Gaussian Blur video effect"));
 #define DEFAULT_SIGMA 1.2
 
-/* Initalize the gaussianblur's class. */
+/* Initialize the gaussianblur's class. */
 static void
 gst_gaussianblur_class_init (GstGaussianBlurClass * klass)
 {
@@ -138,10 +139,10 @@ gst_gaussianblur_class_init (GstGaussianBlurClass * klass)
       "Perform Gaussian blur/sharpen on a video",
       "Jan Schmidt <thaytan@noraisin.net>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_gaussianblur_sink_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_gaussianblur_src_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_gaussianblur_sink_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_gaussianblur_src_template);
 
   gobject_class->set_property = gst_gaussianblur_set_property;
   gobject_class->get_property = gst_gaussianblur_get_property;
@@ -171,6 +172,7 @@ gst_gaussianblur_set_info (GstVideoFilter * filter, GstCaps * incaps,
   /* get stride */
   gb->stride = GST_VIDEO_INFO_COMP_STRIDE (in_info, 0);
   n_elems = gb->stride * gb->height;
+  g_free (gb->tempim);
   gb->tempim = g_malloc (sizeof (gfloat) * n_elems);
 
   return TRUE;
@@ -179,7 +181,7 @@ gst_gaussianblur_set_info (GstVideoFilter * filter, GstCaps * incaps,
 static void
 gst_gaussianblur_init (GstGaussianBlur * gb)
 {
-  gb->sigma = DEFAULT_SIGMA;
+  gb->sigma = (gfloat) DEFAULT_SIGMA;
   gb->cur_sigma = -1.0;
 }
 
@@ -249,7 +251,8 @@ gst_gaussianblur_transform_frame (GstVideoFilter * vfilter,
   src = GST_VIDEO_FRAME_COMP_DATA (in_frame, 0);
   dest = GST_VIDEO_FRAME_COMP_DATA (out_frame, 0);
   gst_video_frame_copy (out_frame, in_frame);
-  gaussian_smooth (filter, src, dest);
+  if (filter->sigma != 0.0)
+    gaussian_smooth (filter, src, dest);
 
   return GST_FLOW_OK;
 }
@@ -452,15 +455,3 @@ gst_gaussianblur_get_property (GObject * object,
       break;
   }
 }
-
-/* Register the element factories and other features. */
-gboolean
-gst_gauss_blur_plugin_init (GstPlugin * plugin)
-{
-  /* debug category for fltering log messages */
-  GST_DEBUG_CATEGORY_INIT (gst_gauss_blur_debug, "gaussianblur",
-      0, "Gaussian Blur video effect");
-
-  return gst_element_register (plugin, "gaussianblur", GST_RANK_NONE,
-      GST_TYPE_GAUSSIANBLUR);
-}
