@@ -1,13 +1,12 @@
-/* 
- * Copyright 2006 BBC and Fluendo S.A. 
+/*
+ * Copyright 2006 BBC and Fluendo S.A.
  *
- * This library is licensed under 4 different licenses and you
+ * This library is licensed under 3 different licenses and you
  * can choose to use it under the terms of any one of them. The
- * four licenses are the MPL 1.1, the LGPL, the GPL and the MIT
- * license.
+ * three licenses are the MPL 1.1, the LGPL, and the MIT license.
  *
  * MPL:
- * 
+ *
  * The contents of this file are subject to the Mozilla Public License
  * Version 1.1 (the "License"); you may not use this file except in
  * compliance with the License. You may obtain a copy of the License at
@@ -35,38 +34,22 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * GPL:
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
- *
  * MIT:
  *
  * Unless otherwise indicated, Source Code is licensed under MIT license.
  * See further explanation attached in License Statement (distributed in the file
  * LICENSE).
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
  * the Software without restriction, including without limitation the rights to
  * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
  * of the Software, and to permit persons to whom the Software is furnished to do
  * so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included in all
  * copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -75,6 +58,7 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  *
+ * SPDX-License-Identifier: MPL-1.1 OR MIT OR LGPL-2.0-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -88,10 +72,10 @@
 #include "tsmux.h"
 #include "tsmuxstream.h"
 
-#define GST_CAT_DEFAULT mpegtsmux_debug
+#define GST_CAT_DEFAULT gst_base_ts_mux_debug
 
-/* Maximum total data length for a PAT section is 1024 bytes, minus an 
- * 8 byte header, then the length of each program entry is 32 bits, 
+/* Maximum total data length for a PAT section is 1024 bytes, minus an
+ * 8 byte header, then the length of each program entry is 32 bits,
  * then finally a 32 bit CRC. Thus the maximum number of programs in this mux
  * is (1024 - 8 - 4) / 4 = 253 because it only supports single section PATs */
 #define TSMUX_MAX_PROGRAMS 253
@@ -101,20 +85,30 @@
 #define TSMUX_DEFAULT_NETWORK_ID 0x0001
 #define TSMUX_DEFAULT_TS_ID 0x0001
 
-/* HACK: We use a fixed buffering offset for the PCR at the moment - 
+/* The last byte of the PCR in the header defines the byte position
+ * at which PCR should be calculated */
+#define PCR_BYTE_OFFSET 11
+
+/* HACK: We use a fixed buffering offset for the PCR at the moment -
  * this is the amount 'in advance' of the stream that the PCR sits.
  * 1/8 second atm */
 #define TSMUX_PCR_OFFSET (TSMUX_CLOCK_FREQ / 8)
 
-/* Times per second to write PCR */
-#define TSMUX_DEFAULT_PCR_FREQ (25)
-
 /* Base for all written PCR and DTS/PTS,
  * so we have some slack to go backwards */
 #define CLOCK_BASE (TSMUX_CLOCK_FREQ * 10 * 360)
 
 static gboolean tsmux_write_pat (TsMux * mux);
 static gboolean tsmux_write_pmt (TsMux * mux, TsMuxProgram * program);
+static gboolean tsmux_write_scte_null (TsMux * mux, TsMuxProgram * program);
+static gint64 get_next_pcr (TsMux * mux, gint64 cur_ts);
+static gint64 get_current_pcr (TsMux * mux, gint64 cur_ts);
+static gint64 write_new_pcr (TsMux * mux, TsMuxStream * stream, gint64 cur_pcr,
+    gint64 next_pcr);
+static gboolean tsmux_write_ts_header (TsMux * mux, guint8 * buf,
+    TsMuxPacketInfo * pi, guint * payload_len_out, guint * payload_offset_out,
+    guint stream_avail);
+
 static void
 tsmux_section_free (TsMuxSection * section)
 {
@@ -143,16 +137,24 @@ tsmux_new (void)
   mux->next_stream_pid = TSMUX_START_ES_PID;
 
   mux->pat_changed = TRUE;
-  mux->last_pat_ts = G_MININT64;
+  mux->next_pat_pcr = -1;
   mux->pat_interval = TSMUX_DEFAULT_PAT_INTERVAL;
 
   mux->si_changed = TRUE;
-  mux->last_si_ts = G_MININT64;
   mux->si_interval = TSMUX_DEFAULT_SI_INTERVAL;
 
+  mux->pcr_interval = TSMUX_DEFAULT_PCR_INTERVAL;
+
+  mux->next_si_pcr = -1;
+
   mux->si_sections = g_hash_table_new_full (g_direct_hash, g_direct_equal,
       NULL, (GDestroyNotify) tsmux_section_free);
 
+  mux->new_stream_func = (TsMuxNewStreamFunc) tsmux_stream_new;
+  mux->new_stream_data = NULL;
+
+  mux->first_pcr_ts = G_MININT64;
+
   return mux;
 }
 
@@ -193,6 +195,26 @@ tsmux_set_alloc_func (TsMux * mux, TsMuxAllocFunc func, void *user_data)
   mux->alloc_func_data = user_data;
 }
 
+/**
+ * tsmux_set_new_stream_func:
+ * @mux: a #TsMux
+ * @func: a user callback function
+ * @user_data: user data passed to @func
+ *
+ * Set the callback function and user data to be called when @mux needs
+ * to create a new stream.
+ * @user_data will be passed as user data in @func.
+ */
+void
+tsmux_set_new_stream_func (TsMux * mux, TsMuxNewStreamFunc func,
+    void *user_data)
+{
+  g_return_if_fail (mux != NULL);
+
+  mux->new_stream_func = func;
+  mux->new_stream_data = user_data;
+}
+
 /**
  * tsmux_set_pat_interval:
  * @mux: a #TsMux
@@ -212,6 +234,21 @@ tsmux_set_pat_interval (TsMux * mux, guint freq)
   mux->pat_interval = freq;
 }
 
+/**
+ * tsmux_set_pcr_interval:
+ * @mux: a #TsMux
+ * @freq: a new PCR interval
+ *
+ * Set the interval (in cycles of the 90kHz clock) for writing the PCR.
+ */
+void
+tsmux_set_pcr_interval (TsMux * mux, guint freq)
+{
+  g_return_if_fail (mux != NULL);
+
+  mux->pcr_interval = freq;
+}
+
 /**
  * tsmux_get_pat_interval:
  * @mux: a #TsMux
@@ -239,7 +276,7 @@ tsmux_resend_pat (TsMux * mux)
 {
   g_return_if_fail (mux != NULL);
 
-  mux->last_pat_ts = G_MININT64;
+  mux->next_pat_pcr = -1;
 }
 
 /**
@@ -286,7 +323,7 @@ tsmux_resend_si (TsMux * mux)
 {
   g_return_if_fail (mux != NULL);
 
-  mux->last_si_ts = G_MININT64;
+  mux->next_si_pcr = -1;
 }
 
 /**
@@ -323,6 +360,7 @@ tsmux_add_mpegts_si_section (TsMux * mux, GstMpegtsSection * section)
   return TRUE;
 }
 
+
 /**
  * tsmux_free:
  * @mux: a #TsMux
@@ -358,7 +396,7 @@ tsmux_free (TsMux * mux)
   g_list_free (mux->streams);
 
   /* Free SI table sections */
-  g_hash_table_destroy (mux->si_sections);
+  g_hash_table_unref (mux->si_sections);
 
   g_slice_free (TsMux, mux);
 }
@@ -373,7 +411,7 @@ tsmux_program_compare (TsMuxProgram * program, gint * needle)
  * tsmux_program_new:
  * @mux: a #TsMux
  *
- * Create a new program in the mising session @mux.
+ * Create a new program in the missing session @mux.
  *
  * Returns: a new #TsMuxProgram or %NULL when the maximum number of programs has
  * been reached.
@@ -392,9 +430,10 @@ tsmux_program_new (TsMux * mux, gint prog_id)
   program = g_slice_new0 (TsMuxProgram);
 
   program->pmt_changed = TRUE;
-  program->last_pmt_ts = G_MININT64;
   program->pmt_interval = TSMUX_DEFAULT_PMT_INTERVAL;
 
+  program->next_pmt_pcr = -1;
+
   if (prog_id == 0) {
     program->pgm_number = mux->next_pgm_no++;
     while (g_list_find_custom (mux->programs, &program->pgm_number,
@@ -412,7 +451,13 @@ tsmux_program_new (TsMux * mux, gint prog_id)
   program->pmt_pid = mux->next_pmt_pid++;
   program->pcr_stream = NULL;
 
-  program->streams = g_array_sized_new (FALSE, TRUE, sizeof (TsMuxStream *), 1);
+  /* SCTE35 is disabled by default */
+  program->scte35_pid = 0;
+  program->scte35_null_interval = TSMUX_DEFAULT_SCTE_35_NULL_INTERVAL;
+  program->next_scte35_pcr = -1;
+
+  /* mux->streams owns the streams */
+  program->streams = g_ptr_array_new_full (1, NULL);
 
   mux->programs = g_list_prepend (mux->programs, program);
   mux->nb_programs++;
@@ -421,6 +466,25 @@ tsmux_program_new (TsMux * mux, gint prog_id)
   return program;
 }
 
+gboolean
+tsmux_program_delete (TsMux * mux, TsMuxProgram * program)
+{
+  g_return_val_if_fail (mux != NULL, FALSE);
+
+  if (mux->nb_programs == 0)
+    return FALSE;
+
+  if (!program)
+    return FALSE;
+
+  mux->programs = g_list_remove (mux->programs, program);
+  mux->nb_programs--;
+  mux->pat_changed = TRUE;
+  tsmux_program_free ((TsMuxProgram *) program);
+
+  return TRUE;
+}
+
 /**
  * tsmux_set_pmt_interval:
  * @program: a #TsMuxProgram
@@ -456,6 +520,22 @@ tsmux_get_pmt_interval (TsMuxProgram * program)
   return program->pmt_interval;
 }
 
+/**
+ * tsmux_program_set_scte35_interval:
+ * @program: a #TsMuxProgram
+ * @freq: a new SCTE-35 NULL interval
+ *
+ * Set the interval (in cycles of the 90kHz clock) for sending out the SCTE-35
+ * NULL command. This is only effective is the SCTE-35 PID is not 0.
+ */
+void
+tsmux_program_set_scte35_interval (TsMuxProgram * program, guint interval)
+{
+  g_return_if_fail (program != NULL);
+
+  program->scte35_null_interval = interval;
+}
+
 /**
  * tsmux_resend_pmt:
  * @program: a #TsMuxProgram
@@ -467,7 +547,56 @@ tsmux_resend_pmt (TsMuxProgram * program)
 {
   g_return_if_fail (program != NULL);
 
-  program->last_pmt_ts = G_MININT64;
+  program->next_pmt_pcr = -1;
+}
+
+/**
+ * tsmux_program_set_scte35_pid:
+ * @program: a #TsMuxProgram
+ * @pid: The pid to use, or 0 to deactivate usage.
+ *
+ * Set the @pid to use for sending SCTE-35 packets on the given
+ * @program.
+ *
+ * This needs to be called as early as possible if SCTE-35 sections
+ * are even going to be used with the given @program so that the PMT
+ * can be properly configured.
+ */
+void
+tsmux_program_set_scte35_pid (TsMuxProgram * program, guint16 pid)
+{
+  TsMuxSection *section;
+  GstMpegtsSCTESIT *sit;
+  g_return_if_fail (program != NULL);
+
+  program->scte35_pid = pid;
+  /* Create/Update the section */
+  if (program->scte35_null_section) {
+    tsmux_section_free (program->scte35_null_section);
+    program->scte35_null_section = NULL;
+  }
+  if (pid != 0) {
+    program->scte35_null_section = section = g_slice_new0 (TsMuxSection);
+    section->pi.pid = pid;
+    sit = gst_mpegts_scte_null_new ();
+    section->section = gst_mpegts_section_from_scte_sit (sit, pid);
+  }
+}
+
+/**
+ * tsmux_program_get_scte35_pid:
+ * @program: a #TsMuxProgram
+ *
+ * Get the PID configured for sending SCTE-35 packets.
+ *
+ * Returns: the configured SCTE-35 PID, or 0 if not active.
+ */
+guint16
+tsmux_program_get_scte35_pid (TsMuxProgram * program)
+{
+  g_return_val_if_fail (program != NULL, 0);
+
+  return program->scte35_pid;
 }
 
 /**
@@ -480,10 +609,45 @@ tsmux_resend_pmt (TsMuxProgram * program)
 void
 tsmux_program_add_stream (TsMuxProgram * program, TsMuxStream * stream)
 {
+  GPtrArray *streams;
+  guint i;
+  gint pmt_index, array_index = -1 /* append */ ;
+  guint16 pid;
+
   g_return_if_fail (program != NULL);
   g_return_if_fail (stream != NULL);
 
-  g_array_append_val (program->streams, stream);
+  streams = program->streams;
+  pmt_index = stream->pmt_index;
+  pid = tsmux_stream_get_pid (stream);
+
+  if (pmt_index >= 0) {
+    /* Insert into streams with known indices */
+    for (i = 0; i < streams->len; i++) {
+      TsMuxStream *s = g_ptr_array_index (streams, i);
+
+      if (s->pmt_index < 0 || pmt_index < s->pmt_index) {
+        array_index = i;
+        GST_DEBUG ("PID 0x%04x: Using known-order index %d/%u",
+            pid, array_index, streams->len);
+        break;
+      }
+    }
+  } else {
+    /* Insert after streams with known indices, sorted by PID */
+    for (i = 0; i < streams->len; i++) {
+      TsMuxStream *s = g_ptr_array_index (streams, i);
+
+      if (s->pmt_index < 0 && pid < tsmux_stream_get_pid (s)) {
+        array_index = i;
+        GST_DEBUG ("PID 0x%04x: Using PID-order index %d/%u",
+            pid, array_index, streams->len);
+        break;
+      }
+    }
+  }
+
+  g_ptr_array_insert (streams, array_index, stream);
   program->pmt_changed = TRUE;
 }
 
@@ -538,7 +702,7 @@ tsmux_get_new_pid (TsMux * mux)
 /**
  * tsmux_create_stream:
  * @mux: a #TsMux
- * @stream_type: a #TsMuxStreamType
+ * @stream_type: the stream type
  * @pid: the PID of the new stream.
  *
  * Create a new stream of @stream_type in the muxer session @mux.
@@ -549,13 +713,14 @@ tsmux_get_new_pid (TsMux * mux)
  * Returns: a new #TsMuxStream.
  */
 TsMuxStream *
-tsmux_create_stream (TsMux * mux, TsMuxStreamType stream_type, guint16 pid,
+tsmux_create_stream (TsMux * mux, guint stream_type, guint16 pid,
     gchar * language)
 {
   TsMuxStream *stream;
   guint16 new_pid;
 
   g_return_val_if_fail (mux != NULL, NULL);
+  g_return_val_if_fail (mux->new_stream_func != NULL, NULL);
 
   if (pid == TSMUX_PID_AUTO) {
     new_pid = tsmux_get_new_pid (mux);
@@ -567,15 +732,17 @@ tsmux_create_stream (TsMux * mux, TsMuxStreamType stream_type, guint16 pid,
   if (tsmux_find_stream (mux, new_pid))
     return NULL;
 
-  stream = tsmux_stream_new (new_pid, stream_type);
+  stream = mux->new_stream_func (new_pid, stream_type, mux->new_stream_data);
 
   mux->streams = g_list_prepend (mux->streams, stream);
   mux->nb_streams++;
 
-  if (language)
-    g_strlcat (stream->language, language, 3 * sizeof (gchar));
-  else
-    g_strlcat (stream->language, "eng", 3 * sizeof (gchar));
+  if (language) {
+    strncpy (stream->language, language, 4);
+    stream->language[3] = 0;
+  } else {
+    stream->language[0] = 0;
+  }
 
   return stream;
 }
@@ -585,7 +752,7 @@ tsmux_create_stream (TsMux * mux, TsMuxStreamType stream_type, guint16 pid,
  * @mux: a #TsMux
  * @pid: the PID to find.
  *
- * Find the stream associated wih PID.
+ * Find the stream associated with PID.
  *
  * Returns: a #TsMuxStream with @pid or NULL when the stream was not found.
  */
@@ -608,6 +775,45 @@ tsmux_find_stream (TsMux * mux, guint16 pid)
   return found;
 }
 
+static gboolean
+tsmux_program_remove_stream (TsMuxProgram * program, TsMuxStream * stream)
+{
+  GPtrArray *streams = program->streams;
+
+  if (!g_ptr_array_remove (streams, stream)) {
+    g_warn_if_reached ();
+    return FALSE;
+  }
+
+  return streams->len == 0;
+}
+
+
+gboolean
+tsmux_remove_stream (TsMux * mux, guint16 pid, TsMuxProgram * program)
+{
+  GList *cur;
+  gboolean ret = FALSE;
+
+  g_return_val_if_fail (mux != NULL, FALSE);
+
+  for (cur = mux->streams; cur; cur = cur->next) {
+    TsMuxStream *stream = (TsMuxStream *) cur->data;
+
+    if (tsmux_stream_get_pid (stream) == pid) {
+      ret = tsmux_program_remove_stream (program, stream);
+      mux->streams = g_list_remove (mux->streams, stream);
+      tsmux_stream_free (stream);
+      break;
+    }
+  }
+
+  if (ret)
+    tsmux_program_delete (mux, program);
+
+  return ret;
+}
+
 static gboolean
 tsmux_get_buffer (TsMux * mux, GstBuffer ** buf)
 {
@@ -634,7 +840,56 @@ tsmux_packet_out (TsMux * mux, GstBuffer * buf, gint64 pcr)
     return TRUE;
   }
 
+  if (mux->bitrate) {
+    GST_BUFFER_PTS (buf) =
+        gst_util_uint64_scale (mux->n_bytes * 8, GST_SECOND, mux->bitrate);
+
+    /* Check and insert a PCR observation for each program if needed,
+     * but only for programs that have written their SI at least once,
+     * so the stream starts with PAT/PMT */
+    if (mux->first_pcr_ts != G_MININT64) {
+      GList *cur;
+
+      for (cur = mux->programs; cur; cur = cur->next) {
+        TsMuxProgram *program = (TsMuxProgram *) cur->data;
+        TsMuxStream *stream = program->pcr_stream;
+        gint64 cur_pcr, next_pcr, new_pcr;
+
+        if (!program->wrote_si)
+          continue;
+
+        cur_pcr = get_current_pcr (mux, 0);
+        next_pcr = get_next_pcr (mux, 0);
+        new_pcr = write_new_pcr (mux, stream, cur_pcr, next_pcr);
+
+        if (new_pcr != -1) {
+          GstBuffer *buf = NULL;
+          GstMapInfo map;
+          guint payload_len, payload_offs;
+
+          if (!tsmux_get_buffer (mux, &buf)) {
+            goto error;
+          }
+
+          gst_buffer_map (buf, &map, GST_MAP_READ);
+          tsmux_write_ts_header (mux, map.data, &stream->pi, &payload_len,
+              &payload_offs, 0);
+          gst_buffer_unmap (buf, &map);
+
+          stream->pi.flags &= TSMUX_PACKET_FLAG_PES_FULL_HEADER;
+          if (!tsmux_packet_out (mux, buf, new_pcr))
+            goto error;
+        }
+      }
+    }
+  }
+
+  mux->n_bytes += gst_buffer_get_size (buf);
+
   return mux->write_func (buf, mux->write_func_data, pcr);
+
+error:
+  return FALSE;
 }
 
 /*
@@ -710,7 +965,7 @@ tsmux_write_adaptation_field (guint8 * buf,
 
   g_assert (min_length <= TSMUX_PAYLOAD_LENGTH);
 
-  /* Write out all the fields from the packet info only if the 
+  /* Write out all the fields from the packet info only if the
    * user set the flag to request the adaptation field - if the flag
    * isn't set, we're just supposed to write stuffing bytes */
   if (pi->flags & TSMUX_PACKET_FLAG_ADAPTATION) {
@@ -793,11 +1048,11 @@ tsmux_write_adaptation_field (guint8 * buf,
 }
 
 static gboolean
-tsmux_write_ts_header (guint8 * buf, TsMuxPacketInfo * pi,
-    guint * payload_len_out, guint * payload_offset_out)
+tsmux_write_ts_header (TsMux * mux, guint8 * buf, TsMuxPacketInfo * pi,
+    guint * payload_len_out, guint * payload_offset_out, guint stream_avail)
 {
   guint8 *tmp;
-  guint8 adaptation_flag;
+  guint8 adaptation_flag = 0;
   guint8 adapt_min_length = 0;
   guint8 adapt_len = 0;
   guint payload_len;
@@ -807,9 +1062,9 @@ tsmux_write_ts_header (guint8 * buf, TsMuxPacketInfo * pi,
   buf[0] = TSMUX_SYNC_BYTE;
 
   TS_DEBUG ("PID 0x%04x, counter = 0x%01x, %u bytes avail", pi->pid,
-      pi->packet_count & 0x0f, pi->stream_avail);
+      mux->pid_packet_counts[pi->pid] & 0x0f, stream_avail);
 
-  /* 3 bits: 
+  /* 3 bits:
    *   transport_error_indicator
    *   payload_unit_start_indicator
    *   transport_priority: (00)
@@ -825,15 +1080,14 @@ tsmux_write_ts_header (guint8 * buf, TsMuxPacketInfo * pi,
    * 2 bits: adaptation field control (1x has_adaptation_field | x1 has_payload)
    * 4 bits: continuity counter (xxxx)
    */
-  adaptation_flag = pi->packet_count & 0x0f;
 
   if (pi->flags & TSMUX_PACKET_FLAG_ADAPTATION) {
     write_adapt = TRUE;
   }
 
-  if (pi->stream_avail < TSMUX_PAYLOAD_LENGTH) {
+  if (stream_avail < TSMUX_PAYLOAD_LENGTH) {
     /* Need an adaptation field regardless for stuffing */
-    adapt_min_length = TSMUX_PAYLOAD_LENGTH - pi->stream_avail;
+    adapt_min_length = TSMUX_PAYLOAD_LENGTH - stream_avail;
     write_adapt = TRUE;
   }
 
@@ -857,19 +1111,21 @@ tsmux_write_ts_header (guint8 * buf, TsMuxPacketInfo * pi,
   *payload_offset_out = TSMUX_HEADER_LENGTH + adapt_len;
 
   /* Now if we are going to write out some payload, flag that fact */
-  if (payload_len > 0 && pi->stream_avail > 0) {
+  if (payload_len > 0 && stream_avail > 0) {
     /* Flag the presence of a payload */
     adaptation_flag |= 0x10;
 
     /* We must have enough data to fill the payload, or some calculation
      * went wrong */
-    g_assert (payload_len <= pi->stream_avail);
+    g_assert (payload_len <= stream_avail);
 
     /* Packet with payload, increment the continuity counter */
-    pi->packet_count++;
+    mux->pid_packet_counts[pi->pid]++;
   }
 
-  /* Write the byte of transport_scrambling_control, adaptation_field_control 
+  adaptation_flag |= mux->pid_packet_counts[pi->pid] & 0x0f;
+
+  /* Write the byte of transport_scrambling_control, adaptation_field_control
    * + continuity counter out */
   buf[3] = adaptation_flag;
 
@@ -884,8 +1140,9 @@ tsmux_write_ts_header (guint8 * buf, TsMuxPacketInfo * pi,
   return TRUE;
 }
 
+/* The unused_arg is needed for g_hash_table_foreach() */
 static gboolean
-tsmux_section_write_packet (GstMpegtsSectionType * type,
+tsmux_section_write_packet (gpointer unused_arg,
     TsMuxSection * section, TsMux * mux)
 {
   GstBuffer *section_buffer;
@@ -931,7 +1188,8 @@ tsmux_section_write_packet (GstMpegtsSectionType * type,
       /* Wee need room for a pointer byte */
       section->pi.stream_avail++;
 
-      if (!tsmux_write_ts_header (packet, &section->pi, &len, &offset))
+      if (!tsmux_write_ts_header (mux, packet, &section->pi, &len, &offset,
+              section->pi.stream_avail))
         goto fail;
 
       /* Write the pointer byte */
@@ -939,7 +1197,8 @@ tsmux_section_write_packet (GstMpegtsSectionType * type,
       payload_len = len - 1;
 
     } else {
-      if (!tsmux_write_ts_header (packet, &section->pi, &len, &offset))
+      if (!tsmux_write_ts_header (mux, packet, &section->pi, &len, &offset,
+              section->pi.stream_avail))
         goto fail;
       payload_len = len;
     }
@@ -1009,6 +1268,38 @@ fail:
   return FALSE;
 }
 
+/**
+ * tsmux_send_section:
+ * @mux: a #TsMux
+ * @section: (transfer full): a #GstMpegtsSection to add
+ *
+ * Send a @section immediately on the stream.
+ *
+ * Returns: %TRUE on success, %FALSE otherwise
+ */
+gboolean
+tsmux_send_section (TsMux * mux, GstMpegtsSection * section)
+{
+  gboolean ret;
+  TsMuxSection tsmux_section;
+
+  g_return_val_if_fail (mux != NULL, FALSE);
+  g_return_val_if_fail (section != NULL, FALSE);
+
+  memset (&tsmux_section, 0, sizeof (tsmux_section));
+
+  GST_DEBUG ("Sending mpegts section with type %d to mux",
+      section->section_type);
+
+  tsmux_section.section = section;
+  tsmux_section.pi.pid = section->pid;
+
+  ret = tsmux_section_write_packet (NULL, &tsmux_section, mux);
+  gst_mpegts_section_unref (section);
+
+  return ret;
+}
+
 static gboolean
 tsmux_write_si (TsMux * mux)
 {
@@ -1021,6 +1312,267 @@ tsmux_write_si (TsMux * mux)
 
 }
 
+static void
+tsmux_write_null_ts_header (guint8 * buf)
+{
+  *buf++ = TSMUX_SYNC_BYTE;
+  *buf++ = 0x1f;
+  *buf++ = 0xff;
+  *buf++ = 0x10;
+}
+
+static gint64
+ts_to_pcr (gint64 ts)
+{
+  if (ts == G_MININT64) {
+    return 0;
+  }
+
+  return (ts - TSMUX_PCR_OFFSET) * (TSMUX_SYS_CLOCK_FREQ / TSMUX_CLOCK_FREQ);
+}
+
+/* Calculate the PCR to write into the current packet */
+static gint64
+get_current_pcr (TsMux * mux, gint64 cur_ts)
+{
+  if (!mux->bitrate)
+    return ts_to_pcr (cur_ts);
+
+  if (mux->first_pcr_ts == G_MININT64) {
+    g_assert (cur_ts != G_MININT64);
+    mux->first_pcr_ts = cur_ts;
+    GST_DEBUG ("First PCR offset is %" G_GUINT64_FORMAT, cur_ts);
+  }
+
+  return ts_to_pcr (mux->first_pcr_ts) +
+      gst_util_uint64_scale ((mux->n_bytes + PCR_BYTE_OFFSET) * 8,
+      TSMUX_SYS_CLOCK_FREQ, mux->bitrate);
+}
+
+/* Predict the PCR at the next packet if possible */
+static gint64
+get_next_pcr (TsMux * mux, gint64 cur_ts)
+{
+  if (!mux->bitrate)
+    return ts_to_pcr (cur_ts);
+
+  if (mux->first_pcr_ts == G_MININT64) {
+    g_assert (cur_ts != G_MININT64);
+    mux->first_pcr_ts = cur_ts;
+    GST_DEBUG ("First PCR offset is %" G_GUINT64_FORMAT, cur_ts);
+  }
+
+  return ts_to_pcr (mux->first_pcr_ts) +
+      gst_util_uint64_scale ((mux->n_bytes + TSMUX_PACKET_LENGTH +
+          PCR_BYTE_OFFSET) * 8, TSMUX_SYS_CLOCK_FREQ, mux->bitrate);
+}
+
+static gint64
+write_new_pcr (TsMux * mux, TsMuxStream * stream, gint64 cur_pcr,
+    gint64 next_pcr)
+{
+  if (stream->next_pcr == -1 || next_pcr > stream->next_pcr) {
+    stream->pi.flags |=
+        TSMUX_PACKET_FLAG_ADAPTATION | TSMUX_PACKET_FLAG_WRITE_PCR;
+    stream->pi.pcr = cur_pcr;
+
+    if (mux->bitrate && stream->next_pcr != -1 && cur_pcr >= stream->next_pcr) {
+      GST_WARNING ("Writing PCR %" G_GUINT64_FORMAT " missed the target %"
+          G_GUINT64_FORMAT " by %f ms", cur_pcr, stream->next_pcr,
+          (double) (cur_pcr - stream->next_pcr) / 27000.0);
+    }
+    /* Next PCR deadline is now plus the scheduled interval */
+    stream->next_pcr = cur_pcr + mux->pcr_interval * 300;
+  } else {
+    cur_pcr = -1;
+  }
+
+  return cur_pcr;
+}
+
+static gboolean
+rewrite_si (TsMux * mux, gint64 cur_ts)
+{
+  gboolean write_pat;
+  gboolean write_si;
+  GList *cur;
+  gint64 next_pcr;
+
+  next_pcr = get_next_pcr (mux, cur_ts);
+
+  /* check if we need to rewrite pat */
+  if (mux->next_pat_pcr == -1 || mux->pat_changed)
+    write_pat = TRUE;
+  else if (next_pcr > mux->next_pat_pcr)
+    write_pat = TRUE;
+  else
+    write_pat = FALSE;
+
+  if (write_pat) {
+    if (mux->next_pat_pcr == -1)
+      mux->next_pat_pcr = next_pcr + mux->pat_interval * 300;
+    else
+      mux->next_pat_pcr += mux->pat_interval * 300;
+
+    if (!tsmux_write_pat (mux))
+      return FALSE;
+
+    next_pcr = get_next_pcr (mux, cur_ts);
+  }
+
+  /* check if we need to rewrite sit */
+  if (mux->next_si_pcr == -1 || mux->si_changed)
+    write_si = TRUE;
+  else if (next_pcr > mux->next_si_pcr)
+    write_si = TRUE;
+  else
+    write_si = FALSE;
+
+  if (write_si) {
+    if (mux->next_si_pcr == -1)
+      mux->next_si_pcr = next_pcr + mux->si_interval * 300;
+    else
+      mux->next_si_pcr += mux->si_interval * 300;
+
+    if (!tsmux_write_si (mux))
+      return FALSE;
+
+    next_pcr = get_current_pcr (mux, cur_ts);
+  }
+
+  /* check if we need to rewrite any of the current pmts */
+  for (cur = mux->programs; cur; cur = cur->next) {
+    TsMuxProgram *program = (TsMuxProgram *) cur->data;
+    gboolean write_pmt;
+
+    if (program->next_pmt_pcr == -1 || program->pmt_changed)
+      write_pmt = TRUE;
+    else if (next_pcr > program->next_pmt_pcr)
+      write_pmt = TRUE;
+    else
+      write_pmt = FALSE;
+
+    if (write_pmt) {
+      if (program->next_pmt_pcr == -1)
+        program->next_pmt_pcr = next_pcr + program->pmt_interval * 300;
+      else
+        program->next_pmt_pcr += program->pmt_interval * 300;
+
+      if (!tsmux_write_pmt (mux, program))
+        return FALSE;
+
+      next_pcr = get_current_pcr (mux, cur_ts);
+    }
+
+    if (program->scte35_pid != 0) {
+      gboolean write_scte_null = FALSE;
+      if (program->next_scte35_pcr == -1)
+        write_scte_null = TRUE;
+      else if (next_pcr > program->next_scte35_pcr)
+        write_scte_null = TRUE;
+
+      if (write_scte_null) {
+        GST_DEBUG ("next scte35 pcr %" G_GINT64_FORMAT,
+            program->next_scte35_pcr);
+        if (program->next_scte35_pcr == -1)
+          program->next_scte35_pcr =
+              next_pcr + program->scte35_null_interval * 300;
+        else
+          program->next_scte35_pcr += program->scte35_null_interval * 300;
+        GST_DEBUG ("next scte35 NOW pcr %" G_GINT64_FORMAT,
+            program->next_scte35_pcr);
+
+        if (!tsmux_write_scte_null (mux, program))
+          return FALSE;
+
+        next_pcr = get_current_pcr (mux, cur_ts);
+      }
+    }
+
+    program->wrote_si = TRUE;
+  }
+
+  return TRUE;
+}
+
+static gboolean
+pad_stream (TsMux * mux, TsMuxStream * stream, gint64 cur_ts)
+{
+  guint64 bitrate;
+  GstBuffer *buf = NULL;
+  GstMapInfo map;
+  gboolean ret = TRUE;
+  GstClockTimeDiff diff;
+  guint64 start_n_bytes;
+
+  if (!mux->bitrate)
+    goto done;
+
+  if (!GST_CLOCK_STIME_IS_VALID (cur_ts))
+    goto done;
+
+  if (!GST_CLOCK_STIME_IS_VALID (stream->first_ts))
+    stream->first_ts = cur_ts;
+
+  diff = GST_CLOCK_DIFF (stream->first_ts, cur_ts);
+  if (diff == 0)
+    goto done;
+
+  start_n_bytes = mux->n_bytes;
+  do {
+    GST_LOG ("Transport stream bitrate: %" G_GUINT64_FORMAT " over %"
+        G_GUINT64_FORMAT " bytes, duration %" GST_TIME_FORMAT,
+        gst_util_uint64_scale (mux->n_bytes * 8, TSMUX_CLOCK_FREQ, diff),
+        mux->n_bytes, GST_TIME_ARGS (diff * GST_SECOND / TSMUX_CLOCK_FREQ));
+
+    /* calculate what the overall bitrate will be if we add 1 more packet */
+    bitrate =
+        gst_util_uint64_scale ((mux->n_bytes + TSMUX_PACKET_LENGTH) * 8,
+        TSMUX_CLOCK_FREQ, diff);
+
+    if (bitrate <= mux->bitrate) {
+      gint64 new_pcr;
+      guint payload_len, payload_offs;
+
+      if (!tsmux_get_buffer (mux, &buf)) {
+        ret = FALSE;
+        goto done;
+      }
+
+      gst_buffer_map (buf, &map, GST_MAP_READ);
+
+      if ((new_pcr =
+              write_new_pcr (mux, stream, get_current_pcr (mux,
+                      cur_ts), get_next_pcr (mux, cur_ts)) != -1)) {
+        GST_LOG ("Writing PCR-only packet on PID 0x%04x", stream->pi.pid);
+        tsmux_write_ts_header (mux, map.data, &stream->pi, &payload_len,
+            &payload_offs, 0);
+      } else {
+        GST_LOG ("Writing null stuffing packet");
+        if (!rewrite_si (mux, cur_ts)) {
+          ret = FALSE;
+          goto done;
+        }
+        tsmux_write_null_ts_header (map.data);
+      }
+
+      gst_buffer_unmap (buf, &map);
+
+      stream->pi.flags &= TSMUX_PACKET_FLAG_PES_FULL_HEADER;
+
+      if (!(ret = tsmux_packet_out (mux, buf, new_pcr)))
+        goto done;
+    }
+  } while (bitrate < mux->bitrate);
+
+  if (mux->n_bytes != start_n_bytes) {
+    GST_LOG ("Finished padding the mux");
+  }
+
+done:
+  return ret;
+}
+
 /**
  * tsmux_write_stream_packet:
  * @mux: a #TsMux
@@ -1036,7 +1588,7 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
   guint payload_len, payload_offs;
   TsMuxPacketInfo *pi = &stream->pi;
   gboolean res;
-  gint64 cur_pcr = -1;
+  gint64 new_pcr = -1;
   GstBuffer *buf = NULL;
   GstMapInfo map;
 
@@ -1044,84 +1596,21 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
   g_return_val_if_fail (stream != NULL, FALSE);
 
   if (tsmux_stream_is_pcr (stream)) {
-    gint64 cur_pts = tsmux_stream_get_pts (stream);
-    gboolean write_pat;
-    gboolean write_si;
-    GList *cur;
-
-    cur_pcr = 0;
-    if (cur_pts != G_MININT64) {
-      TS_DEBUG ("TS for PCR stream is %" G_GINT64_FORMAT, cur_pts);
-    }
-
-    /* FIXME: The current PCR needs more careful calculation than just
-     * writing a fixed offset */
-    if (cur_pts != G_MININT64) {
-      /* CLOCK_BASE >= TSMUX_PCR_OFFSET */
-      cur_pts += CLOCK_BASE;
-      cur_pcr = (cur_pts - TSMUX_PCR_OFFSET) *
-          (TSMUX_SYS_CLOCK_FREQ / TSMUX_CLOCK_FREQ);
-    }
-
-    /* Need to decide whether to write a new PCR in this packet */
-    if (stream->last_pcr == -1 ||
-        (cur_pcr - stream->last_pcr >
-            (TSMUX_SYS_CLOCK_FREQ / TSMUX_DEFAULT_PCR_FREQ))) {
-
-      stream->pi.flags |=
-          TSMUX_PACKET_FLAG_ADAPTATION | TSMUX_PACKET_FLAG_WRITE_PCR;
-      stream->pi.pcr = cur_pcr;
-      stream->last_pcr = cur_pcr;
-    } else {
-      cur_pcr = -1;
-    }
-
-    /* check if we need to rewrite pat */
-    if (mux->last_pat_ts == G_MININT64 || mux->pat_changed)
-      write_pat = TRUE;
-    else if (cur_pts >= mux->last_pat_ts + mux->pat_interval)
-      write_pat = TRUE;
+    gint64 cur_ts = CLOCK_BASE;
+    if (tsmux_stream_get_dts (stream) != G_MININT64)
+      cur_ts += tsmux_stream_get_dts (stream);
     else
-      write_pat = FALSE;
+      cur_ts += tsmux_stream_get_pts (stream);
 
-    if (write_pat) {
-      mux->last_pat_ts = cur_pts;
-      if (!tsmux_write_pat (mux))
-        return FALSE;
-    }
+    if (!rewrite_si (mux, cur_ts))
+      goto fail;
 
-    /* check if we need to rewrite sit */
-    if (mux->last_si_ts == G_MININT64 || mux->si_changed)
-      write_si = TRUE;
-    else if (cur_pts >= mux->last_si_ts + mux->si_interval)
-      write_si = TRUE;
-    else
-      write_si = FALSE;
+    if (!pad_stream (mux, stream, cur_ts))
+      goto fail;
 
-    if (write_si) {
-      mux->last_si_ts = cur_pts;
-      if (!tsmux_write_si (mux))
-        return FALSE;
-    }
-
-    /* check if we need to rewrite any of the current pmts */
-    for (cur = mux->programs; cur; cur = cur->next) {
-      TsMuxProgram *program = (TsMuxProgram *) cur->data;
-      gboolean write_pmt;
-
-      if (program->last_pmt_ts == G_MININT64 || program->pmt_changed)
-        write_pmt = TRUE;
-      else if (cur_pts >= program->last_pmt_ts + program->pmt_interval)
-        write_pmt = TRUE;
-      else
-        write_pmt = FALSE;
-
-      if (write_pmt) {
-        program->last_pmt_ts = cur_pts;
-        if (!tsmux_write_pmt (mux, program))
-          return FALSE;
-      }
-    }
+    new_pcr =
+        write_new_pcr (mux, stream, get_current_pcr (mux, cur_ts),
+        get_next_pcr (mux, cur_ts));
   }
 
   pi->packet_start_unit_indicator = tsmux_stream_at_pes_start (stream);
@@ -1140,7 +1629,8 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
 
   gst_buffer_map (buf, &map, GST_MAP_READ);
 
-  if (!tsmux_write_ts_header (map.data, pi, &payload_len, &payload_offs))
+  if (!tsmux_write_ts_header (mux, map.data, pi, &payload_len, &payload_offs,
+          pi->stream_avail))
     goto fail;
 
 
@@ -1150,7 +1640,7 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
   gst_buffer_unmap (buf, &map);
 
   GST_DEBUG ("Writing PES of size %d", (int) gst_buffer_get_size (buf));
-  res = tsmux_packet_out (mux, buf, cur_pcr);
+  res = tsmux_packet_out (mux, buf, new_pcr);
 
   /* Reset all dynamic flags */
   stream->pi.flags &= TSMUX_PACKET_FLAG_PES_FULL_HEADER;
@@ -1160,9 +1650,10 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
   /* ERRORS */
 fail:
   {
-    gst_buffer_unmap (buf, &map);
-    if (buf)
+    if (buf) {
+      gst_buffer_unmap (buf, &map);
       gst_buffer_unref (buf);
+    }
     return FALSE;
   }
 }
@@ -1182,11 +1673,34 @@ tsmux_program_free (TsMuxProgram * program)
   /* Free PMT section */
   if (program->pmt.section)
     gst_mpegts_section_unref (program->pmt.section);
+  if (program->scte35_null_section)
+    tsmux_section_free (program->scte35_null_section);
 
-  g_array_free (program->streams, TRUE);
+  g_ptr_array_free (program->streams, TRUE);
   g_slice_free (TsMuxProgram, program);
 }
 
+/**
+ * tsmux_program_set_pmt_pid:
+ * @program: A #TsmuxProgram
+ * @pmt_pid: PID to write PMT for this program
+ */
+void
+tsmux_program_set_pmt_pid (TsMuxProgram * program, guint16 pmt_pid)
+{
+  program->pmt_pid = pmt_pid;
+}
+
+static gint
+compare_program_number (gconstpointer a, gconstpointer b)
+{
+  const GstMpegtsPatProgram *pgm1 = *(const GstMpegtsPatProgram * const *) a;
+  const GstMpegtsPatProgram *pgm2 = *(const GstMpegtsPatProgram * const *) b;
+  gint num1 = pgm1->program_number, num2 = pgm2->program_number;
+
+  return num1 - num2;
+}
+
 static gboolean
 tsmux_write_pat (TsMux * mux)
 {
@@ -1216,6 +1730,8 @@ tsmux_write_pat (TsMux * mux)
       g_ptr_array_add (pat, pat_pgm);
     }
 
+    g_ptr_array_sort (pat, compare_program_number);
+
     if (mux->pat.section)
       gst_mpegts_section_unref (mux->pat.section);
 
@@ -1227,8 +1743,7 @@ tsmux_write_pat (TsMux * mux)
     mux->pat_changed = FALSE;
   }
 
-  return tsmux_section_write_packet (GINT_TO_POINTER (GST_MPEGTS_SECTION_PAT),
-      &mux->pat, mux);
+  return tsmux_section_write_packet (NULL, &mux->pat, mux);
 }
 
 static gboolean
@@ -1257,7 +1772,10 @@ tsmux_write_pmt (TsMux * mux, TsMuxProgram * program)
      */
     GstMpegtsDescriptor *descriptor;
     GstMpegtsPMT *pmt;
+#if 0
+    /* See note about bluray descriptors below */
     guint8 desc[] = { 0x0F, 0xFF, 0xFC, 0xFC };
+#endif
     guint i;
 
     pmt = gst_mpegts_pmt_new ();
@@ -1267,16 +1785,29 @@ tsmux_write_pmt (TsMux * mux, TsMuxProgram * program)
     else
       pmt->pcr_pid = tsmux_stream_get_pid (program->pcr_stream);
 
+#if 0
+    /* FIXME : These two descriptors should not be added in all PMT
+     * but only in "bluray-compatible" mpeg-ts output. I even have my
+     * doubt whether the DTCP descriptor is even needed */
     descriptor = gst_mpegts_descriptor_from_registration ("HDMV", NULL, 0);
     g_ptr_array_add (pmt->descriptors, descriptor);
 
+    /* DTCP descriptor, see
+     * http://www.dtcp.com/documents/dtcp/info-20150204-dtcp-v1-rev%201-71.pdf */
     descriptor = gst_mpegts_descriptor_from_custom (0x88, desc, 4);
     g_ptr_array_add (pmt->descriptors, descriptor);
+#endif
+
+    /* Will SCTE-35 be potentially used ? */
+    if (program->scte35_pid != 0) {
+      descriptor = gst_mpegts_descriptor_from_registration ("CUEI", NULL, 0);
+      g_ptr_array_add (pmt->descriptors, descriptor);
+    }
 
     /* Write out the entries */
     for (i = 0; i < program->streams->len; i++) {
       GstMpegtsPMTStream *pmt_stream;
-      TsMuxStream *stream = g_array_index (program->streams, TsMuxStream *, i);
+      TsMuxStream *stream = g_ptr_array_index (program->streams, i);
 
       pmt_stream = gst_mpegts_pmt_stream_new ();
 
@@ -1289,6 +1820,14 @@ tsmux_write_pmt (TsMux * mux, TsMuxProgram * program)
       g_ptr_array_add (pmt->streams, pmt_stream);
     }
 
+    /* Will SCTE-35 be potentially used ? */
+    if (program->scte35_pid != 0) {
+      GstMpegtsPMTStream *pmt_stream = gst_mpegts_pmt_stream_new ();
+      pmt_stream->stream_type = GST_MPEGTS_STREAM_TYPE_SCTE_SIT;
+      pmt_stream->pid = program->scte35_pid;
+      g_ptr_array_add (pmt->streams, pmt_stream);
+    }
+
     TS_DEBUG ("PMT for program %d has %d streams",
         program->pgm_number, program->streams->len);
 
@@ -1304,6 +1843,19 @@ tsmux_write_pmt (TsMux * mux, TsMuxProgram * program)
     program->pmt.section->version_number = program->pmt_version++;
   }
 
-  return tsmux_section_write_packet (GINT_TO_POINTER (GST_MPEGTS_SECTION_PMT),
-      &program->pmt, mux);
+  return tsmux_section_write_packet (NULL, &program->pmt, mux);
+}
+
+static gboolean
+tsmux_write_scte_null (TsMux * mux, TsMuxProgram * program)
+{
+  /* SCTE-35 NULL section is created when PID is set */
+  GST_LOG ("Writing SCTE NULL packet");
+  return tsmux_section_write_packet (NULL, program->scte35_null_section, mux);
+}
+
+void
+tsmux_set_bitrate (TsMux * mux, guint64 bitrate)
+{
+  mux->bitrate = bitrate;
 }
