@@ -23,6 +23,7 @@
 #endif
 
 #include "gsttranscoding.h"
+#include "gsttranscodeelements.h"
 #if HAVE_GETRUSAGE
 #include "gst-cpu-throttling-clock.h"
 #endif
@@ -73,7 +74,10 @@ typedef struct
 
 #define DEFAULT_AVOID_REENCODING   FALSE
 
-G_DEFINE_TYPE (GstUriTranscodeBin, gst_uri_transcode_bin, GST_TYPE_PIPELINE)
+G_DEFINE_TYPE (GstUriTranscodeBin, gst_uri_transcode_bin, GST_TYPE_PIPELINE);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (uritranscodebin, "uritranscodebin", GST_RANK_NONE,
+    gst_uri_transcode_bin_get_type (), transcodebin_element_init (plugin));
+
 enum
 {
  PROP_0,
@@ -89,6 +93,15 @@ enum
  LAST_PROP
 };
 
+/* signals */
+enum
+{
+  SIGNAL_SOURCE_SETUP,
+  SIGNAL_ELEMENT_SETUP,
+  LAST_SIGNAL
+};
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void
 post_missing_plugin_error (GstElement * dec, const gchar * element_name)
 {
@@ -103,66 +116,59 @@ post_missing_plugin_error (GstElement * dec, const gchar * element_name)
 }
 /* *INDENT-ON* */
 
-static gboolean
-make_transcodebin (GstUriTranscodeBin * self)
-{
-  GST_INFO_OBJECT (self, "making new transcodebin");
-
-  self->transcodebin = gst_element_factory_make ("transcodebin", NULL);
-  if (!self->transcodebin)
-    goto no_decodebin;
-
-  g_object_set (self->transcodebin, "profile", self->profile,
-      "video-filter", self->video_filter,
-      "audio-filter", self->audio_filter,
-      "avoid-reencoding", self->avoid_reencoding, NULL);
-
-  gst_bin_add (GST_BIN (self), self->transcodebin);
-  if (!gst_element_link (self->transcodebin, self->sink))
-    return FALSE;
-
-  return TRUE;
-
-  /* ERRORS */
-no_decodebin:
-  {
-    post_missing_plugin_error (GST_ELEMENT_CAST (self), "transcodebin");
-
-    GST_ELEMENT_ERROR (self, CORE, MISSING_PLUGIN, (NULL),
-        ("No transcodebin element, check your installation"));
-
-    return FALSE;
-  }
-}
-
 static gboolean
 make_dest (GstUriTranscodeBin * self)
 {
   GError *err = NULL;
 
+  GST_OBJECT_LOCK (self);
+  if (!self->dest_uri) {
+    GST_INFO_OBJECT (self, "Sink already set: %" GST_PTR_FORMAT, self->sink);
+    goto ok_unlock;
+  }
+
+  if (!self->dest_uri)
+    goto ok_unlock;
+
   if (!gst_uri_is_valid (self->dest_uri))
-    goto invalid_uri;
+    goto invalid_uri_unlock;
 
   self->sink = gst_element_make_from_uri (GST_URI_SINK, self->dest_uri,
       "sink", &err);
   if (!self->sink)
-    goto no_sink;
+    goto no_sink_unlock;
 
+  GST_OBJECT_UNLOCK (self);
   gst_bin_add (GST_BIN (self), self->sink);
   g_object_set (self->sink, "sync", TRUE, "max-lateness", GST_CLOCK_TIME_NONE,
       NULL);
+
   return TRUE;
 
-invalid_uri:
+ok_unlock:
+  GST_OBJECT_UNLOCK (self);
+  return TRUE;
+
+invalid_uri_unlock:
   {
+    GST_OBJECT_UNLOCK (self);
     GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
         ("Invalid URI \"%s\".", self->dest_uri), (NULL));
     g_clear_error (&err);
     return FALSE;
   }
 
-no_sink:
+invalid_uri:
   {
+    GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
+        ("Invalid URI \"%s\".", self->source_uri), (NULL));
+    g_clear_error (&err);
+    return FALSE;
+  }
+
+no_sink_unlock:
+  {
+    GST_OBJECT_UNLOCK (self);
     /* whoops, could not create the source element, dig a little deeper to
      * figure out what might be wrong. */
     if (err != NULL && err->code == GST_URI_ERROR_UNSUPPORTED_PROTOCOL) {
@@ -191,6 +197,117 @@ no_sink:
   }
 }
 
+static void
+transcodebin_pad_added_cb (GstElement * transcodebin, GstPad * pad,
+    GstUriTranscodeBin * self)
+{
+
+  GstPad *sinkpad;
+
+  if (GST_PAD_IS_SINK (pad))
+    return;
+
+  make_dest (self);
+  if (!self->sink) {
+    GST_ELEMENT_ERROR (self, CORE, FAILED, (NULL), ("No sink configured."));
+    return;
+  }
+
+  sinkpad = gst_element_get_static_pad (self->sink, "sink");
+  if (!sinkpad) {
+    GST_ELEMENT_ERROR (self, CORE, FAILED, (NULL), ("Sink has not sinkpad?!"));
+    return;
+  }
+
+  if (gst_pad_link (pad, sinkpad) != GST_PAD_LINK_OK) {
+    GST_ERROR_OBJECT (self,
+        "Could not link %" GST_PTR_FORMAT " and %" GST_PTR_FORMAT, pad,
+        sinkpad);
+    /* Let `pad unlinked` error pop up later */
+  }
+  gst_object_unref (sinkpad);
+}
+
+static gboolean
+make_transcodebin (GstUriTranscodeBin * self)
+{
+  GST_INFO_OBJECT (self, "making new transcodebin");
+
+  self->transcodebin = gst_element_factory_make ("transcodebin", NULL);
+  if (!self->transcodebin)
+    goto no_transcodebin;
+
+  g_signal_connect (self->transcodebin, "pad-added",
+      G_CALLBACK (transcodebin_pad_added_cb), self);
+
+  g_object_set (self->transcodebin, "profile", self->profile,
+      "video-filter", self->video_filter,
+      "audio-filter", self->audio_filter,
+      "avoid-reencoding", self->avoid_reencoding, NULL);
+
+  gst_bin_add (GST_BIN (self), self->transcodebin);
+
+  return TRUE;
+
+  /* ERRORS */
+no_transcodebin:
+  {
+    post_missing_plugin_error (GST_ELEMENT_CAST (self), "transcodebin");
+
+    GST_ELEMENT_ERROR (self, CORE, MISSING_PLUGIN, (NULL),
+        ("No transcodebin element, check your installation"));
+
+    return FALSE;
+  }
+}
+
+static void
+src_pad_added_cb (GstElement * src, GstPad * pad, GstUriTranscodeBin * self)
+{
+  GstPad *sinkpad = NULL;
+  GstPadLinkReturn res;
+
+  GST_DEBUG_OBJECT (self,
+      "New pad %" GST_PTR_FORMAT " from source %" GST_PTR_FORMAT, pad, src);
+
+  sinkpad = gst_element_get_static_pad (self->transcodebin, "sink");
+
+  if (gst_pad_is_linked (sinkpad))
+    sinkpad = gst_element_request_pad_simple (self->transcodebin, "sink_%u");
+
+  if (sinkpad) {
+    GST_DEBUG_OBJECT (self,
+        "Linking %" GST_PTR_FORMAT " to %" GST_PTR_FORMAT, pad, sinkpad);
+    res = gst_pad_link (pad, sinkpad);
+    gst_object_unref (sinkpad);
+    if (GST_PAD_LINK_FAILED (res))
+      goto link_failed;
+  }
+  return;
+
+link_failed:
+  {
+    GST_ERROR_OBJECT (self,
+        "failed to link pad %s:%s to decodebin, reason %s (%d)",
+        GST_DEBUG_PAD_NAME (pad), gst_pad_link_get_name (res), res);
+    return;
+  }
+}
+
+static void
+src_pad_removed_cb (GstElement * element, GstPad * pad,
+    GstUriTranscodeBin * self)
+{
+  /* FIXME : IMPLEMENT */
+}
+
+static void
+source_setup_cb (GstElement * element, GstElement * source,
+    GstUriTranscodeBin * self)
+{
+  g_signal_emit (self, signals[SIGNAL_SOURCE_SETUP], 0, source);
+}
+
 static gboolean
 make_source (GstUriTranscodeBin * self)
 {
@@ -199,15 +316,19 @@ make_source (GstUriTranscodeBin * self)
   if (!gst_uri_is_valid (self->source_uri))
     goto invalid_uri;
 
-  self->src = gst_element_make_from_uri (GST_URI_SRC, self->source_uri,
-      "src", &err);
+  self->src = gst_element_factory_make ("urisourcebin", NULL);
   if (!self->src)
-    goto no_sink;
+    goto no_urisourcebin;
 
   gst_bin_add (GST_BIN (self), self->src);
 
-  if (!gst_element_link (self->src, self->transcodebin))
-    return FALSE;
+  g_object_set (self->src, "uri", self->source_uri, NULL);
+
+  g_signal_connect (self->src, "pad-added", (GCallback) src_pad_added_cb, self);
+  g_signal_connect (self->src, "pad-removed",
+      (GCallback) src_pad_removed_cb, self);
+  g_signal_connect (self->src, "source-setup",
+      G_CALLBACK (source_setup_cb), self);
 
   return TRUE;
 
@@ -219,34 +340,16 @@ invalid_uri:
     return FALSE;
   }
 
-no_sink:
+no_urisourcebin:
   {
-    /* whoops, could not create the source element, dig a little deeper to
-     * figure out what might be wrong. */
-    if (err != NULL && err->code == GST_URI_ERROR_UNSUPPORTED_PROTOCOL) {
-      gchar *prot;
+    post_missing_plugin_error (GST_ELEMENT_CAST (self), "urisourcebin");
 
-      prot = gst_uri_get_protocol (self->source_uri);
-      if (prot == NULL)
-        goto invalid_uri;
-
-      gst_element_post_message (GST_ELEMENT_CAST (self),
-          gst_missing_uri_source_message_new (GST_ELEMENT (self), prot));
-
-      GST_ELEMENT_ERROR (self, CORE, MISSING_PLUGIN,
-          ("No URI handler implemented for \"%s\".", prot), (NULL));
-
-      g_free (prot);
-    } else {
-      GST_ELEMENT_ERROR (self, RESOURCE, NOT_FOUND,
-          ("%s", (err) ? err->message : "URI was not accepted by any element"),
-          ("No element accepted URI '%s'", self->dest_uri));
-    }
-
-    g_clear_error (&err);
+    GST_ELEMENT_ERROR (self, CORE, MISSING_PLUGIN, (NULL),
+        ("No urisourcebin element, check your installation"));
 
     return FALSE;
   }
+
 }
 
 static void
@@ -271,6 +374,53 @@ remove_all_children (GstUriTranscodeBin * self)
   }
 }
 
+static void
+set_location_on_muxer_if_sink (GstUriTranscodeBin * self, GstElement * child)
+{
+  GstElementFactory *factory = gst_element_get_factory (child);
+
+  if (!factory)
+    return;
+
+  if (!self->dest_uri)
+    return;
+
+  /* Set out dest URI as location for muxer sinks. */
+  if (!gst_element_factory_list_is_type (factory,
+          GST_ELEMENT_FACTORY_TYPE_MUXER) ||
+      !gst_element_factory_list_is_type (factory,
+          GST_ELEMENT_FACTORY_TYPE_SINK)) {
+
+    return;
+  }
+
+  if (!g_object_class_find_property (G_OBJECT_GET_CLASS (child), "location"))
+    return;
+
+  if (!gst_uri_has_protocol (self->dest_uri, "file")) {
+    GST_ELEMENT_ERROR (self, RESOURCE, SETTINGS,
+        ("Trying to use a not local file with a muxing sink which is not"
+            " supported."), (NULL));
+    return;
+  }
+
+  GST_OBJECT_FLAG_SET (self->transcodebin, GST_ELEMENT_FLAG_SINK);
+  g_object_set (child, "location", &self->dest_uri[strlen ("file://")], NULL);
+  GST_DEBUG_OBJECT (self, "Setting location: %s",
+      &self->dest_uri[strlen ("file://")]);
+}
+
+static void
+deep_element_added (GstBin * bin, GstBin * sub_bin, GstElement * child)
+{
+  GstUriTranscodeBin *self = GST_URI_TRANSCODE_BIN (bin);
+
+  set_location_on_muxer_if_sink (self, child);
+  g_signal_emit (bin, signals[SIGNAL_ELEMENT_SETUP], 0, child);
+
+  GST_BIN_CLASS (parent_class)->deep_element_added (bin, sub_bin, child);
+}
+
 static GstStateChangeReturn
 gst_uri_transcode_bin_change_state (GstElement * element,
     GstStateChange transition)
@@ -281,16 +431,13 @@ gst_uri_transcode_bin_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
 
-      if (!make_dest (self))
-        goto setup_failed;
-
       if (!make_transcodebin (self))
         goto setup_failed;
 
       if (!make_source (self))
         goto setup_failed;
 
-      if (gst_element_set_state (self->sink,
+      if (self->sink && gst_element_set_state (self->sink,
               GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
         GST_ERROR_OBJECT (self,
             "Could not set %" GST_PTR_FORMAT " state to PAUSED", self->sink);
@@ -360,6 +507,9 @@ gst_uri_transcode_bin_dispose (GObject * object)
   g_clear_object (&self->video_filter);
   g_clear_object (&self->audio_filter);
   g_clear_object (&self->cpu_clock);
+  g_free (self->source_uri);
+  g_free (self->dest_uri);
+  gst_clear_object (&self->profile);
 
   G_OBJECT_CLASS (gst_uri_transcode_bin_parent_class)->dispose (object);
 }
@@ -453,11 +603,13 @@ gst_uri_transcode_bin_set_property (GObject * object,
       break;
     case PROP_AUDIO_FILTER:
       GST_OBJECT_LOCK (self);
+      gst_object_unref (self->audio_filter);
       self->audio_filter = g_value_dup_object (value);
       GST_OBJECT_UNLOCK (self);
       break;
     case PROP_VIDEO_FILTER:
       GST_OBJECT_LOCK (self);
+      gst_object_unref (self->video_filter);
       self->video_filter = g_value_dup_object (value);
       GST_OBJECT_UNLOCK (self);
       break;
@@ -471,6 +623,7 @@ gst_uri_transcode_bin_class_init (GstUriTranscodeBinClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_klass;
+  GstBinClass *gstbin_klass;
 
   object_class->get_property = gst_uri_transcode_bin_get_property;
   object_class->set_property = gst_uri_transcode_bin_set_property;
@@ -481,6 +634,9 @@ gst_uri_transcode_bin_class_init (GstUriTranscodeBinClass * klass)
   gstelement_klass->change_state =
       GST_DEBUG_FUNCPTR (gst_uri_transcode_bin_change_state);
 
+  gstbin_klass = (GstBinClass *) klass;
+  gstbin_klass->deep_element_added = GST_DEBUG_FUNCPTR (deep_element_added);
+
   GST_DEBUG_CATEGORY_INIT (gst_uri_transcodebin_debug, "uritranscodebin", 0,
       "UriTranscodebin element");
 
@@ -553,6 +709,46 @@ gst_uri_transcode_bin_class_init (GstUriTranscodeBinClass * klass)
       g_param_spec_object ("audio-filter", "Audio filter",
           "the audio filter(s) to apply, if possible",
           GST_TYPE_ELEMENT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstUriTranscodeBin::source-setup:
+   * @uritranscodebin: a #GstUriTranscodeBin
+   * @source: source element
+   *
+   * This signal is emitted after the source element has been created, so
+   * it can be configured by setting additional properties (e.g. set a
+   * proxy server for an http source, or set the device and read speed for
+   * an audio cd source). This is functionally equivalent to connecting to
+   * the notify::source signal, but more convenient.
+   *
+   * This signal is usually emitted from the context of a GStreamer streaming
+   * thread.
+   *
+   * Since: 1.20
+   */
+  signals[SIGNAL_SOURCE_SETUP] =
+      g_signal_new ("source-setup", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, GST_TYPE_ELEMENT);
+
+  /**
+   * GstUriTranscodeBin::element-setup:
+   * @uritranscodebin: a #GstUriTranscodeBin
+   * @element: an element that was added to the uritranscodebin hierarchy
+   *
+   * This signal is emitted when a new element is added to uritranscodebin or any of
+   * its sub-bins. This signal can be used to configure elements, e.g. to set
+   * properties on decoders. This is functionally equivalent to connecting to
+   * the deep-element-added signal, but more convenient.
+   *
+   * This signal is usually emitted from the context of a GStreamer streaming
+   * thread, so might be called at the same time as code running in the main
+   * application thread.
+   *
+   * Since: 1.20
+   */
+  signals[SIGNAL_ELEMENT_SETUP] =
+      g_signal_new ("element-setup", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, GST_TYPE_ELEMENT);
 }
 
 static void
