@@ -798,13 +798,16 @@ gst_amc_color_format_copy (GstAmcColorFormatInfo * cinfo,
 {
   gboolean ret = FALSE;
   guint8 *cptr = NULL, *vptr = NULL;
+  GstMapFlags vmap_mode;
   guint8 **src, **dest;
 
   if (direction == COLOR_FORMAT_COPY_OUT) {
     src = &cptr;
     dest = &vptr;
+    vmap_mode = GST_MAP_WRITE;
   } else {
     src = &vptr;
+    vmap_mode = GST_MAP_READ;
     dest = &cptr;
   }
 
@@ -813,7 +816,8 @@ gst_amc_color_format_copy (GstAmcColorFormatInfo * cinfo,
     GstMapInfo minfo;
 
     GST_DEBUG ("Buffer sizes equal, doing fast copy");
-    gst_buffer_map (vbuffer, &minfo, GST_MAP_WRITE);
+    if (!gst_buffer_map (vbuffer, &minfo, vmap_mode))
+      goto fail_map;
 
     cptr = cbuffer->data + cbuffer_info->offset;
     vptr = minfo.data;
@@ -841,7 +845,8 @@ gst_amc_color_format_copy (GstAmcColorFormatInfo * cinfo,
       slice_height = cinfo->slice_height;
       g_assert (stride > 0 && slice_height > 0);
 
-      gst_video_frame_map (&vframe, vinfo, vbuffer, GST_MAP_WRITE);
+      if (!gst_video_frame_map (&vframe, vinfo, vbuffer, vmap_mode))
+        goto fail_map;
 
       for (i = 0; i < 3; i++) {
         if (i == 0) {
@@ -899,7 +904,9 @@ gst_amc_color_format_copy (GstAmcColorFormatInfo * cinfo,
 
       /* FIXME: This does not work for odd widths or heights
        * but might as well be a bug in the codec */
-      gst_video_frame_map (&vframe, vinfo, vbuffer, GST_MAP_WRITE);
+      if (!gst_video_frame_map (&vframe, vinfo, vbuffer, vmap_mode))
+        goto fail_map;
+
       for (i = 0; i < 2; i++) {
         if (i == 0) {
           c_stride = cinfo->stride;
@@ -942,7 +949,8 @@ gst_amc_color_format_copy (GstAmcColorFormatInfo * cinfo,
       /* This should always be set */
       g_assert (cinfo->stride > 0 && cinfo->slice_height > 0);
 
-      gst_video_frame_map (&vframe, vinfo, vbuffer, GST_MAP_WRITE);
+      if (!gst_video_frame_map (&vframe, vinfo, vbuffer, vmap_mode))
+        goto fail_map;
 
       for (i = 0; i < 2; i++) {
         c_stride = cinfo->stride;
@@ -988,7 +996,9 @@ gst_amc_color_format_copy (GstAmcColorFormatInfo * cinfo,
       const size_t tile_h_chroma = (height / 2 - 1) / TILE_HEIGHT + 1;
       size_t luma_size = tile_w_align * tile_h_luma * TILE_SIZE;
 
-      gst_video_frame_map (&vframe, vinfo, vbuffer, GST_MAP_WRITE);
+      if (!gst_video_frame_map (&vframe, vinfo, vbuffer, vmap_mode))
+        goto fail_map;
+
       v_luma = GST_VIDEO_FRAME_PLANE_DATA (&vframe, 0);
       v_chroma = GST_VIDEO_FRAME_PLANE_DATA (&vframe, 1);
       v_luma_stride = GST_VIDEO_FRAME_COMP_STRIDE (&vframe, 0);
@@ -1069,6 +1079,10 @@ gst_amc_color_format_copy (GstAmcColorFormatInfo * cinfo,
 
 done:
   return ret;
+
+fail_map:
+  GST_ERROR ("Failed to map GStreamer buffer memory in mode %d", vmap_mode);
+  return FALSE;
 }
 
 static const struct
@@ -1141,7 +1155,8 @@ static const struct
   HEVCHighTierLevel51, "high", "5.1"}, {
   HEVCHighTierLevel52, "high", "5.2"}, {
   HEVCHighTierLevel6, "high", "6"}, {
-  HEVCHighTierLevel61, "high", "6.1"}
+  HEVCHighTierLevel61, "high", "6.1"}, {
+  HEVCHighTierLevel62, "high", "6.2"}
 };
 
 const gchar *
@@ -1197,7 +1212,8 @@ gst_amc_avc_profile_to_string (gint profile, const gchar ** alternative)
 
   for (i = 0; i < G_N_ELEMENTS (avc_profile_mapping_table); i++) {
     if (avc_profile_mapping_table[i].id == profile) {
-      *alternative = avc_profile_mapping_table[i].alt_str;
+      if (alternative != NULL)
+        *alternative = avc_profile_mapping_table[i].alt_str;
       return avc_profile_mapping_table[i].str;
     }
   }
@@ -2333,35 +2349,40 @@ gst_amc_codec_info_to_caps (const GstAmcCodecInfo * codec_info,
               tmp2 = gst_structure_copy (tmp);
               gst_structure_set (tmp2, "profile", G_TYPE_STRING, profile, NULL);
 
-              /* FIXME: Implement tier/level support here */
-#if 0
               if (codec_info->is_encoder) {
                 const gchar *level, *tier;
                 gint k;
-                GValue va = { 0, };
                 GValue v = { 0, };
 
-                g_value_init (&va, GST_TYPE_LIST);
                 g_value_init (&v, G_TYPE_STRING);
                 for (k = 1; k <= type->profile_levels[j].level && k != 0;
                     k <<= 1) {
                   level = gst_amc_hevc_tier_level_to_string (k, &tier);
-                  if (!level)
+                  if (!level || !tier)
                     continue;
 
-                  g_value_set_string (&v, level);
-                  gst_value_list_append_value (&va, &v);
+                  tmp3 = gst_structure_copy (tmp2);
+
+                  g_value_set_string (&v, tier);
+                  gst_structure_set_value (tmp3, "tier", &v);
                   g_value_reset (&v);
+
+                  g_value_set_string (&v, level);
+                  gst_structure_set_value (tmp3, "level", &v);
+                  g_value_reset (&v);
+
+                  encoded_ret = gst_caps_merge_structure (encoded_ret, tmp3);
+
+                  have_profile_level = TRUE;
                 }
-
-                gst_structure_set_value (tmp2, "level", &va);
-
-                g_value_unset (&va);
-                g_value_unset (&v);
               }
-#endif
 
-              encoded_ret = gst_caps_merge_structure (encoded_ret, tmp2);
+              if (have_profile_level) {
+                gst_structure_free (tmp2);
+              } else {
+                encoded_ret = gst_caps_merge_structure (encoded_ret, tmp2);
+              }
+
               have_profile_level = TRUE;
             }
           }
