@@ -19,9 +19,10 @@
  */
 /**
  * SECTION:element-vtdec
- * @title: gstvtdec
+ * @title: vtdec
  *
- * Apple VideoToolbox based decoder.
+ * Apple VideoToolbox based decoder which might use a HW or a SW
+ * implementation depending on the device.
  *
  * ## Example launch line
  * |[
@@ -31,6 +32,20 @@
  *
  */
 
+/**
+ * SECTION:element-vtdec_hw
+ * @title: vtdec_hw
+ *
+ * Apple VideoToolbox based HW-only decoder.
+ *
+ * ## Example launch line
+ * |[
+ * gst-launch-1.0 -v filesrc location=file.mov ! qtdemux ! queue ! h264parse ! vtdec_hw ! videoconvert ! autovideosink
+ * ]|
+ * Decode h264 video from a mov file.
+ *
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -90,6 +105,8 @@ static void gst_vtdec_session_output_callback (void
     CMTime duration);
 static gboolean compute_h264_decode_picture_buffer_length (GstVtdec * vtdec,
     GstBuffer * codec_data, int *length);
+static gboolean compute_hevc_decode_picture_buffer_length (GstVtdec * vtdec,
+    GstBuffer * codec_data, int *length);
 static gboolean gst_vtdec_compute_reorder_queue_length (GstVtdec * vtdec,
     CMVideoCodecType cm_format, GstBuffer * codec_data);
 static void gst_vtdec_set_latency (GstVtdec * vtdec);
@@ -100,6 +117,8 @@ static GstStaticPadTemplate gst_vtdec_sink_template =
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-h264, stream-format=avc, alignment=au,"
+        " width=(int)[1, MAX], height=(int)[1, MAX];"
+        "video/x-h265, stream-format=(string){ hev1, hvc1 }, alignment=au,"
         " width=(int)[1, MAX], height=(int)[1, MAX];"
         "video/mpeg, mpegversion=2, systemstream=false, parsed=true;"
         "image/jpeg;"
@@ -117,7 +136,7 @@ const CFStringRef
 CFSTR ("RequireHardwareAcceleratedVideoDecoder");
 #endif
 
-#define VIDEO_SRC_CAPS_FORMATS "{ NV12, AYUV64, RGBA64_LE, ARGB64_BE }"
+#define VIDEO_SRC_CAPS_FORMATS "{ NV12, AYUV64, ARGB64_BE }"
 
 #define VIDEO_SRC_CAPS_NATIVE                                           \
     GST_VIDEO_CAPS_MAKE(VIDEO_SRC_CAPS_FORMATS) ";"                     \
@@ -146,9 +165,15 @@ gst_vtdec_class_init (GstVtdecClass * klass)
      base_class_init if you intend to subclass this class. */
   gst_element_class_add_static_pad_template (element_class,
       &gst_vtdec_sink_template);
-  gst_element_class_add_pad_template (element_class,
-      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-          gst_caps_from_string (VIDEO_SRC_CAPS)));
+
+  {
+    GstCaps *caps = gst_caps_from_string (VIDEO_SRC_CAPS);
+    /* RGBA64_LE is kCVPixelFormatType_64RGBALE, only available on macOS 11.3+ */
+    if (GST_VTUTIL_HAVE_64ARGBALE)
+      caps = gst_vtutil_caps_append_video_format (caps, "RGBA64_LE");
+    gst_element_class_add_pad_template (element_class,
+        gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, caps));
+  }
 
   gst_element_class_set_static_metadata (element_class,
       "Apple VideoToolbox decoder",
@@ -270,10 +295,18 @@ get_preferred_video_format (GstStructure * s, gboolean prores)
         break;
       case GST_VIDEO_FORMAT_AYUV64:
       case GST_VIDEO_FORMAT_ARGB64_BE:
-      case GST_VIDEO_FORMAT_RGBA64_LE:
         if (prores)
           return vfmt;
         break;
+      case GST_VIDEO_FORMAT_RGBA64_LE:
+        if (GST_VTUTIL_HAVE_64ARGBALE) {
+          if (prores)
+            return vfmt;
+        } else {
+          /* Codepath will never be hit on macOS older than Big Sur (11.3) */
+          g_warn_if_reached ();
+        }
+        break;
       default:
         break;
     }
@@ -506,6 +539,8 @@ gst_vtdec_set_format (GstVideoDecoder * decoder, GstVideoCodecState * state)
   caps_name = gst_structure_get_name (structure);
   if (!strcmp (caps_name, "video/x-h264")) {
     cm_format = kCMVideoCodecType_H264;
+  } else if (!strcmp (caps_name, "video/x-h265")) {
+    cm_format = kCMVideoCodecType_HEVC;
   } else if (!strcmp (caps_name, "video/mpeg")) {
     cm_format = kCMVideoCodecType_MPEG2Video;
   } else if (!strcmp (caps_name, "image/jpeg")) {
@@ -522,7 +557,9 @@ gst_vtdec_set_format (GstVideoDecoder * decoder, GstVideoCodecState * state)
     }
   }
 
-  if (cm_format == kCMVideoCodecType_H264 && state->codec_data == NULL) {
+  if ((cm_format == kCMVideoCodecType_H264
+          || cm_format == kCMVideoCodecType_HEVC)
+      && state->codec_data == NULL) {
     GST_INFO_OBJECT (vtdec, "no codec data, wait for one");
     return TRUE;
   }
@@ -579,7 +616,7 @@ gst_vtdec_handle_frame (GstVideoDecoder * decoder, GstVideoCodecFrame * frame)
 {
   OSStatus status;
   CMSampleBufferRef cm_sample_buffer = NULL;
-  VTDecodeFrameFlags input_flags, output_flags;
+  VTDecodeFrameFlags input_flags;
   GstVtdec *vtdec = GST_VTDEC (decoder);
   GstFlowReturn ret = GST_FLOW_OK;
   int decode_frame_number = frame->decode_frame_number;
@@ -600,7 +637,6 @@ gst_vtdec_handle_frame (GstVideoDecoder * decoder, GstVideoCodecFrame * frame)
    * reordering ourselves.
    */
   input_flags = kVTDecodeFrame_EnableAsynchronousDecompression;
-  output_flags = 0;
 
   cm_sample_buffer =
       cm_sample_buffer_from_gst_buffer (vtdec, frame->input_buffer);
@@ -661,7 +697,11 @@ gst_vtdec_create_session (GstVtdec * vtdec, GstVideoFormat format,
       cv_format = kCVPixelFormatType_64ARGB;
       break;
     case GST_VIDEO_FORMAT_RGBA64_LE:
-      cv_format = kCVPixelFormatType_64RGBALE;
+      if (GST_VTUTIL_HAVE_64ARGBALE)
+        cv_format = kCVPixelFormatType_64RGBALE;
+      else
+        /* Codepath will never be hit on macOS older than Big Sur (11.3) */
+        g_warn_if_reached ();
       break;
     default:
       g_warn_if_reached ();
@@ -756,7 +796,12 @@ create_format_description_from_codec_data (GstVtdec * vtdec,
   gst_buffer_map (codec_data, &map, GST_MAP_READ);
   atoms = CFDictionaryCreateMutable (NULL, 0, &kCFTypeDictionaryKeyCallBacks,
       &kCFTypeDictionaryValueCallBacks);
-  gst_vtutil_dict_set_data (atoms, CFSTR ("avcC"), map.data, map.size);
+
+  if (cm_format == kCMVideoCodecType_HEVC)
+    gst_vtutil_dict_set_data (atoms, CFSTR ("hvcC"), map.data, map.size);
+  else
+    gst_vtutil_dict_set_data (atoms, CFSTR ("avcC"), map.data, map.size);
+
   gst_vtutil_dict_set_object (extensions,
       CFSTR ("SampleDescriptionExtensionAtoms"), (CFTypeRef *) atoms);
   gst_buffer_unmap (codec_data, &map);
@@ -1116,6 +1161,11 @@ gst_vtdec_compute_reorder_queue_length (GstVtdec * vtdec,
             &vtdec->reorder_queue_length)) {
       return FALSE;
     }
+  } else if (cm_format == kCMVideoCodecType_HEVC) {
+    if (!compute_hevc_decode_picture_buffer_length (vtdec, codec_data,
+            &vtdec->reorder_queue_length)) {
+      return FALSE;
+    }
   } else {
     vtdec->reorder_queue_length = 0;
   }
@@ -1163,6 +1213,34 @@ compute_h264_decode_picture_buffer_length (GstVtdec * vtdec,
   return TRUE;
 }
 
+static gboolean
+compute_hevc_decode_picture_buffer_length (GstVtdec * vtdec,
+    GstBuffer * codec_data, int *length)
+{
+  /* This value should be level dependent (table A.8)
+   * but let's assume the maximum possible one for simplicity. */
+  const gint max_luma_ps = 35651584;
+  const gint max_dpb_pic_buf = 6;
+  gint max_dbp_size, pic_size_samples_y;
+
+  if (vtdec->video_info.width == 0 || vtdec->video_info.height == 0)
+    return FALSE;
+
+  /* A.4.2 */
+  pic_size_samples_y = vtdec->video_info.width * vtdec->video_info.height;
+  if (pic_size_samples_y <= (max_luma_ps >> 2))
+    max_dbp_size = max_dpb_pic_buf * 4;
+  else if (pic_size_samples_y <= (max_luma_ps >> 1))
+    max_dbp_size = max_dpb_pic_buf * 2;
+  else if (pic_size_samples_y <= ((3 * max_luma_ps) >> 2))
+    max_dbp_size = (max_dpb_pic_buf * 4) / 3;
+  else
+    max_dbp_size = max_dpb_pic_buf;
+
+  *length = MIN (max_dbp_size, 16);
+  return TRUE;
+}
+
 static void
 gst_vtdec_set_latency (GstVtdec * vtdec)
 {
