@@ -17,6 +17,49 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+
+/**
+ * SECTION:element-vtenc_h264
+ * @title: vtenc_h264
+ *
+ * Apple VideoToolbox H264 encoder, which can either use HW or a SW
+ * implementation depending on the device.
+ *
+ * ## Example pipeline
+ * |[
+ * gst-launch-1.0 -v videotestsrc ! vtenc_h264 ! qtmux ! filesink location=out.mov
+ * ]| Encode a test video pattern and save it as an MOV file
+ *
+ */
+
+/**
+ * SECTION:element-vtenc_h264_hw
+ * @title: vtenc_h264_hw
+ *
+ * Apple VideoToolbox H264 HW-only encoder (only available on macOS at
+ * present).
+ *
+ * ## Example pipeline
+ * |[
+ * gst-launch-1.0 -v videotestsrc ! vtenc_h264_hw ! qtmux ! filesink location=out.mov
+ * ]| Encode a test video pattern and save it as an MOV file
+ *
+ */
+
+/**
+ * SECTION:element-vtenc_prores
+ * @title: vtenc_prores
+ *
+ * Apple VideoToolbox ProRes encoder
+ *
+ * ## Example pipeline
+ * |[
+ * gst-launch-1.0 -v videotestsrc ! vtenc_prores ! qtmux ! filesink location=out.mov
+ * ]| Encode a test video pattern and save it as an MOV file
+ *
+ * Since: 1.20
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -28,7 +71,6 @@
 #include "vtutil.h"
 #include <gst/pbutils/codec-utils.h>
 
-#define VTENC_DEFAULT_USAGE       6     /* Profile: Baseline  Level: 2.1 */
 #define VTENC_DEFAULT_BITRATE     0
 #define VTENC_DEFAULT_FRAME_REORDERING TRUE
 #define VTENC_DEFAULT_REALTIME FALSE
@@ -160,9 +202,10 @@ GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("{ NV12, I420 }"));
 #else
 static GstStaticCaps sink_caps =
 GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE
-    ("{ AYUV64, UYVY, NV12, I420, RGBA64_LE, ARGB64_BE }"));
+    ("{ AYUV64, UYVY, NV12, I420, ARGB64_BE }"));
 #endif
 
+
 static void
 gst_vtenc_base_init (GstVTEncClass * klass)
 {
@@ -173,7 +216,6 @@ gst_vtenc_base_init (GstVTEncClass * klass)
   const int min_height = 1, max_height = G_MAXINT;
   const int min_fps_n = 0, max_fps_n = G_MAXINT;
   const int min_fps_d = 1, max_fps_d = 1;
-  GstPadTemplate *sink_template, *src_template;
   GstCaps *src_caps;
   gchar *longname, *description;
 
@@ -187,9 +229,14 @@ gst_vtenc_base_init (GstVTEncClass * klass)
   g_free (longname);
   g_free (description);
 
-  sink_template = gst_pad_template_new ("sink",
-      GST_PAD_SINK, GST_PAD_ALWAYS, gst_static_caps_get (&sink_caps));
-  gst_element_class_add_pad_template (element_class, sink_template);
+  {
+    GstCaps *caps = gst_static_caps_get (&sink_caps);
+    /* RGBA64_LE is kCVPixelFormatType_64RGBALE, only available on macOS 11.3+ */
+    if (GST_VTUTIL_HAVE_64ARGBALE)
+      caps = gst_vtutil_caps_append_video_format (caps, "RGBA64_LE");
+    gst_element_class_add_pad_template (element_class,
+        gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS, caps));
+  }
 
   src_caps = gst_caps_new_simple (codec_details->mimetype,
       "width", GST_TYPE_INT_RANGE, min_width, max_width,
@@ -219,6 +266,11 @@ gst_vtenc_base_init (GstVTEncClass * klass)
           "stream-format", G_TYPE_STRING, "avc",
           "alignment", G_TYPE_STRING, "au", NULL);
       break;
+    case kCMVideoCodecType_HEVC:
+      gst_structure_set (gst_caps_get_structure (src_caps, 0),
+          "stream-format", G_TYPE_STRING, "hvc1",
+          "alignment", G_TYPE_STRING, "au", NULL);
+      break;
     case GST_kCMVideoCodecType_Some_AppleProRes:
       if (g_strcmp0 (codec_details->mimetype, "video/x-prores") == 0) {
         G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
@@ -249,9 +301,9 @@ gst_vtenc_base_init (GstVTEncClass * klass)
     default:
       g_assert_not_reached ();
   }
-  src_template = gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-      src_caps);
-  gst_element_class_add_pad_template (element_class, src_template);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, src_caps));
   gst_caps_unref (src_caps);
 }
 
@@ -315,11 +367,24 @@ gst_vtenc_class_init (GstVTEncClass * klass)
           G_MAXUINT64, VTENC_DEFAULT_MAX_KEYFRAME_INTERVAL_DURATION,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property (gobject_class, PROP_PRESERVE_ALPHA,
-      g_param_spec_boolean ("preserve-alpha", "Preserve Video Alpha Values",
-          "Video alpha values (non opaque) need to be perserved.",
-          VTENC_DEFAULT_PRESERVE_ALPHA,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
+  /*
+   * H264 doesn't support alpha components, so only add the property for prores
+   */
+  if (g_strcmp0 (G_OBJECT_CLASS_NAME (klass), "vtenc_prores") == 0) {
+    /**
+     * vtenc_prores:preserve-alpha
+     *
+     * Preserve non-opaque video alpha values from the input video when
+     * compressing, else treat all alpha component as opaque.
+     *
+     * Since: 1.20
+     */
+    g_object_class_install_property (gobject_class, PROP_PRESERVE_ALPHA,
+        g_param_spec_boolean ("preserve-alpha", "Preserve Video Alpha Values",
+            "Video alpha values (non opaque) need to be preserved",
+            VTENC_DEFAULT_PRESERVE_ALPHA,
+            G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
+  }
 }
 
 static void
@@ -658,7 +723,7 @@ gst_vtenc_stop (GstVideoEncoder * enc)
 }
 
 static CFStringRef
-gst_vtenc_profile_level_key (GstVTEnc * self, const gchar * profile,
+gst_vtenc_h264_profile_level_key (GstVTEnc * self, const gchar * profile,
     const gchar * level_arg)
 {
   char level[64];
@@ -701,6 +766,37 @@ gst_vtenc_profile_level_key (GstVTEnc * self, const gchar * profile,
   return ret;
 }
 
+static CFStringRef
+gst_vtenc_hevc_profile_level_key (GstVTEnc * self, const gchar * profile,
+    const gchar * level_arg)
+{
+  gchar *key = NULL;
+  CFStringRef ret = NULL;
+
+  if (profile == NULL || !strcmp (profile, "main"))
+    profile = "Main";
+  else if (!strcmp (profile, "main-10"))
+    profile = "Main10";
+  else if (!strcmp (profile, "main-422-10"))
+    /* TODO: this should probably be guarded with a version check (macOS 12.3+ / iOS 15.4+)
+     * https://developer.apple.com/documentation/videotoolbox/kvtprofilelevel_hevc_main10_autolevel */
+    profile = "Main42210";
+  else {
+    GST_ERROR_OBJECT (self, "invalid profile: %s", profile);
+    return ret;
+  }
+
+  /* VT does not support specific levels for HEVC */
+  key = g_strdup_printf ("HEVC_%s_AutoLevel", profile);
+  ret = CFStringCreateWithBytes (NULL, (const guint8 *) key, strlen (key),
+      kCFStringEncodingASCII, 0);
+
+  GST_INFO_OBJECT (self, "negotiated profile and level %s", key);
+
+  g_free (key);
+  return ret;
+}
+
 static gboolean
 gst_vtenc_negotiate_profile_and_level (GstVTEnc * self, GstStructure * s)
 {
@@ -709,9 +805,16 @@ gst_vtenc_negotiate_profile_and_level (GstVTEnc * self, GstStructure * s)
 
   if (self->profile_level)
     CFRelease (self->profile_level);
-  self->profile_level = gst_vtenc_profile_level_key (self, profile, level);
+
+  if (self->specific_format_id == kCMVideoCodecType_HEVC)
+    self->profile_level =
+        gst_vtenc_hevc_profile_level_key (self, profile, level);
+  else
+    self->profile_level =
+        gst_vtenc_h264_profile_level_key (self, profile, level);
+
   if (self->profile_level == NULL) {
-    GST_ERROR_OBJECT (self, "unsupported h264 profile '%s' or level '%s'",
+    GST_ERROR_OBJECT (self, "unsupported profile '%s' or level '%s'",
         profile, level);
     return FALSE;
   }
@@ -760,6 +863,11 @@ gst_vtenc_negotiate_specific_format_details (GstVideoEncoder * enc)
         if (!gst_vtenc_negotiate_profile_and_level (self, s))
           goto fail;
         break;
+      case kCMVideoCodecType_HEVC:
+        self->specific_format_id = kCMVideoCodecType_HEVC;
+        if (!gst_vtenc_negotiate_profile_and_level (self, s))
+          goto fail;
+        break;
       case GST_kCMVideoCodecType_Some_AppleProRes:
         if (g_strcmp0 (self->details->mimetype, "video/x-prores") != 0) {
           GST_ERROR_OBJECT (self, "format_id == %i mimetype must be Apple "
@@ -869,35 +977,48 @@ gst_vtenc_negotiate_downstream (GstVTEnc * self, CMSampleBufferRef sbuf)
 
   switch (self->details->format_id) {
     case kCMVideoCodecType_H264:
+    case kCMVideoCodecType_HEVC:
     {
       CMFormatDescriptionRef fmt;
       CFDictionaryRef atoms;
-      CFStringRef avccKey;
-      CFDataRef avcc;
+      CFStringRef boxKey;
+      CFDataRef box;
       guint8 *codec_data;
       gsize codec_data_size;
       GstBuffer *codec_data_buf;
-      guint8 sps[3];
+      guint8 sps[12];
 
       fmt = CMSampleBufferGetFormatDescription (sbuf);
       atoms = CMFormatDescriptionGetExtension (fmt,
           kCMFormatDescriptionExtension_SampleDescriptionExtensionAtoms);
-      avccKey = CFStringCreateWithCString (NULL, "avcC", kCFStringEncodingUTF8);
-      avcc = CFDictionaryGetValue (atoms, avccKey);
-      CFRelease (avccKey);
-      codec_data_size = CFDataGetLength (avcc);
+
+      if (self->details->format_id == kCMVideoCodecType_HEVC)
+        boxKey =
+            CFStringCreateWithCString (NULL, "hvcC", kCFStringEncodingUTF8);
+      else
+        boxKey =
+            CFStringCreateWithCString (NULL, "avcC", kCFStringEncodingUTF8);
+
+      box = CFDictionaryGetValue (atoms, boxKey);
+      CFRelease (boxKey);
+      codec_data_size = CFDataGetLength (box);
       codec_data = g_malloc (codec_data_size);
-      CFDataGetBytes (avcc, CFRangeMake (0, codec_data_size), codec_data);
+      CFDataGetBytes (box, CFRangeMake (0, codec_data_size), codec_data);
       codec_data_buf = gst_buffer_new_wrapped (codec_data, codec_data_size);
 
       gst_structure_set (s, "codec_data", GST_TYPE_BUFFER, codec_data_buf,
           NULL);
 
-      sps[0] = codec_data[1];
-      sps[1] = codec_data[2] & ~0xDF;
-      sps[2] = codec_data[3];
-
-      gst_codec_utils_h264_caps_set_level_and_profile (caps, sps, 3);
+      if (self->details->format_id == kCMVideoCodecType_HEVC) {
+        sps[0] = codec_data[1];
+        sps[11] = codec_data[12];
+        gst_codec_utils_h265_caps_set_level_tier_and_profile (caps, sps, 12);
+      } else {
+        sps[0] = codec_data[1];
+        sps[1] = codec_data[2] & ~0xDF;
+        sps[2] = codec_data[3];
+        gst_codec_utils_h264_caps_set_level_and_profile (caps, sps, 3);
+      }
 
       gst_buffer_unref (codec_data_buf);
     }
@@ -1127,7 +1248,7 @@ gst_vtenc_create_session (GstVTEnc * self)
         (gdouble) self->negotiated_fps_n / (gdouble) self->negotiated_fps_d);
 
     /*
-     * https://developer.apple.com/documentation/videotoolbox/vtcompressionsession/compression_properties/profile_and_level_constants
+     * https://developer.apple.com/documentation/videotoolbox/kvtcompressionpropertykey_profilelevel
      */
     status = VTSessionSetProperty (session,
         kVTCompressionPropertyKey_ProfileLevel, self->profile_level);
@@ -1585,7 +1706,11 @@ gst_vtenc_encode_frame (GstVTEnc * self, GstVideoCodecFrame * frame)
           pixel_format_type = kCVPixelFormatType_4444AYpCbCr16;
           break;
         case GST_VIDEO_FORMAT_RGBA64_LE:
-          pixel_format_type = kCVPixelFormatType_64RGBALE;
+          if (GST_VTUTIL_HAVE_64ARGBALE)
+            pixel_format_type = kCVPixelFormatType_64RGBALE;
+          else
+            /* Codepath will never be hit on macOS older than Big Sur (11.3) */
+            g_assert_not_reached ();
           break;
         case GST_VIDEO_FORMAT_I420:
           pixel_format_type = kCVPixelFormatType_420YpCbCr8Planar;
@@ -1819,8 +1944,11 @@ gst_vtenc_register (GstPlugin * plugin,
 
 static const GstVTEncoderDetails gst_vtenc_codecs[] = {
   {"H.264", "h264", "video/x-h264", kCMVideoCodecType_H264, FALSE},
+  {"H.265/HEVC", "h265", "video/x-h265", kCMVideoCodecType_HEVC, FALSE},
 #ifndef HAVE_IOS
   {"H.264 (HW only)", "h264_hw", "video/x-h264", kCMVideoCodecType_H264, TRUE},
+  {"H.265/HEVC (HW only)", "h265_hw", "video/x-h265", kCMVideoCodecType_HEVC,
+      TRUE},
 #endif
   {"Apple ProRes", "prores", "video/x-prores",
       GST_kCMVideoCodecType_Some_AppleProRes, FALSE},
