@@ -22,7 +22,6 @@
 #endif
 
 #include "gstd3d11overlaycompositor.h"
-#include "gstd3d11shader.h"
 #include "gstd3d11pluginutils.h"
 #include <wrl.h>
 
@@ -40,21 +39,19 @@ typedef struct
     FLOAT z;
   } position;
   struct {
-    FLOAT x;
-    FLOAT y;
+    FLOAT u;
+    FLOAT v;
   } texture;
 } VertexData;
 
 static const gchar templ_pixel_shader[] =
     "Texture2D shaderTexture;\n"
     "SamplerState samplerState;\n"
-    "\n"
     "struct PS_INPUT\n"
     "{\n"
     "  float4 Position: SV_POSITION;\n"
-    "  float3 Texture: TEXCOORD0;\n"
+    "  float2 Texture: TEXCOORD;\n"
     "};\n"
-    "\n"
     "float4 main(PS_INPUT input): SV_TARGET\n"
     "{\n"
     "  return shaderTexture.Sample(samplerState, input.Texture);\n"
@@ -64,13 +61,13 @@ static const gchar templ_vertex_shader[] =
     "struct VS_INPUT\n"
     "{\n"
     "  float4 Position : POSITION;\n"
-    "  float4 Texture : TEXCOORD0;\n"
+    "  float2 Texture : TEXCOORD;\n"
     "};\n"
     "\n"
     "struct VS_OUTPUT\n"
     "{\n"
     "  float4 Position: SV_POSITION;\n"
-    "  float4 Texture: TEXCOORD0;\n"
+    "  float2 Texture: TEXCOORD;\n"
     "};\n"
     "\n"
     "VS_OUTPUT main(VS_INPUT input)\n"
@@ -79,10 +76,9 @@ static const gchar templ_vertex_shader[] =
     "}\n";
 /* *INDENT-ON* */
 
-struct _GstD3D11OverlayCompositor
+struct _GstD3D11OverlayCompositorPrivate
 {
-  GstD3D11Device *device;
-  GstVideoInfo out_info;
+  GstVideoInfo info;
 
   D3D11_VIEWPORT viewport;
 
@@ -102,14 +98,58 @@ typedef struct
   GstVideoOverlayRectangle *overlay_rect;
   ID3D11Texture2D *texture;
   ID3D11ShaderResourceView *srv;
-  GstD3D11Quad *quad;
+  ID3D11Buffer *vertex_buffer;
 } GstD3D11CompositionOverlay;
 
+static void gst_d3d11_overlay_compositor_dispose (GObject * object);
+static void
+gst_d3d11_overlay_compositor_free_overlays (GstD3D11OverlayCompositor * self);
+
+#define gst_d3d11_overlay_compositor_parent_class parent_class
+G_DEFINE_TYPE_WITH_PRIVATE (GstD3D11OverlayCompositor,
+    gst_d3d11_overlay_compositor, GST_TYPE_OBJECT);
+
+static void
+gst_d3d11_overlay_compositor_class_init (GstD3D11OverlayCompositorClass * klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->dispose = gst_d3d11_overlay_compositor_dispose;
+}
+
+static void
+gst_d3d11_overlay_compositor_init (GstD3D11OverlayCompositor * self)
+{
+  self->priv = (GstD3D11OverlayCompositorPrivate *)
+      gst_d3d11_overlay_compositor_get_instance_private (self);
+}
+
+static void
+gst_d3d11_overlay_compositor_dispose (GObject * object)
+{
+  GstD3D11OverlayCompositor *self = GST_D3D11_OVERLAY_COMPOSITOR (object);
+  GstD3D11OverlayCompositorPrivate *priv = self->priv;
+
+  gst_d3d11_overlay_compositor_free_overlays (self);
+
+  GST_D3D11_CLEAR_COM (priv->ps);
+  GST_D3D11_CLEAR_COM (priv->vs);
+  GST_D3D11_CLEAR_COM (priv->layout);
+  GST_D3D11_CLEAR_COM (priv->sampler);
+  GST_D3D11_CLEAR_COM (priv->blend);
+  GST_D3D11_CLEAR_COM (priv->index_buffer);
+
+  gst_clear_object (&self->device);
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
 static GstD3D11CompositionOverlay *
 gst_d3d11_composition_overlay_new (GstD3D11OverlayCompositor * self,
     GstVideoOverlayRectangle * overlay_rect)
 {
-  GstD3D11CompositionOverlay *overlay = NULL;
+  GstD3D11OverlayCompositorPrivate *priv = self->priv;
+  GstD3D11CompositionOverlay *overlay = nullptr;
   gint x, y;
   guint width, height;
   D3D11_SUBRESOURCE_DATA subresource_data;
@@ -127,16 +167,11 @@ gst_d3d11_composition_overlay_new (GstD3D11OverlayCompositor * self,
   ID3D11Device *device_handle;
   ID3D11DeviceContext *context_handle;
   GstD3D11Device *device = self->device;
-  const guint index_count = 2 * 3;
   FLOAT x1, y1, x2, y2;
   gdouble val;
-  /* *INDENT-OFF* */
-  ComPtr<ID3D11Texture2D> texture;
-  ComPtr<ID3D11ShaderResourceView> srv;
-  ComPtr<ID3D11Buffer> vertex_buffer;
-  /* *INDENT-ON* */
-
-  g_return_val_if_fail (overlay_rect != NULL, NULL);
+  ComPtr < ID3D11Texture2D > texture;
+  ComPtr < ID3D11ShaderResourceView > srv;
+  ComPtr < ID3D11Buffer > vertex_buffer;
 
   memset (&subresource_data, 0, sizeof (subresource_data));
   memset (&texture_desc, 0, sizeof (texture_desc));
@@ -148,27 +183,27 @@ gst_d3d11_composition_overlay_new (GstD3D11OverlayCompositor * self,
 
   if (!gst_video_overlay_rectangle_get_render_rectangle (overlay_rect, &x, &y,
           &width, &height)) {
-    GST_ERROR ("Failed to get render rectangle");
-    return NULL;
+    GST_ERROR_OBJECT (self, "Failed to get render rectangle");
+    return nullptr;
   }
 
   buf = gst_video_overlay_rectangle_get_pixels_unscaled_argb (overlay_rect,
       GST_VIDEO_OVERLAY_FORMAT_FLAG_NONE);
   if (!buf) {
-    GST_ERROR ("Failed to get overlay buffer");
-    return NULL;
+    GST_ERROR_OBJECT (self, "Failed to get overlay buffer");
+    return nullptr;
   }
 
   vmeta = gst_buffer_get_video_meta (buf);
   if (!vmeta) {
-    GST_ERROR ("Failed to get video meta");
-    return NULL;
+    GST_ERROR_OBJECT (self, "Failed to get video meta");
+    return nullptr;
   }
 
   if (!gst_video_meta_map (vmeta,
           0, &info, (gpointer *) & data, &stride, GST_MAP_READ)) {
-    GST_ERROR ("Failed to map");
-    return NULL;
+    GST_ERROR_OBJECT (self, "Failed to map");
+    return nullptr;
   }
 
   /* Do create texture and upload data at once, for create immutable texture */
@@ -180,7 +215,6 @@ gst_d3d11_composition_overlay_new (GstD3D11OverlayCompositor * self,
   texture_desc.Height = height;
   texture_desc.MipLevels = 1;
   texture_desc.ArraySize = 1;
-  /* FIXME: need to consider non-BGRA ? */
   texture_desc.Format = DXGI_FORMAT_B8G8R8A8_UNORM;
   texture_desc.SampleDesc.Count = 1;
   texture_desc.SampleDesc.Quality = 0;
@@ -193,8 +227,8 @@ gst_d3d11_composition_overlay_new (GstD3D11OverlayCompositor * self,
   gst_video_meta_unmap (vmeta, 0, &info);
 
   if (!gst_d3d11_result (hr, device)) {
-    GST_ERROR ("Failed to create texture");
-    return NULL;
+    GST_ERROR_OBJECT (self, "Failed to create texture");
+    return nullptr;
   }
 
   srv_desc.ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2D;
@@ -203,8 +237,8 @@ gst_d3d11_composition_overlay_new (GstD3D11OverlayCompositor * self,
   hr = device_handle->CreateShaderResourceView (texture.Get (), &srv_desc,
       &srv);
   if (!gst_d3d11_result (hr, device) || !srv) {
-    GST_ERROR ("Failed to create shader resource view");
-    return NULL;
+    GST_ERROR_OBJECT (self, "Failed to create shader resource view");
+    return nullptr;
   }
 
   buffer_desc.Usage = D3D11_USAGE_DYNAMIC;
@@ -212,79 +246,74 @@ gst_d3d11_composition_overlay_new (GstD3D11OverlayCompositor * self,
   buffer_desc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
   buffer_desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
 
-  hr = device_handle->CreateBuffer (&buffer_desc, NULL, &vertex_buffer);
+  hr = device_handle->CreateBuffer (&buffer_desc, nullptr, &vertex_buffer);
   if (!gst_d3d11_result (hr, device)) {
-    GST_ERROR ("Couldn't create vertex buffer, hr: 0x%x", (guint) hr);
-    return NULL;
+    GST_ERROR_OBJECT (self,
+        "Couldn't create vertex buffer, hr: 0x%x", (guint) hr);
+    return nullptr;
   }
 
-  gst_d3d11_device_lock (device);
+  GstD3D11DeviceLockGuard lk (device);
   hr = context_handle->Map (vertex_buffer.Get (),
       0, D3D11_MAP_WRITE_DISCARD, 0, &map);
 
   if (!gst_d3d11_result (hr, device)) {
-    GST_ERROR ("Couldn't map vertex buffer, hr: 0x%x", (guint) hr);
-    gst_d3d11_device_unlock (device);
-    return NULL;
+    GST_ERROR_OBJECT (self, "Couldn't map vertex buffer, hr: 0x%x", (guint) hr);
+    return nullptr;
   }
 
   vertex_data = (VertexData *) map.pData;
   /* bottom left */
-  gst_util_fraction_to_double (x, GST_VIDEO_INFO_WIDTH (&self->out_info), &val);
+  gst_util_fraction_to_double (x, GST_VIDEO_INFO_WIDTH (&priv->info), &val);
   x1 = (val * 2.0f) - 1.0f;
 
   gst_util_fraction_to_double (y + height,
-      GST_VIDEO_INFO_HEIGHT (&self->out_info), &val);
+      GST_VIDEO_INFO_HEIGHT (&priv->info), &val);
   y1 = (val * -2.0f) + 1.0f;
 
   /* top right */
   gst_util_fraction_to_double (x + width,
-      GST_VIDEO_INFO_WIDTH (&self->out_info), &val);
+      GST_VIDEO_INFO_WIDTH (&priv->info), &val);
   x2 = (val * 2.0f) - 1.0f;
 
-  gst_util_fraction_to_double (y,
-      GST_VIDEO_INFO_HEIGHT (&self->out_info), &val);
+  gst_util_fraction_to_double (y, GST_VIDEO_INFO_HEIGHT (&priv->info), &val);
   y2 = (val * -2.0f) + 1.0f;
 
   /* bottom left */
   vertex_data[0].position.x = x1;
   vertex_data[0].position.y = y1;
   vertex_data[0].position.z = 0.0f;
-  vertex_data[0].texture.x = 0.0f;
-  vertex_data[0].texture.y = 1.0f;
+  vertex_data[0].texture.u = 0.0f;
+  vertex_data[0].texture.v = 1.0f;
 
   /* top left */
   vertex_data[1].position.x = x1;
   vertex_data[1].position.y = y2;
   vertex_data[1].position.z = 0.0f;
-  vertex_data[1].texture.x = 0.0f;
-  vertex_data[1].texture.y = 0.0f;
+  vertex_data[1].texture.u = 0.0f;
+  vertex_data[1].texture.v = 0.0f;
 
   /* top right */
   vertex_data[2].position.x = x2;
   vertex_data[2].position.y = y2;
   vertex_data[2].position.z = 0.0f;
-  vertex_data[2].texture.x = 1.0f;
-  vertex_data[2].texture.y = 0.0f;
+  vertex_data[2].texture.u = 1.0f;
+  vertex_data[2].texture.v = 0.0f;
 
   /* bottom right */
   vertex_data[3].position.x = x2;
   vertex_data[3].position.y = y1;
   vertex_data[3].position.z = 0.0f;
-  vertex_data[3].texture.x = 1.0f;
-  vertex_data[3].texture.y = 1.0f;
+  vertex_data[3].texture.u = 1.0f;
+  vertex_data[3].texture.v = 1.0f;
 
   context_handle->Unmap (vertex_buffer.Get (), 0);
-  gst_d3d11_device_unlock (device);
 
   overlay = g_new0 (GstD3D11CompositionOverlay, 1);
   overlay->overlay_rect = gst_video_overlay_rectangle_ref (overlay_rect);
   overlay->texture = texture.Detach ();
   overlay->srv = srv.Detach ();
-  overlay->quad = gst_d3d11_quad_new (device,
-      self->ps, self->vs, self->layout, nullptr, 0,
-      vertex_buffer.Get (), sizeof (VertexData),
-      self->index_buffer, DXGI_FORMAT_R16_UINT, index_count);
+  overlay->vertex_buffer = vertex_buffer.Detach ();
 
   return overlay;
 }
@@ -300,17 +329,17 @@ gst_d3d11_composition_overlay_free (GstD3D11CompositionOverlay * overlay)
 
   GST_D3D11_CLEAR_COM (overlay->srv);
   GST_D3D11_CLEAR_COM (overlay->texture);
-
-  if (overlay->quad)
-    gst_d3d11_quad_free (overlay->quad);
+  GST_D3D11_CLEAR_COM (overlay->vertex_buffer);
 
   g_free (overlay);
 }
 
 static gboolean
-gst_d3d11_overlay_compositor_setup_shader (GstD3D11OverlayCompositor * self,
-    GstD3D11Device * device)
+gst_d3d11_overlay_compositor_setup_shader (GstD3D11OverlayCompositor * self)
 {
+  GstD3D11OverlayCompositorPrivate *priv = self->priv;
+  GstVideoInfo *info = &priv->info;
+  GstD3D11Device *device = self->device;
   HRESULT hr;
   D3D11_SAMPLER_DESC sampler_desc;
   D3D11_INPUT_ELEMENT_DESC input_desc[2];
@@ -320,15 +349,12 @@ gst_d3d11_overlay_compositor_setup_shader (GstD3D11OverlayCompositor * self,
   WORD *indices;
   ID3D11Device *device_handle;
   ID3D11DeviceContext *context_handle;
-  /* *INDENT-OFF* */
-  ComPtr<ID3D11PixelShader> ps;
-  ComPtr<ID3D11VertexShader> vs;
-  ComPtr<ID3D11InputLayout> layout;
-  ComPtr<ID3D11SamplerState> sampler;
-  ComPtr<ID3D11BlendState> blend;
-  ComPtr<ID3D11Buffer> index_buffer;
-  /* *INDENT-ON* */
-  const guint index_count = 2 * 3;
+  ComPtr < ID3D11PixelShader > ps;
+  ComPtr < ID3D11VertexShader > vs;
+  ComPtr < ID3D11InputLayout > layout;
+  ComPtr < ID3D11SamplerState > sampler;
+  ComPtr < ID3D11BlendState > blend;
+  ComPtr < ID3D11Buffer > index_buffer;
 
   memset (&sampler_desc, 0, sizeof (sampler_desc));
   memset (input_desc, 0, sizeof (input_desc));
@@ -349,14 +375,15 @@ gst_d3d11_overlay_compositor_setup_shader (GstD3D11OverlayCompositor * self,
 
   hr = device_handle->CreateSamplerState (&sampler_desc, &sampler);
   if (!gst_d3d11_result (hr, device)) {
-    GST_ERROR ("Couldn't create sampler state, hr: 0x%x", (guint) hr);
+    GST_ERROR_OBJECT (self, "Couldn't create sampler state, hr: 0x%x",
+        (guint) hr);
     return FALSE;
   }
 
-  GST_LOG ("Create Pixel Shader \n%s", templ_pixel_shader);
-
-  if (!gst_d3d11_create_pixel_shader (device, templ_pixel_shader, &ps)) {
-    GST_ERROR ("Couldn't create pixel shader");
+  hr = gst_d3d11_create_pixel_shader_simple (device,
+      templ_pixel_shader, "main", &ps);
+  if (!gst_d3d11_result (hr, device)) {
+    GST_ERROR_OBJECT (self, "Couldn't create pixel shader");
     return FALSE;
   }
 
@@ -376,9 +403,10 @@ gst_d3d11_overlay_compositor_setup_shader (GstD3D11OverlayCompositor * self,
   input_desc[1].InputSlotClass = D3D11_INPUT_PER_VERTEX_DATA;
   input_desc[1].InstanceDataStepRate = 0;
 
-  if (!gst_d3d11_create_vertex_shader (device, templ_vertex_shader,
-          input_desc, G_N_ELEMENTS (input_desc), &vs, &layout)) {
-    GST_ERROR ("Couldn't vertex pixel shader");
+  hr = gst_d3d11_create_vertex_shader_simple (device, templ_vertex_shader,
+      "main", input_desc, G_N_ELEMENTS (input_desc), &vs, &layout);
+  if (!gst_d3d11_result (hr, device)) {
+    GST_ERROR_OBJECT (self, "Couldn't vertex pixel shader");
     return FALSE;
   }
 
@@ -396,28 +424,29 @@ gst_d3d11_overlay_compositor_setup_shader (GstD3D11OverlayCompositor * self,
 
   hr = device_handle->CreateBlendState (&blend_desc, &blend);
   if (!gst_d3d11_result (hr, device)) {
-    GST_ERROR ("Couldn't create blend staten, hr: 0x%x", (guint) hr);
+    GST_ERROR_OBJECT (self,
+        "Couldn't create blend staten, hr: 0x%x", (guint) hr);
     return FALSE;
   }
 
   buffer_desc.Usage = D3D11_USAGE_DYNAMIC;
-  buffer_desc.ByteWidth = sizeof (WORD) * index_count;
+  buffer_desc.ByteWidth = sizeof (WORD) * 6;
   buffer_desc.BindFlags = D3D11_BIND_INDEX_BUFFER;
   buffer_desc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
 
-  hr = device_handle->CreateBuffer (&buffer_desc, NULL, &index_buffer);
+  hr = device_handle->CreateBuffer (&buffer_desc, nullptr, &index_buffer);
   if (!gst_d3d11_result (hr, device)) {
-    GST_ERROR ("Couldn't create index buffer, hr: 0x%x", (guint) hr);
+    GST_ERROR_OBJECT (self,
+        "Couldn't create index buffer, hr: 0x%x", (guint) hr);
     return FALSE;
   }
 
-  gst_d3d11_device_lock (device);
+  GstD3D11DeviceLockGuard lk (device);
   hr = context_handle->Map (index_buffer.Get (),
       0, D3D11_MAP_WRITE_DISCARD, 0, &map);
 
   if (!gst_d3d11_result (hr, device)) {
-    GST_ERROR ("Couldn't map index buffer, hr: 0x%x", (guint) hr);
-    gst_d3d11_device_unlock (device);
+    GST_ERROR_OBJECT (self, "Couldn't map index buffer, hr: 0x%x", (guint) hr);
     return FALSE;
   }
 
@@ -433,64 +462,61 @@ gst_d3d11_overlay_compositor_setup_shader (GstD3D11OverlayCompositor * self,
   indices[5] = 2;               /* top right */
 
   context_handle->Unmap (index_buffer.Get (), 0);
-  gst_d3d11_device_unlock (device);
 
-  self->ps = ps.Detach ();
-  self->vs = vs.Detach ();
-  self->layout = layout.Detach ();
-  self->sampler = sampler.Detach ();
-  self->blend = blend.Detach ();
-  self->index_buffer = index_buffer.Detach ();
+  priv->ps = ps.Detach ();
+  priv->vs = vs.Detach ();
+  priv->layout = layout.Detach ();
+  priv->sampler = sampler.Detach ();
+  priv->blend = blend.Detach ();
+  priv->index_buffer = index_buffer.Detach ();
+
+  priv->viewport.TopLeftX = 0;
+  priv->viewport.TopLeftY = 0;
+  priv->viewport.Width = GST_VIDEO_INFO_WIDTH (info);
+  priv->viewport.Height = GST_VIDEO_INFO_HEIGHT (info);
+  priv->viewport.MinDepth = 0.0f;
+  priv->viewport.MaxDepth = 1.0f;
 
   return TRUE;
 }
 
-
 GstD3D11OverlayCompositor *
 gst_d3d11_overlay_compositor_new (GstD3D11Device * device,
-    GstVideoInfo * out_info)
+    const GstVideoInfo * info)
 {
-  GstD3D11OverlayCompositor *compositor = NULL;
+  GstD3D11OverlayCompositor *self = nullptr;
+  GstD3D11OverlayCompositorPrivate *priv;
 
-  g_return_val_if_fail (GST_IS_D3D11_DEVICE (device), NULL);
-  g_return_val_if_fail (out_info != NULL, NULL);
+  g_return_val_if_fail (GST_IS_D3D11_DEVICE (device), nullptr);
+  g_return_val_if_fail (info != nullptr, nullptr);
 
-  compositor = g_new0 (GstD3D11OverlayCompositor, 1);
+  self = (GstD3D11OverlayCompositor *)
+      g_object_new (GST_TYPE_D3D11_OVERLAY_COMPOSITOR, nullptr);
+  gst_object_ref_sink (self);
+  priv = self->priv;
 
-  if (!gst_d3d11_overlay_compositor_setup_shader (compositor, device)) {
-    gst_d3d11_overlay_compositor_free (compositor);
-    return NULL;
+  self->device = (GstD3D11Device *) gst_object_ref (device);
+  priv->info = *info;
+
+  if (!gst_d3d11_overlay_compositor_setup_shader (self)) {
+    gst_object_unref (self);
+    return nullptr;
   }
 
-  compositor->device = (GstD3D11Device *) gst_object_ref (device);
-  compositor->out_info = *out_info;
-
-  compositor->viewport.TopLeftX = 0;
-  compositor->viewport.TopLeftY = 0;
-  compositor->viewport.Width = GST_VIDEO_INFO_WIDTH (out_info);
-  compositor->viewport.Height = GST_VIDEO_INFO_HEIGHT (out_info);
-  compositor->viewport.MinDepth = 0.0f;
-  compositor->viewport.MaxDepth = 1.0f;
-
-  return compositor;
+  return self;
 }
 
-void
-gst_d3d11_overlay_compositor_free (GstD3D11OverlayCompositor * compositor)
+static void
+gst_d3d11_overlay_compositor_free_overlays (GstD3D11OverlayCompositor * self)
 {
-  g_return_if_fail (compositor != NULL);
+  GstD3D11OverlayCompositorPrivate *priv = self->priv;
 
-  gst_d3d11_overlay_compositor_free_overlays (compositor);
+  if (priv->overlays) {
+    g_list_free_full (priv->overlays,
+        (GDestroyNotify) gst_d3d11_composition_overlay_free);
 
-  GST_D3D11_CLEAR_COM (compositor->ps);
-  GST_D3D11_CLEAR_COM (compositor->vs);
-  GST_D3D11_CLEAR_COM (compositor->layout);
-  GST_D3D11_CLEAR_COM (compositor->sampler);
-  GST_D3D11_CLEAR_COM (compositor->blend);
-  GST_D3D11_CLEAR_COM (compositor->index_buffer);
-
-  gst_clear_object (&compositor->device);
-  g_free (compositor);
+    priv->overlays = nullptr;
+  }
 }
 
 static gint
@@ -519,15 +545,17 @@ gboolean
 gst_d3d11_overlay_compositor_upload (GstD3D11OverlayCompositor * compositor,
     GstBuffer * buf)
 {
+  GstD3D11OverlayCompositorPrivate *priv;
   GstVideoOverlayCompositionMeta *meta;
   gint i, num_overlays;
   GList *iter;
 
-  g_return_val_if_fail (compositor != NULL, FALSE);
+  g_return_val_if_fail (compositor != nullptr, FALSE);
   g_return_val_if_fail (GST_IS_BUFFER (buf), FALSE);
 
-  meta = gst_buffer_get_video_overlay_composition_meta (buf);
+  priv = compositor->priv;
 
+  meta = gst_buffer_get_video_overlay_composition_meta (buf);
   if (!meta) {
     gst_d3d11_overlay_compositor_free_overlays (compositor);
     return TRUE;
@@ -539,35 +567,35 @@ gst_d3d11_overlay_compositor_upload (GstD3D11OverlayCompositor * compositor,
     return TRUE;
   }
 
-  GST_LOG ("Upload %d overlay rectangles", num_overlays);
+  GST_LOG_OBJECT (compositor, "Upload %d overlay rectangles", num_overlays);
 
   /* Upload new overlay */
   for (i = 0; i < num_overlays; i++) {
     GstVideoOverlayRectangle *rectangle =
         gst_video_overlay_composition_get_rectangle (meta->overlay, i);
 
-    if (!g_list_find_custom (compositor->overlays,
+    if (!g_list_find_custom (priv->overlays,
             rectangle, (GCompareFunc) find_in_compositor)) {
-      GstD3D11CompositionOverlay *overlay = NULL;
+      GstD3D11CompositionOverlay *overlay = nullptr;
 
       overlay = gst_d3d11_composition_overlay_new (compositor, rectangle);
 
       if (!overlay)
         return FALSE;
 
-      compositor->overlays = g_list_append (compositor->overlays, overlay);
+      priv->overlays = g_list_append (priv->overlays, overlay);
     }
   }
 
   /* Remove old overlay */
-  iter = compositor->overlays;
+  iter = priv->overlays;
   while (iter) {
     GstD3D11CompositionOverlay *overlay =
         (GstD3D11CompositionOverlay *) iter->data;
     GList *next = iter->next;
 
     if (!is_in_video_overlay_composition (meta->overlay, overlay)) {
-      compositor->overlays = g_list_delete_link (compositor->overlays, iter);
+      priv->overlays = g_list_delete_link (priv->overlays, iter);
       gst_d3d11_composition_overlay_free (overlay);
     }
 
@@ -577,28 +605,14 @@ gst_d3d11_overlay_compositor_upload (GstD3D11OverlayCompositor * compositor,
   return TRUE;
 }
 
-void
-gst_d3d11_overlay_compositor_free_overlays (GstD3D11OverlayCompositor *
-    compositor)
-{
-  g_return_if_fail (compositor != NULL);
-
-  if (compositor->overlays) {
-    g_list_free_full (compositor->overlays,
-        (GDestroyNotify) gst_d3d11_composition_overlay_free);
-
-    compositor->overlays = NULL;
-  }
-}
-
 gboolean
 gst_d3d11_overlay_compositor_update_viewport (GstD3D11OverlayCompositor *
     compositor, D3D11_VIEWPORT * viewport)
 {
-  g_return_val_if_fail (compositor != NULL, FALSE);
-  g_return_val_if_fail (viewport != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_D3D11_OVERLAY_COMPOSITOR (compositor), FALSE);
+  g_return_val_if_fail (viewport != nullptr, FALSE);
 
-  compositor->viewport = *viewport;
+  compositor->priv->viewport = *viewport;
 
   return TRUE;
 }
@@ -607,39 +621,56 @@ gboolean
 gst_d3d11_overlay_compositor_draw (GstD3D11OverlayCompositor * compositor,
     ID3D11RenderTargetView * rtv[GST_VIDEO_MAX_PLANES])
 {
-  gboolean ret = TRUE;
+  g_return_val_if_fail (compositor != nullptr, FALSE);
+  g_return_val_if_fail (rtv != nullptr, FALSE);
 
-  g_return_val_if_fail (compositor != NULL, FALSE);
-  g_return_val_if_fail (rtv != NULL, FALSE);
-
-  gst_d3d11_device_lock (compositor->device);
-  ret = gst_d3d11_overlay_compositor_draw_unlocked (compositor, rtv);
-  gst_d3d11_device_unlock (compositor->device);
-
-  return ret;
+  GstD3D11DeviceLockGuard lk (compositor->device);
+  return gst_d3d11_overlay_compositor_draw_unlocked (compositor, rtv);
 }
 
 gboolean
 gst_d3d11_overlay_compositor_draw_unlocked (GstD3D11OverlayCompositor *
     compositor, ID3D11RenderTargetView * rtv[GST_VIDEO_MAX_PLANES])
 {
-  gboolean ret = TRUE;
+  GstD3D11OverlayCompositorPrivate *priv;
   GList *iter;
+  ID3D11DeviceContext *context;
+  ID3D11ShaderResourceView *clear_view[GST_VIDEO_MAX_PLANES] = { nullptr, };
+  UINT strides = sizeof (VertexData);
+  UINT offsets = 0;
 
-  g_return_val_if_fail (compositor != NULL, FALSE);
-  g_return_val_if_fail (rtv != NULL, FALSE);
+  g_return_val_if_fail (compositor != nullptr, FALSE);
+  g_return_val_if_fail (rtv != nullptr, FALSE);
 
-  for (iter = compositor->overlays; iter; iter = g_list_next (iter)) {
+  priv = compositor->priv;
+
+  if (!priv->overlays)
+    return TRUE;
+
+  context = gst_d3d11_device_get_device_context_handle (compositor->device);
+  context->IASetPrimitiveTopology (D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
+  context->IASetInputLayout (priv->layout);
+  context->IASetIndexBuffer (priv->index_buffer, DXGI_FORMAT_R16_UINT, 0);
+  context->PSSetSamplers (0, 1, &priv->sampler);
+  context->VSSetShader (priv->vs, nullptr, 0);
+  context->PSSetShader (priv->ps, nullptr, 0);
+  context->RSSetViewports (1, &priv->viewport);
+  context->OMSetRenderTargets (1, rtv, nullptr);
+  context->OMSetBlendState (priv->blend, nullptr, 0xffffffff);
+
+  for (iter = priv->overlays; iter; iter = g_list_next (iter)) {
     GstD3D11CompositionOverlay *overlay =
         (GstD3D11CompositionOverlay *) iter->data;
 
-    ret = gst_d3d11_draw_quad_unlocked (overlay->quad,
-        &compositor->viewport, 1, &overlay->srv, 1, rtv, 1,
-        compositor->blend, NULL, &compositor->sampler, 1);
+    context->PSSetShaderResources (0, 1, &overlay->srv);
+    context->IASetVertexBuffers (0,
+        1, &overlay->vertex_buffer, &strides, &offsets);
 
-    if (!ret)
-      break;
+    context->DrawIndexed (6, 0, 0);
   }
 
-  return ret;
+  context->PSSetShaderResources (0, 1, clear_view);
+  context->OMSetRenderTargets (0, nullptr, nullptr);
+
+  return TRUE;
 }
