@@ -84,9 +84,6 @@
 GST_DEBUG_CATEGORY_EXTERN (gst_d3d11_vp9_dec_debug);
 #define GST_CAT_DEFAULT gst_d3d11_vp9_dec_debug
 
-/* reference list 8 + 4 margin */
-#define NUM_OUTPUT_VIEW 12
-
 /* *INDENT-OFF* */
 typedef struct _GstD3D11Vp9DecInner
 {
@@ -143,7 +140,7 @@ static gboolean gst_d3d11_vp9_dec_sink_event (GstVideoDecoder * decoder,
 
 /* GstVp9Decoder */
 static GstFlowReturn gst_d3d11_vp9_dec_new_sequence (GstVp9Decoder * decoder,
-    const GstVp9FrameHeader * frame_hdr);
+    const GstVp9FrameHeader * frame_hdr, gint max_dpb_size);
 static GstFlowReturn gst_d3d11_vp9_dec_new_picture (GstVp9Decoder * decoder,
     GstVideoCodecFrame * frame, GstVp9Picture * picture);
 static GstVp9Picture *gst_d3d11_vp9_dec_duplicate_picture (GstVp9Decoder *
@@ -178,7 +175,7 @@ gst_d3d11_vp9_dec_class_init (GstD3D11Vp9DecClass * klass, gpointer data)
   /**
    * GstD3D11Vp9Dec:adapter-luid:
    *
-   * DXGI Adapter LUID for this elemenet
+   * DXGI Adapter LUID for this element
    *
    * Since: 1.20
    */
@@ -361,7 +358,7 @@ gst_d3d11_vp9_dec_sink_event (GstVideoDecoder * decoder, GstEvent * event)
 
 static GstFlowReturn
 gst_d3d11_vp9_dec_new_sequence (GstVp9Decoder * decoder,
-    const GstVp9FrameHeader * frame_hdr)
+    const GstVp9FrameHeader * frame_hdr, gint max_dpb_size)
 {
   GstD3D11Vp9Dec *self = GST_D3D11_VP9_DEC (decoder);
   GstD3D11Vp9DecInner *inner = self->inner;
@@ -384,14 +381,14 @@ gst_d3d11_vp9_dec_new_sequence (GstVp9Decoder * decoder,
       out_format, frame_hdr->width, frame_hdr->height);
 
   if (!gst_d3d11_decoder_configure (inner->d3d11_decoder,
-          decoder->input_state, &info, (gint) frame_hdr->width,
-          (gint) frame_hdr->height, NUM_OUTPUT_VIEW)) {
+          decoder->input_state, &info, 0, 0, (gint) frame_hdr->width,
+          (gint) frame_hdr->height, max_dpb_size)) {
     GST_ERROR_OBJECT (self, "Failed to create decoder");
     return GST_FLOW_NOT_NEGOTIATED;
   }
 
   if (!gst_video_decoder_negotiate (GST_VIDEO_DECODER (self))) {
-    GST_ERROR_OBJECT (self, "Failed to negotiate with downstream");
+    GST_WARNING_OBJECT (self, "Failed to negotiate with downstream");
     return GST_FLOW_NOT_NEGOTIATED;
   }
 
@@ -761,10 +758,8 @@ gst_d3d11_vp9_dec_end_picture (GstVp9Decoder * decoder, GstVp9Picture * picture)
   input_args.bitstream = &inner->bitstream_buffer[0];
   input_args.bitstream_size = inner->bitstream_buffer.size ();
 
-  if (!gst_d3d11_decoder_decode_frame (inner->d3d11_decoder, view, &input_args))
-    return GST_FLOW_ERROR;
-
-  return GST_FLOW_OK;
+  return gst_d3d11_decoder_decode_frame (inner->d3d11_decoder,
+      view, &input_args);
 }
 
 static GstFlowReturn
@@ -786,8 +781,8 @@ gst_d3d11_vp9_dec_output_picture (GstVp9Decoder * decoder,
   }
 
   if (!gst_d3d11_decoder_process_output (inner->d3d11_decoder, vdec,
-          picture->frame_hdr.width, picture->frame_hdr.height, view_buffer,
-          &frame->output_buffer)) {
+          picture->discont_state, picture->frame_hdr.width,
+          picture->frame_hdr.height, view_buffer, &frame->output_buffer)) {
     GST_ERROR_OBJECT (self, "Failed to copy buffer");
     goto error;
   }
@@ -828,13 +823,13 @@ gst_d3d11_vp9_dec_register (GstPlugin * plugin, GstD3D11Device * device,
   const GUID *profile0_guid = NULL;
   GstCaps *sink_caps = NULL;
   GstCaps *src_caps = NULL;
+  GstCaps *d3d11_caps = NULL;
   guint max_width = 0;
   guint max_height = 0;
   guint resolution;
   gboolean have_profile2 = FALSE;
   gboolean have_profile0 = FALSE;
   DXGI_FORMAT format = DXGI_FORMAT_UNKNOWN;
-  GValue vp9_profiles = G_VALUE_INIT;
 
   have_profile2 = gst_d3d11_decoder_get_supported_decoder_profile (device,
       GST_DXVA_CODEC_VP9, GST_VIDEO_FORMAT_P010_10LE, &profile2_guid);
@@ -895,60 +890,39 @@ gst_d3d11_vp9_dec_register (GstPlugin * plugin, GstD3D11Device * device,
     return;
   }
 
-  sink_caps = gst_caps_from_string ("video/x-vp9, alignment = (string) frame");
-  src_caps = gst_caps_from_string ("video/x-raw("
-      GST_CAPS_FEATURE_MEMORY_D3D11_MEMORY "); video/x-raw");
-
-  g_value_init (&vp9_profiles, GST_TYPE_LIST);
-
-  if (have_profile0) {
-    GValue vp9_profile_val = G_VALUE_INIT;
-
-    g_value_init (&vp9_profile_val, G_TYPE_STRING);
-    g_value_set_string (&vp9_profile_val, "0");
-    gst_value_list_append_and_take_value (&vp9_profiles, &vp9_profile_val);
-  }
-
-  if (have_profile2) {
-    GValue format_list = G_VALUE_INIT;
-    GValue format_value = G_VALUE_INIT;
-    GValue vp9_profile_val = G_VALUE_INIT;
-
-    g_value_init (&format_list, GST_TYPE_LIST);
-
-    g_value_init (&format_value, G_TYPE_STRING);
-    g_value_set_string (&format_value, "NV12");
-    gst_value_list_append_and_take_value (&format_list, &format_value);
-
-    g_value_init (&format_value, G_TYPE_STRING);
-    g_value_set_string (&format_value, "P010_10LE");
-    gst_value_list_append_and_take_value (&format_list, &format_value);
-
-    gst_caps_set_value (src_caps, "format", &format_list);
-    g_value_unset (&format_list);
-
-    g_value_init (&vp9_profile_val, G_TYPE_STRING);
-    g_value_set_string (&vp9_profile_val, "2");
-    gst_value_list_append_and_take_value (&vp9_profiles, &vp9_profile_val);
+  if (have_profile0 && have_profile2) {
+    sink_caps = gst_caps_from_string ("video/x-vp9, "
+        "alignment = (string) frame, profile = (string) 0; "
+        "video/x-vp9, alignment = (string) frame, profile = (string) 2, "
+        "bit-depth-luma = (uint) 10, bit-depth-chroma = (uint) 10");
+    src_caps = gst_caps_from_string ("video/x-raw, "
+        "format = (string) { NV12, P010_10LE }");
+  } else if (have_profile0) {
+    sink_caps = gst_caps_from_string ("video/x-vp9, "
+        "alignment = (string) frame, profile = (string) 0");
+    src_caps = gst_caps_from_string ("video/x-raw, " "format = (string) NV12");
+  } else if (have_profile2) {
+    sink_caps = gst_caps_from_string ("video/x-vp9, "
+        "alignment = (string) frame, profile = (string) 2, "
+        "bit-depth-luma = (uint) 10, bit-depth-chroma = (uint) 10");
+    src_caps = gst_caps_from_string ("video/x-raw, "
+        "format = (string) P010_10LE");
   } else {
-    gst_caps_set_simple (src_caps, "format", G_TYPE_STRING, "NV12", NULL);
+    g_assert_not_reached ();
+    return;
   }
 
-  gst_caps_set_value (sink_caps, "profile", &vp9_profiles);
-  g_value_unset (&vp9_profiles);
+  d3d11_caps = gst_caps_copy (src_caps);
+  gst_caps_set_features_simple (d3d11_caps,
+      gst_caps_features_new (GST_CAPS_FEATURE_MEMORY_D3D11_MEMORY, NULL));
+  src_caps = gst_caps_merge (d3d11_caps, src_caps);
 
   /* To cover both landscape and portrait, select max value */
   resolution = MAX (max_width, max_height);
-  gst_caps_set_simple (sink_caps,
-      "width", GST_TYPE_INT_RANGE, 1, resolution,
-      "height", GST_TYPE_INT_RANGE, 1, resolution, NULL);
-  gst_caps_set_simple (src_caps,
-      "width", GST_TYPE_INT_RANGE, 1, resolution,
-      "height", GST_TYPE_INT_RANGE, 1, resolution, NULL);
 
   type_info.class_data =
       gst_d3d11_decoder_class_data_new (device, GST_DXVA_CODEC_VP9,
-      sink_caps, src_caps);
+      sink_caps, src_caps, resolution);
 
   type_name = g_strdup ("GstD3D11Vp9Dec");
   feature_name = g_strdup ("d3d11vp9dec");
