@@ -30,6 +30,13 @@
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+/**
+ * SECTION: plugin-msdk
+ *
+ * Since: 1.12
+ *
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -45,11 +52,19 @@
 #include "gstmsdkmpeg2dec.h"
 #include "gstmsdkmpeg2enc.h"
 #include "gstmsdkvp8dec.h"
-#include "gstmsdkvp8enc.h"
+#ifdef USE_MSDK_VP9_ENC
+#include "gstmsdkvp9enc.h"
+#endif
 #include "gstmsdkvc1dec.h"
 #ifdef USE_MSDK_VP9_DEC
 #include "gstmsdkvp9dec.h"
 #endif
+#ifdef USE_MSDK_AV1_DEC
+#include "gstmsdkav1dec.h"
+#endif
+#ifdef USE_MSDK_AV1_ENC
+#include "gstmsdkav1enc.h"
+#endif
 #include "gstmsdkvpp.h"
 
 GST_DEBUG_CATEGORY (gst_msdk_debug);
@@ -65,9 +80,36 @@ GST_DEBUG_CATEGORY (gst_msdkmjpegenc_debug);
 GST_DEBUG_CATEGORY (gst_msdkmpeg2enc_debug);
 GST_DEBUG_CATEGORY (gst_msdkmpeg2dec_debug);
 GST_DEBUG_CATEGORY (gst_msdkvp8dec_debug);
-GST_DEBUG_CATEGORY (gst_msdkvp8enc_debug);
 GST_DEBUG_CATEGORY (gst_msdkvc1dec_debug);
 GST_DEBUG_CATEGORY (gst_msdkvp9dec_debug);
+GST_DEBUG_CATEGORY (gst_msdkvp9enc_debug);
+GST_DEBUG_CATEGORY (gst_msdkav1dec_debug);
+GST_DEBUG_CATEGORY (gst_msdkav1enc_debug);
+
+static void
+plugin_add_dependencies (GstPlugin * plugin)
+{
+#ifndef _WIN32
+  const gchar *env_vars[] = { "LIBVA_DRIVER_NAME", NULL };
+  const gchar *kernel_paths[] = { "/dev/dri", NULL };
+  const gchar *kernel_names[] = { "card", "render", NULL };
+
+  /* features get updated upon changes in /dev/dri/card* */
+  gst_plugin_add_dependency (plugin, NULL, kernel_paths, kernel_names,
+      GST_PLUGIN_DEPENDENCY_FLAG_FILE_NAME_IS_PREFIX);
+
+  /* features get updated upon changes in VA environment variables */
+  gst_plugin_add_dependency (plugin, env_vars, NULL, NULL,
+      GST_PLUGIN_DEPENDENCY_FLAG_NONE);
+
+  /* features get updated upon changes in default VA drivers
+   * directory */
+  gst_plugin_add_dependency_simple (plugin, "LIBVA_DRIVERS_PATH",
+      VA_DRIVERS_PATH, "_drv_video.so",
+      GST_PLUGIN_DEPENDENCY_FLAG_FILE_NAME_IS_SUFFIX |
+      GST_PLUGIN_DEPENDENCY_FLAG_PATHS_ARE_DEFAULT_ONLY);
+#endif
+}
 
 static gboolean
 plugin_init (GstPlugin * plugin)
@@ -95,12 +137,16 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (gst_msdkmpeg2dec_debug, "msdkmpeg2dec", 0,
       "msdkmpeg2dec");
   GST_DEBUG_CATEGORY_INIT (gst_msdkvp8dec_debug, "msdkvp8dec", 0, "msdkvp8dec");
-  GST_DEBUG_CATEGORY_INIT (gst_msdkvp8enc_debug, "msdkvp8enc", 0, "msdkvp8enc");
   GST_DEBUG_CATEGORY_INIT (gst_msdkvc1dec_debug, "msdkvc1dec", 0, "msdkvc1dec");
   GST_DEBUG_CATEGORY_INIT (gst_msdkvp9dec_debug, "msdkvp9dec", 0, "msdkvp9dec");
+  GST_DEBUG_CATEGORY_INIT (gst_msdkvp9enc_debug, "msdkvp9enc", 0, "msdkvp9enc");
+  GST_DEBUG_CATEGORY_INIT (gst_msdkav1dec_debug, "msdkav1dec", 0, "msdkav1dec");
+  GST_DEBUG_CATEGORY_INIT (gst_msdkav1dec_debug, "msdkav1enc", 0, "msdkav1enc");
+
+  plugin_add_dependencies (plugin);
 
   if (!msdk_is_available ())
-    return FALSE;
+    return TRUE;                /* return TRUE to avoid getting blacklisted */
 
   ret = gst_element_register (plugin, "msdkh264dec", GST_RANK_NONE,
       GST_TYPE_MSDKH264DEC);
@@ -129,16 +175,24 @@ plugin_init (GstPlugin * plugin)
   ret = gst_element_register (plugin, "msdkvp8dec", GST_RANK_NONE,
       GST_TYPE_MSDKVP8DEC);
 
-  ret = gst_element_register (plugin, "msdkvp8enc", GST_RANK_NONE,
-      GST_TYPE_MSDKVP8ENC);
-
   ret = gst_element_register (plugin, "msdkvc1dec", GST_RANK_NONE,
       GST_TYPE_MSDKVC1DEC);
 #ifdef USE_MSDK_VP9_DEC
   ret = gst_element_register (plugin, "msdkvp9dec", GST_RANK_NONE,
       GST_TYPE_MSDKVP9DEC);
 #endif
-
+#ifdef USE_MSDK_VP9_ENC
+  ret = gst_element_register (plugin, "msdkvp9enc", GST_RANK_NONE,
+      GST_TYPE_MSDKVP9ENC);
+#endif
+#ifdef USE_MSDK_AV1_DEC
+  ret = gst_element_register (plugin, "msdkav1dec", GST_RANK_NONE,
+      GST_TYPE_MSDKAV1DEC);
+#endif
+#ifdef USE_MSDK_AV1_ENC
+  ret = gst_element_register (plugin, "msdkav1enc", GST_RANK_NONE,
+      GST_TYPE_MSDKAV1ENC);
+#endif
   ret = gst_element_register (plugin, "msdkvpp", GST_RANK_NONE,
       GST_TYPE_MSDKVPP);
 
@@ -148,5 +202,5 @@ plugin_init (GstPlugin * plugin)
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     msdk,
-    "Intel Media SDK based elements",
+    "MFX API (" MFX_API_SDK ") based elements",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
