@@ -312,7 +312,7 @@ msdk_init_msdk_session (mfxIMPL impl, mfxVersion * pver,
 }
 
 void
-MFXUnload (mfxLoader loader)
+GstMFXUnload (mfxLoader loader)
 {
   g_assert (loader == NULL);
 }
@@ -553,6 +553,18 @@ gst_msdk_is_msdk_buffer (GstBuffer * buf)
     return FALSE;
 }
 
+gboolean
+gst_msdk_is_va_mem (GstMemory * mem)
+{
+  GstAllocator *allocator;
+
+  allocator = mem->allocator;
+  if (!allocator)
+    return FALSE;
+
+  return g_str_equal (allocator->mem_type, "VAMemory");
+}
+
 mfxFrameSurface1 *
 gst_msdk_get_surface_from_buffer (GstBuffer * buf)
 {
