@@ -29,9 +29,11 @@
 #include "config.h"
 #endif
 
+#include <gst/cuda/gstcudautils.h>
+#include <gst/cuda/gstcudabufferpool.h>
+
+#include "gstcuvidloader.h"
 #include "gstnvdec.h"
-#include "gstcudautils.h"
-#include "gstcudabufferpool.h"
 
 #include <string.h>
 
@@ -44,6 +46,7 @@ enum
 {
   PROP_0,
   PROP_MAX_DISPLAY_DELAY,
+  PROP_CUDA_DEVICE_ID,
 };
 
 #ifdef HAVE_NVCODEC_GST_GL
@@ -195,11 +198,15 @@ gst_nv_dec_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
 {
   GstNvDec *nvdec = GST_NVDEC (object);
+  GstNvDecClass *klass = GST_NVDEC_GET_CLASS (nvdec);
 
   switch (prop_id) {
     case PROP_MAX_DISPLAY_DELAY:
       g_value_set_int (value, nvdec->max_display_delay);
       break;
+    case PROP_CUDA_DEVICE_ID:
+      g_value_set_uint (value, klass->cuda_device_id);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -245,6 +252,18 @@ gst_nvdec_class_init (GstNvDecClass * klass)
           "(auto = -1)",
           -1, G_MAXINT, DEFAULT_MAX_DISPLAY_DELAY,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstNvDec:cuda-device-id:
+   *
+   * Assigned CUDA device id
+   *
+   * Since: 1.22
+   */
+  g_object_class_install_property (gobject_class, PROP_CUDA_DEVICE_ID,
+      g_param_spec_uint ("cuda-device-id", "CUDA device id",
+          "Assigned CUDA device id", 0, G_MAXINT, 0,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -527,8 +546,8 @@ parser_sequence_callback (GstNvDec * nvdec, CUVIDEOFORMAT * format)
     }
 
     GST_DEBUG_OBJECT (nvdec, "creating decoder");
-    create_info.ulWidth = width;
-    create_info.ulHeight = height;
+    create_info.ulWidth = format->coded_width;
+    create_info.ulHeight = format->coded_height;
     create_info.ulNumDecodeSurfaces = nvdec->num_decode_surface;
     create_info.CodecType = format->codec;
     create_info.ChromaFormat = format->chroma_format;
@@ -1148,6 +1167,7 @@ copy_video_frame_to_gl_textures (GstGLContext * context,
   proc_params.progressive_frame = dispinfo->progressive_frame;
   proc_params.top_field_first = dispinfo->top_field_first;
   proc_params.unpaired_field = dispinfo->repeat_first_field == -1;
+  proc_params.output_stream = nvdec->cuda_stream;
 
   data->ret = TRUE;
 
@@ -1265,40 +1285,32 @@ gst_nvdec_copy_device_to_memory (GstNvDec * nvdec,
   GstVideoInfo *info = &nvdec->output_state->info;
   gint i;
   GstMemory *mem;
-  GstCudaMemory *cuda_mem = NULL;
-
-  if (!gst_cuda_context_push (nvdec->cuda_ctx)) {
-    GST_WARNING_OBJECT (nvdec, "failed to lock CUDA context");
-    return FALSE;
-  }
+  gboolean use_device_copy = FALSE;
+  GstMapFlags map_flags = GST_MAP_WRITE;
 
   if (nvdec->mem_type == GST_NVDEC_MEM_TYPE_CUDA &&
       (mem = gst_buffer_peek_memory (output_buffer, 0)) &&
       gst_is_cuda_memory (mem)) {
-    GstCudaMemory *cmem = GST_CUDA_MEMORY_CAST (mem);
-
-    if (cmem->context == nvdec->cuda_ctx ||
-        gst_cuda_context_get_handle (cmem->context) ==
-        gst_cuda_context_get_handle (nvdec->cuda_ctx) ||
-        (gst_cuda_context_can_access_peer (cmem->context, nvdec->cuda_ctx) &&
-            gst_cuda_context_can_access_peer (nvdec->cuda_ctx,
-                cmem->context))) {
-      cuda_mem = cmem;
-    }
+    map_flags |= GST_MAP_CUDA;
+    use_device_copy = TRUE;
   }
 
-  if (!cuda_mem) {
-    if (!gst_video_frame_map (&video_frame, info, output_buffer, GST_MAP_WRITE)) {
-      GST_ERROR_OBJECT (nvdec, "frame map failure");
-      gst_cuda_context_pop (NULL);
-      return FALSE;
-    }
+  if (!gst_video_frame_map (&video_frame, info, output_buffer, map_flags)) {
+    GST_ERROR_OBJECT (nvdec, "frame map failure");
+    return FALSE;
+  }
+
+  if (!gst_cuda_context_push (nvdec->cuda_ctx)) {
+    gst_video_frame_unmap (&video_frame);
+    GST_WARNING_OBJECT (nvdec, "failed to lock CUDA context");
+    return FALSE;
   }
 
   params.progressive_frame = dispinfo->progressive_frame;
   params.second_field = dispinfo->repeat_first_field + 1;
   params.top_field_first = dispinfo->top_field_first;
   params.unpaired_field = dispinfo->repeat_first_field < 0;
+  params.output_stream = nvdec->cuda_stream;
 
   if (!gst_cuda_result (CuvidMapVideoFrame (nvdec->decoder,
               dispinfo->picture_index, &dptr, &pitch, &params))) {
@@ -1310,17 +1322,17 @@ gst_nvdec_copy_device_to_memory (GstNvDec * nvdec,
   copy_params.srcMemoryType = CU_MEMORYTYPE_DEVICE;
   copy_params.srcPitch = pitch;
   copy_params.dstMemoryType =
-      cuda_mem ? CU_MEMORYTYPE_DEVICE : CU_MEMORYTYPE_HOST;
+      use_device_copy ? CU_MEMORYTYPE_DEVICE : CU_MEMORYTYPE_HOST;
 
   for (i = 0; i < GST_VIDEO_INFO_N_PLANES (info); i++) {
     copy_params.srcDevice = dptr + (i * pitch * GST_VIDEO_INFO_HEIGHT (info));
-    if (cuda_mem) {
-      copy_params.dstDevice = cuda_mem->data + cuda_mem->offset[i];
-      copy_params.dstPitch = cuda_mem->stride;
+    if (use_device_copy) {
+      copy_params.dstDevice =
+          (CUdeviceptr) GST_VIDEO_FRAME_PLANE_DATA (&video_frame, i);
     } else {
       copy_params.dstHost = GST_VIDEO_FRAME_PLANE_DATA (&video_frame, i);
-      copy_params.dstPitch = GST_VIDEO_FRAME_PLANE_STRIDE (&video_frame, i);
     }
+    copy_params.dstPitch = GST_VIDEO_FRAME_PLANE_STRIDE (&video_frame, i);
     copy_params.WidthInBytes = GST_VIDEO_INFO_COMP_WIDTH (info, i)
         * GST_VIDEO_INFO_COMP_PSTRIDE (info, i);
     copy_params.Height = GST_VIDEO_INFO_COMP_HEIGHT (info, i);
@@ -1328,8 +1340,7 @@ gst_nvdec_copy_device_to_memory (GstNvDec * nvdec,
     if (!gst_cuda_result (CuMemcpy2DAsync (&copy_params, nvdec->cuda_stream))) {
       GST_ERROR_OBJECT (nvdec, "failed to copy %dth plane", i);
       CuvidUnmapVideoFrame (nvdec->decoder, dptr);
-      if (!cuda_mem)
-        gst_video_frame_unmap (&video_frame);
+      gst_video_frame_unmap (&video_frame);
       gst_cuda_context_pop (NULL);
       return FALSE;
     }
@@ -1337,8 +1348,7 @@ gst_nvdec_copy_device_to_memory (GstNvDec * nvdec,
 
   gst_cuda_result (CuStreamSynchronize (nvdec->cuda_stream));
 
-  if (!cuda_mem)
-    gst_video_frame_unmap (&video_frame);
+  gst_video_frame_unmap (&video_frame);
 
   if (!gst_cuda_result (CuvidUnmapVideoFrame (nvdec->decoder, dptr)))
     GST_WARNING_OBJECT (nvdec, "failed to unmap video frame");
@@ -1884,9 +1894,15 @@ gst_nvdec_ensure_cuda_pool (GstNvDec * nvdec, GstQuery * query)
   n = gst_query_get_n_allocation_pools (query);
   if (n > 0) {
     gst_query_parse_nth_allocation_pool (query, 0, &pool, &size, &min, &max);
-    if (pool && !GST_IS_CUDA_BUFFER_POOL (pool)) {
-      gst_object_unref (pool);
-      pool = NULL;
+    if (pool) {
+      if (!GST_IS_CUDA_BUFFER_POOL (pool)) {
+        gst_clear_object (&pool);
+      } else {
+        GstCudaBufferPool *cpool = GST_CUDA_BUFFER_POOL (pool);
+
+        if (cpool->context != nvdec->cuda_ctx)
+          gst_clear_object (&pool);
+      }
     }
   }
 
@@ -1904,6 +1920,12 @@ gst_nvdec_ensure_cuda_pool (GstNvDec * nvdec, GstQuery * query)
   gst_buffer_pool_config_set_params (config, outcaps, size, min, max);
   gst_buffer_pool_config_add_option (config, GST_BUFFER_POOL_OPTION_VIDEO_META);
   gst_buffer_pool_set_config (pool, config);
+
+  /* Get updated size by cuda buffer pool */
+  config = gst_buffer_pool_get_config (pool);
+  gst_buffer_pool_config_get_params (config, NULL, &size, NULL, NULL);
+  gst_structure_free (config);
+
   if (n > 0)
     gst_query_set_nth_allocation_pool (query, 0, pool, size, min, max);
   else
@@ -2047,6 +2069,7 @@ gst_nvdec_subclass_register (GstPlugin * plugin, GType type,
   gchar *type_name;
   GstNvDecClassData *cdata;
   gboolean is_default = TRUE;
+  gint index = 0;
 
   cdata = g_new0 (GstNvDecClassData, 1);
   cdata->sink_caps = gst_caps_ref (sink_caps);
@@ -2063,10 +2086,10 @@ gst_nvdec_subclass_register (GstPlugin * plugin, GType type,
   type_info.class_data = cdata;
 
   type_name = g_strdup_printf ("nv%sdec", codec);
-
-  if (g_type_from_name (type_name) != 0) {
+  while (g_type_from_name (type_name)) {
+    index++;
     g_free (type_name);
-    type_name = g_strdup_printf ("nv%sdevice%ddec", codec, device_id);
+    type_name = g_strdup_printf ("nv%sdevice%ddec", codec, index);
     is_default = FALSE;
   }
 
