@@ -75,7 +75,7 @@ struct _GstV4l2CodecH264Dec
   GstV4l2CodecPool *src_pool;
   gint min_pool_size;
   gboolean has_videometa;
-  gboolean need_negotiation;
+  gboolean streaming;
   gboolean interlaced;
   gboolean need_sequence;
   gboolean copy_frames;
@@ -233,6 +233,16 @@ gst_v4l2_codec_h264_dec_close (GstVideoDecoder * decoder)
   return TRUE;
 }
 
+static void
+gst_v4l2_codec_h264_dec_streamoff (GstV4l2CodecH264Dec * self)
+{
+  if (self->streaming) {
+    gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SINK);
+    gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SRC);
+    self->streaming = FALSE;
+  }
+}
+
 static void
 gst_v4l2_codec_h264_dec_reset_allocation (GstV4l2CodecH264Dec * self)
 {
@@ -253,9 +263,7 @@ gst_v4l2_codec_h264_dec_stop (GstVideoDecoder * decoder)
 {
   GstV4l2CodecH264Dec *self = GST_V4L2_CODEC_H264_DEC (decoder);
 
-  gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SINK);
-  gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SRC);
-
+  gst_v4l2_codec_h264_dec_streamoff (self);
   gst_v4l2_codec_h264_dec_reset_allocation (self);
 
   if (self->output_state)
@@ -314,15 +322,11 @@ gst_v4l2_codec_h264_dec_negotiate (GstVideoDecoder * decoder)
   GstCaps *filter, *caps;
 
   /* Ignore downstream renegotiation request. */
-  if (!self->need_negotiation)
-    return TRUE;
-  self->need_negotiation = FALSE;
+  if (self->streaming)
+    goto done;
 
   GST_DEBUG_OBJECT (self, "Negotiate");
 
-  gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SINK);
-  gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SRC);
-
   gst_v4l2_codec_h264_dec_reset_allocation (self);
 
   if (!gst_v4l2_decoder_set_sink_fmt (self->decoder, V4L2_PIX_FMT_H264_SLICE,
@@ -366,6 +370,7 @@ gst_v4l2_codec_h264_dec_negotiate (GstVideoDecoder * decoder)
   if (self->output_state)
     gst_video_codec_state_unref (self->output_state);
 
+done:
   self->output_state =
       gst_video_decoder_set_output_state (GST_VIDEO_DECODER (self),
       self->vinfo.finfo->format, self->display_width,
@@ -377,6 +382,9 @@ gst_v4l2_codec_h264_dec_negotiate (GstVideoDecoder * decoder)
   self->output_state->caps = gst_video_info_to_caps (&self->output_state->info);
 
   if (GST_VIDEO_DECODER_CLASS (parent_class)->negotiate (decoder)) {
+    if (self->streaming)
+      return TRUE;
+
     if (!gst_v4l2_decoder_streamon (self->decoder, GST_PAD_SINK)) {
       GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
           ("Could not enable the decoder driver."),
@@ -391,6 +399,8 @@ gst_v4l2_codec_h264_dec_negotiate (GstVideoDecoder * decoder)
       return FALSE;
     }
 
+    self->streaming = TRUE;
+
     return TRUE;
   }
 
@@ -404,6 +414,11 @@ gst_v4l2_codec_h264_dec_decide_allocation (GstVideoDecoder * decoder,
   GstV4l2CodecH264Dec *self = GST_V4L2_CODEC_H264_DEC (decoder);
   guint min = 0, num_bitstream;
 
+  /* If we are streaming here, then it means there is nothing allocation
+   * related in the new state and allocation can be ignored */
+  if (self->streaming)
+    return TRUE;
+
   self->has_videometa = gst_query_find_allocation_meta (query,
       GST_VIDEO_META_API_TYPE, NULL);
 
@@ -559,19 +574,26 @@ gst_v4l2_codec_h264_dec_fill_decoder_params (GstV4l2CodecH264Dec * self,
              (slice_hdr->field_pic_flag ? V4L2_H264_DECODE_PARAM_FLAG_FIELD_PIC : 0) |
              (slice_hdr->bottom_field_flag ? V4L2_H264_DECODE_PARAM_FLAG_BOTTOM_FIELD : 0),
   };
+  /* *INDENT-ON* */
 
   switch (picture->field) {
     case GST_H264_PICTURE_FIELD_FRAME:
       self->decode_params.top_field_order_cnt = picture->top_field_order_cnt;
       self->decode_params.bottom_field_order_cnt =
-        picture->bottom_field_order_cnt;
+          picture->bottom_field_order_cnt;
       break;
     case GST_H264_PICTURE_FIELD_TOP_FIELD:
       self->decode_params.top_field_order_cnt = picture->top_field_order_cnt;
       self->decode_params.bottom_field_order_cnt = 0;
+      if (picture->other_field)
+        self->decode_params.bottom_field_order_cnt =
+            picture->other_field->bottom_field_order_cnt;
       break;
     case GST_H264_PICTURE_FIELD_BOTTOM_FIELD:
       self->decode_params.top_field_order_cnt = 0;
+      if (picture->other_field)
+        self->decode_params.top_field_order_cnt =
+            picture->other_field->top_field_order_cnt;
       self->decode_params.bottom_field_order_cnt =
           picture->bottom_field_order_cnt;
       break;
@@ -599,6 +621,7 @@ gst_v4l2_codec_h264_dec_fill_decoder_params (GstV4l2CodecH264Dec * self,
     }
 
     entry = &self->decode_params.dpb[entry_id++];
+    /* *INDENT-OFF* */
     *entry = (struct v4l2_h264_dpb_entry) {
       /*
        * The reference is multiplied by 1000 because it's was set as micro
@@ -609,8 +632,10 @@ gst_v4l2_codec_h264_dec_fill_decoder_params (GstV4l2CodecH264Dec * self,
       .pic_num = pic_num,
       .flags = V4L2_H264_DPB_ENTRY_FLAG_VALID
           | (GST_H264_PICTURE_IS_REF (ref_pic) ? V4L2_H264_DPB_ENTRY_FLAG_ACTIVE : 0)
-          | (GST_H264_PICTURE_IS_LONG_TERM_REF (ref_pic) ? V4L2_H264_DPB_ENTRY_FLAG_LONG_TERM : 0),
+          | (GST_H264_PICTURE_IS_LONG_TERM_REF (ref_pic) ? V4L2_H264_DPB_ENTRY_FLAG_LONG_TERM : 0)
+          | (ref_pic->field_pic_flag ? V4L2_H264_DPB_ENTRY_FLAG_FIELD : 0),
     };
+    /* *INDENT-ON* */
 
     switch (ref_pic->field) {
       case GST_H264_PICTURE_FIELD_FRAME:
@@ -626,8 +651,6 @@ gst_v4l2_codec_h264_dec_fill_decoder_params (GstV4l2CodecH264Dec * self,
           entry->bottom_field_order_cnt =
               ref_pic->other_field->bottom_field_order_cnt;
           entry->fields |= V4L2_H264_BOTTOM_FIELD_REF;
-        } else {
-          entry->flags |= V4L2_H264_DPB_ENTRY_FLAG_FIELD;
         }
         break;
       case GST_H264_PICTURE_FIELD_BOTTOM_FIELD:
@@ -636,15 +659,12 @@ gst_v4l2_codec_h264_dec_fill_decoder_params (GstV4l2CodecH264Dec * self,
 
         if (ref_pic->other_field) {
           entry->top_field_order_cnt =
-            ref_pic->other_field->top_field_order_cnt;
+              ref_pic->other_field->top_field_order_cnt;
           entry->fields |= V4L2_H264_TOP_FIELD_REF;
-        } else {
-          entry->flags |= V4L2_H264_DPB_ENTRY_FLAG_FIELD;
         }
         break;
     }
   }
-  /* *INDENT-ON* */
 
   g_array_unref (refs);
 }
@@ -889,7 +909,7 @@ gst_v4l2_codec_h264_dec_new_sequence (GstH264Decoder * decoder,
   self->need_sequence = TRUE;
 
   if (negotiation_needed) {
-    self->need_negotiation = TRUE;
+    gst_v4l2_codec_h264_dec_streamoff (self);
     if (!gst_video_decoder_negotiate (GST_VIDEO_DECODER (self))) {
       GST_ERROR_OBJECT (self, "Failed to negotiate with downstream");
       return GST_FLOW_NOT_NEGOTIATED;
@@ -979,6 +999,7 @@ gst_v4l2_codec_h264_dec_start_picture (GstH264Decoder * decoder,
       dpb);
 
   self->first_slice = TRUE;
+  self->num_slices = 0;
 
   return GST_FLOW_OK;
 }
@@ -1040,6 +1061,13 @@ gst_v4l2_codec_h264_dec_output_picture (GstH264Decoder * decoder,
   GstV4l2Request *request = gst_h264_picture_get_user_data (picture);
   gint ret;
 
+  if (picture->discont_state) {
+    if (!gst_video_decoder_negotiate (vdec)) {
+      GST_ERROR_OBJECT (vdec, "Could not re-negotiate with updated state");
+      return FALSE;
+    }
+  }
+
   GST_DEBUG_OBJECT (self, "Output picture %u", picture->system_frame_number);
 
   ret = gst_v4l2_request_set_done (request);
@@ -1124,7 +1152,7 @@ gst_v4l2_codec_h264_dec_submit_bitstream (GstV4l2CodecH264Dec * self,
   GstV4l2Request *prev_request, *request = NULL;
   gsize bytesused;
   gboolean ret = FALSE;
-  guint count = 0;
+  guint num_controls = 0;
 
   /* *INDENT-OFF* */
   /* Reserve space for controls */
@@ -1171,49 +1199,53 @@ gst_v4l2_codec_h264_dec_submit_bitstream (GstV4l2CodecH264Dec * self,
   }
 
   if (self->need_sequence) {
-    control[count].id = V4L2_CID_STATELESS_H264_SPS;
-    control[count].ptr = &self->sps;
-    control[count].size = sizeof (self->sps);
-    count++;
+    control[num_controls].id = V4L2_CID_STATELESS_H264_SPS;
+    control[num_controls].ptr = &self->sps;
+    control[num_controls].size = sizeof (self->sps);
+    num_controls++;
     self->need_sequence = FALSE;
   }
 
   if (self->first_slice) {
-    control[count].id = V4L2_CID_STATELESS_H264_PPS;
-    control[count].ptr = &self->pps;
-    control[count].size = sizeof (self->pps);
-    count++;
+    control[num_controls].id = V4L2_CID_STATELESS_H264_PPS;
+    control[num_controls].ptr = &self->pps;
+    control[num_controls].size = sizeof (self->pps);
+    num_controls++;
 
     if (self->scaling_matrix_present) {
-      control[count].id = V4L2_CID_STATELESS_H264_SCALING_MATRIX;
-      control[count].ptr = &self->scaling_matrix;
-      control[count].size = sizeof (self->scaling_matrix);
-      count++;
+      control[num_controls].id = V4L2_CID_STATELESS_H264_SCALING_MATRIX;
+      control[num_controls].ptr = &self->scaling_matrix;
+      control[num_controls].size = sizeof (self->scaling_matrix);
+      num_controls++;
     }
 
-    control[count].id = V4L2_CID_STATELESS_H264_DECODE_PARAMS;
-    control[count].ptr = &self->decode_params;
-    control[count].size = sizeof (self->decode_params);
-    count++;
+    control[num_controls].id = V4L2_CID_STATELESS_H264_DECODE_PARAMS;
+    control[num_controls].ptr = &self->decode_params;
+    control[num_controls].size = sizeof (self->decode_params);
+    num_controls++;
 
     self->first_slice = FALSE;
   }
 
   /* If it's not slice-based then it doesn't support per-slice controls. */
   if (is_slice_based (self)) {
-    control[count].id = V4L2_CID_STATELESS_H264_SLICE_PARAMS;
-    control[count].ptr = self->slice_params->data;
-    control[count].size = g_array_get_element_size (self->slice_params)
+    control[num_controls].id = V4L2_CID_STATELESS_H264_SLICE_PARAMS;
+    control[num_controls].ptr = self->slice_params->data;
+    control[num_controls].size = g_array_get_element_size (self->slice_params)
         * self->num_slices;
-    count++;
+    num_controls++;
 
-    control[count].id = V4L2_CID_STATELESS_H264_PRED_WEIGHTS;
-    control[count].ptr = &self->pred_weight;
-    control[count].size = sizeof (self->pred_weight);
-    count++;
+    control[num_controls].id = V4L2_CID_STATELESS_H264_PRED_WEIGHTS;
+    control[num_controls].ptr = &self->pred_weight;
+    control[num_controls].size = sizeof (self->pred_weight);
+    num_controls++;
   }
 
-  if (!gst_v4l2_decoder_set_controls (self->decoder, request, control, count)) {
+  if (num_controls > G_N_ELEMENTS (control))
+    g_error ("Set too many controls, increase control[] size");
+
+  if (!gst_v4l2_decoder_set_controls (self->decoder, request, control,
+          num_controls)) {
     GST_ELEMENT_ERROR (self, RESOURCE, WRITE,
         ("Driver did not accept the bitstream parameters."), (NULL));
     goto done;
@@ -1286,6 +1318,16 @@ gst_v4l2_codec_h264_dec_decode_slice (GstH264Decoder * decoder,
       slice->nalu.size);
   self->bitstream_map.size += nal_size;
 
+  switch (slice->header.type % 5) {
+    case GST_H264_P_SLICE:
+      self->decode_params.flags |= V4L2_H264_DECODE_PARAM_FLAG_PFRAME;
+      break;
+
+    case GST_H264_B_SLICE:
+      self->decode_params.flags |= V4L2_H264_DECODE_PARAM_FLAG_BFRAME;
+      break;
+  }
+
   return GST_FLOW_OK;
 }
 
@@ -1308,11 +1350,10 @@ gst_v4l2_codec_h264_dec_end_picture (GstH264Decoder * decoder,
 
 static GstFlowReturn
 gst_v4l2_codec_h264_dec_new_field_picture (GstH264Decoder * decoder,
-    const GstH264Picture * first_field, GstH264Picture * second_field)
+    GstH264Picture * first_field, GstH264Picture * second_field)
 {
   GstV4l2CodecH264Dec *self = GST_V4L2_CODEC_H264_DEC (decoder);
-  GstV4l2Request *request =
-      gst_h264_picture_get_user_data ((GstH264Picture *) first_field);
+  GstV4l2Request *request = gst_h264_picture_get_user_data (first_field);
 
   if (!request) {
     GST_WARNING_OBJECT (self,
@@ -1320,7 +1361,8 @@ gst_v4l2_codec_h264_dec_new_field_picture (GstH264Decoder * decoder,
     return GST_FLOW_OK;
   }
 
-  GST_DEBUG_OBJECT (self, "Assigned request %p to second field.", request);
+  GST_DEBUG_OBJECT (self, "Assigned request %i to second field.",
+      gst_v4l2_request_get_fd (request));
 
   /* Associate the previous request with the new picture so that
    * submit_bitstream can create sub-request */
@@ -1441,6 +1483,7 @@ gst_v4l2_codec_h264_dec_subinit (GstV4l2CodecH264Dec * self,
   gst_video_info_init (&self->vinfo);
   self->slice_params = g_array_sized_new (FALSE, TRUE,
       sizeof (struct v4l2_ctrl_h264_slice_params), 4);
+  g_array_set_size (self->slice_params, 4);
 }
 
 static void
