@@ -34,7 +34,7 @@ GST_DEBUG_CATEGORY_STATIC (v4l2_vp9dec_debug);
 #define GST_CAT_DEFAULT v4l2_vp9dec_debug
 
 /* Used to mark picture that have been outputed */
-#define FLAG_PICTURE_OUTPUTED GST_MINI_OBJECT_FLAG_LAST
+#define FLAG_PICTURE_HOLDS_BUFFER GST_MINI_OBJECT_FLAG_LAST
 
 enum
 {
@@ -73,7 +73,7 @@ struct _GstV4l2CodecVp9Dec
   GstV4l2CodecAllocator *src_allocator;
   GstV4l2CodecPool *src_pool;
   gboolean has_videometa;
-  gboolean need_negotiation;
+  gboolean streaming;
   gboolean copy_frames;
 
   struct v4l2_ctrl_vp9_frame v4l2_vp9_frame;
@@ -308,7 +308,6 @@ gst_v4l2_codec_vp9_dec_fill_dec_params (GstV4l2CodecVp9Dec * self,
     .compressed_header_size = h->header_size_in_bytes,
     .uncompressed_header_size = h->frame_header_length_in_bytes,
     .profile = h->profile,
-    .reset_frame_context = h->reset_frame_context,
     .frame_context_idx = h->frame_context_idx,
     .bit_depth = self->bit_depth,
     .interpolation_filter = h->interpolation_filter,
@@ -350,12 +349,26 @@ gst_v4l2_codec_vp9_dec_fill_dec_params (GstV4l2CodecVp9Dec * self,
   };
   /* *INDENT-ON* */
 
+  switch (h->reset_frame_context) {
+    case 0:
+    case 1:
+      self->v4l2_vp9_frame.reset_frame_context = V4L2_VP9_RESET_FRAME_CTX_NONE;
+      break;
+    case 2:
+      self->v4l2_vp9_frame.reset_frame_context = V4L2_VP9_RESET_FRAME_CTX_SPEC;
+      break;
+    case 3:
+      self->v4l2_vp9_frame.reset_frame_context = V4L2_VP9_RESET_FRAME_CTX_ALL;
+      break;
+    default:
+      break;
+  }
+
   gst_v4l2_codecs_vp9_dec_fill_refs (self, h, reference_frames);
   gst_v4l2_codec_vp9_dec_fill_lf_params (self, &h->loop_filter_params);
   gst_v4l2_codec_vp9_dec_fill_seg_params (self, &h->segmentation_params);
 }
 
-
 static gboolean
 gst_v4l2_codec_vp9_dec_open (GstVideoDecoder * decoder)
 {
@@ -369,6 +382,10 @@ gst_v4l2_codec_vp9_dec_open (GstVideoDecoder * decoder)
     return FALSE;
   }
 
+  vp9dec->parse_compressed_headers =
+      gst_v4l2_decoder_query_control_size (self->decoder,
+      V4L2_CID_STATELESS_VP9_COMPRESSED_HDR, NULL);
+
   /* V4L2 does not support non-keyframe resolution change, this will ask the
    * base class to drop frame until the next keyframe as a workaround. */
   gst_vp9_decoder_set_non_keyframe_format_change_support (vp9dec, FALSE);
@@ -384,6 +401,16 @@ gst_v4l2_codec_vp9_dec_close (GstVideoDecoder * decoder)
   return TRUE;
 }
 
+static void
+gst_v4l2_codec_vp9_dec_streamoff (GstV4l2CodecVp9Dec * self)
+{
+  if (self->streaming) {
+    gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SINK);
+    gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SRC);
+    self->streaming = FALSE;
+  }
+}
+
 static void
 gst_v4l2_codec_vp9_dec_reset_allocation (GstV4l2CodecVp9Dec * self)
 {
@@ -434,15 +461,11 @@ gst_v4l2_codec_vp9_dec_negotiate (GstVideoDecoder * decoder)
 
   GstCaps *filter, *caps;
   /* Ignore downstream renegotiation request. */
-  if (!self->need_negotiation)
-    return TRUE;
-  self->need_negotiation = FALSE;
+  if (self->streaming)
+    goto done;
 
   GST_DEBUG_OBJECT (self, "Negotiate");
 
-  gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SINK);
-  gst_v4l2_decoder_streamoff (self->decoder, GST_PAD_SRC);
-
   gst_v4l2_codec_vp9_dec_reset_allocation (self);
 
   if (!gst_v4l2_decoder_set_sink_fmt (self->decoder, V4L2_PIX_FMT_VP9_FRAME,
@@ -482,6 +505,7 @@ gst_v4l2_codec_vp9_dec_negotiate (GstVideoDecoder * decoder)
   }
   gst_caps_unref (caps);
 
+done:
   if (self->output_state)
     gst_video_codec_state_unref (self->output_state);
 
@@ -493,6 +517,9 @@ gst_v4l2_codec_vp9_dec_negotiate (GstVideoDecoder * decoder)
   self->output_state->caps = gst_video_info_to_caps (&self->output_state->info);
 
   if (GST_VIDEO_DECODER_CLASS (parent_class)->negotiate (decoder)) {
+    if (self->streaming)
+      return TRUE;
+
     if (!gst_v4l2_decoder_streamon (self->decoder, GST_PAD_SINK)) {
       GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
           ("Could not enable the decoder driver."),
@@ -507,6 +534,8 @@ gst_v4l2_codec_vp9_dec_negotiate (GstVideoDecoder * decoder)
       return FALSE;
     }
 
+    self->streaming = TRUE;
+
     return TRUE;
   }
 
@@ -563,7 +592,7 @@ gst_v4l2_codec_vp9_dec_decide_allocation (GstVideoDecoder * decoder,
 
 static GstFlowReturn
 gst_v4l2_codec_vp9_dec_new_sequence (GstVp9Decoder * decoder,
-    const GstVp9FrameHeader * frame_hdr)
+    const GstVp9FrameHeader * frame_hdr, gint max_dpb_size)
 {
   GstV4l2CodecVp9Dec *self = GST_V4L2_CODEC_VP9_DEC (decoder);
   gboolean negotiation_needed = FALSE;
@@ -623,10 +652,12 @@ gst_v4l2_codec_vp9_dec_new_sequence (GstVp9Decoder * decoder,
   }
 
   gst_v4l2_codec_vp9_dec_fill_dec_params (self, frame_hdr, NULL);
-  gst_v4l2_codec_vp9_dec_fill_prob_updates (self, frame_hdr);
+
+  if (decoder->parse_compressed_headers)
+    gst_v4l2_codec_vp9_dec_fill_prob_updates (self, frame_hdr);
 
   if (negotiation_needed) {
-    self->need_negotiation = TRUE;
+    gst_v4l2_codec_vp9_dec_streamoff (self);
     if (!gst_video_decoder_negotiate (GST_VIDEO_DECODER (self))) {
       GST_ERROR_OBJECT (self, "Failed to negotiate with downstream");
       return GST_FLOW_ERROR;
@@ -717,7 +748,9 @@ gst_v4l2_codec_vp9_dec_decode_picture (GstVp9Decoder * decoder,
   }
 
   gst_v4l2_codec_vp9_dec_fill_dec_params (self, &picture->frame_hdr, dpb);
-  gst_v4l2_codec_vp9_dec_fill_prob_updates (self, &picture->frame_hdr);
+
+  if (decoder->parse_compressed_headers)
+    gst_v4l2_codec_vp9_dec_fill_prob_updates (self, &picture->frame_hdr);
 
   memcpy (bitstream_data, picture->data, picture->size);
   self->bitstream_map.size = picture->size;
@@ -734,6 +767,7 @@ gst_v4l2_codec_vp9_dec_end_picture (GstVp9Decoder * decoder,
   GstV4l2Request *request = NULL;
   GstFlowReturn flow_ret;
   gsize bytesused;
+  guint num_controls = 1;
 
   /* *INDENT-OFF* */
   struct v4l2_ext_control decode_params_control[] = {
@@ -743,11 +777,17 @@ gst_v4l2_codec_vp9_dec_end_picture (GstVp9Decoder * decoder,
       .size = sizeof(self->v4l2_vp9_frame),
     },
     {
+      /* V4L2_CID_STATELESS_VP9_COMPRESSED_HDR */
+    },
+  };
+
+  if (decoder->parse_compressed_headers) {
+    decode_params_control[num_controls++] = (struct v4l2_ext_control) {
       .id = V4L2_CID_STATELESS_VP9_COMPRESSED_HDR,
       .ptr = &self->v4l2_delta_probs,
       .size = sizeof (self->v4l2_delta_probs),
-    },
-  };
+    };
+  }
   /* *INDENT-ON* */
 
   bytesused = self->bitstream_map.size;
@@ -782,7 +822,7 @@ gst_v4l2_codec_vp9_dec_end_picture (GstVp9Decoder * decoder,
   }
 
   if (!gst_v4l2_decoder_set_controls (self->decoder, request,
-          decode_params_control, G_N_ELEMENTS (decode_params_control))) {
+          decode_params_control, num_controls)) {
     GST_ELEMENT_ERROR (decoder, RESOURCE, WRITE,
         ("Driver did not accept the bitstream parameters."), (NULL));
     goto fail;
@@ -869,10 +909,22 @@ gst_v4l2_codec_vp9_dec_duplicate_picture (GstVp9Decoder * decoder,
   new_picture->frame_hdr = picture->frame_hdr;
   new_picture->system_frame_number = frame->system_frame_number;
 
-  if (GST_MINI_OBJECT_FLAG_IS_SET (picture, FLAG_PICTURE_OUTPUTED)) {
+  if (GST_MINI_OBJECT_FLAG_IS_SET (picture, FLAG_PICTURE_HOLDS_BUFFER)) {
     GstBuffer *output_buffer = gst_vp9_picture_get_user_data (picture);
-    if (output_buffer)
+
+    if (output_buffer) {
       frame->output_buffer = gst_buffer_ref (output_buffer);
+
+      /* We need to also hold on the picture so it stays alive, but also to
+       * ensure we can duplicate it too. */
+      gst_vp9_picture_set_user_data (new_picture,
+          gst_buffer_ref (frame->output_buffer),
+          (GDestroyNotify) gst_buffer_unref);
+    }
+
+    /* Flag regardless if the buffer is null, so we don't start thinking it
+     * should hold a request unconditionally. */
+    GST_MINI_OBJECT_FLAG_SET (new_picture, FLAG_PICTURE_HOLDS_BUFFER);
   } else {
     GstV4l2Request *request = gst_vp9_picture_get_user_data (picture);
     gst_vp9_picture_set_user_data (new_picture, gst_v4l2_request_ref (request),
@@ -889,11 +941,21 @@ gst_v4l2_codec_vp9_dec_output_picture (GstVp9Decoder * decoder,
 {
   GstV4l2CodecVp9Dec *self = GST_V4L2_CODEC_VP9_DEC (decoder);
   GstVideoDecoder *vdec = GST_VIDEO_DECODER (decoder);
-  GstV4l2Request *request = gst_vp9_picture_get_user_data (picture);
+  GstV4l2Request *request = NULL;
   gint ret;
 
+  if (picture->discont_state) {
+    if (!gst_video_decoder_negotiate (vdec)) {
+      GST_ERROR_OBJECT (vdec, "Could not re-negotiate with updated state");
+      return FALSE;
+    }
+  }
+
   GST_DEBUG_OBJECT (self, "Output picture %u", picture->system_frame_number);
 
+  if (!GST_MINI_OBJECT_FLAG_IS_SET (picture, FLAG_PICTURE_HOLDS_BUFFER))
+    request = gst_vp9_picture_get_user_data (picture);
+
   if (request) {
     ret = gst_v4l2_request_set_done (request);
     if (ret == 0) {
@@ -918,7 +980,7 @@ gst_v4l2_codec_vp9_dec_output_picture (GstVp9Decoder * decoder,
         gst_buffer_ref (frame->output_buffer),
         (GDestroyNotify) gst_buffer_unref);
 
-    GST_MINI_OBJECT_FLAG_SET (picture, FLAG_PICTURE_OUTPUTED);
+    GST_MINI_OBJECT_FLAG_SET (picture, FLAG_PICTURE_HOLDS_BUFFER);
   }
 
   /* This may happen if we duplicate a picture witch failed to decode */
@@ -1025,8 +1087,6 @@ gst_v4l2_codec_vp9_dec_get_property (GObject * object, guint prop_id,
 static void
 gst_v4l2_codec_vp9_dec_init (GstV4l2CodecVp9Dec * self)
 {
-  GstVp9Decoder *parent = GST_VP9_DECODER (self);
-  parent->parse_compressed_headers = TRUE;
 }
 
 static void
